@@ -27,33 +27,32 @@
 #include <dolfin.h>
 #include "AdvectionDiffusion.h"
 #include "Velocity.h"
-#include <dolfin/io/XMLFile.h>
 
 using namespace dolfin;
 
 int main(int argc, char *argv[])
 {
   // Read mesh
-  Mesh mesh("../dolfin_fine.xml.gz");
+  auto mesh = std::make_shared<Mesh>("../dolfin_fine.xml.gz");
 
   // Create velocity FunctionSpace
-  Velocity::FunctionSpace V_u(mesh);
+  auto V_u = std::make_shared<Velocity::FunctionSpace>(mesh);
 
   // Create velocity function
-  Function velocity(V_u);
-  XMLFile file_u(mesh.mpi_comm(), "../dolfin_fine_velocity.xml.gz");
-  file_u >> velocity;
+  auto velocity = std::make_shared<Function>(V_u);
+  File file_u(mesh->mpi_comm(), "../dolfin_fine_velocity.xml.gz");
+  file_u >> *velocity;
 
   // Read sub domain markers
-  MeshFunction<std::size_t> sub_domains(mesh,
+  auto sub_domains = std::make_shared<MeshFunction<std::size_t>>(mesh,
                                         "../dolfin_fine_subdomains.xml.gz");
 
   // Create function space
-  AdvectionDiffusion::FunctionSpace V(mesh);
+  auto V = std::make_shared<AdvectionDiffusion::FunctionSpace>(mesh);
 
   // Source term and initial condition
-  Constant f(0.0);
-  Function u(V);
+  auto f = std::make_shared<Constant>(0.0);
+  auto u = std::make_shared<Function>(V);
 
   // Set up forms
   AdvectionDiffusion::BilinearForm a(V, V);
@@ -62,12 +61,9 @@ int main(int argc, char *argv[])
   L.u0 = u; L.b = velocity; L.f = f;
 
   // Set up boundary condition
-  Constant g(1.0);
+  auto g = std::make_shared<Constant>(1.0);
   DirichletBC bc(V, g, sub_domains, 1);
 
-  // Solution
-  Function u1(V);
-
   // Linear system
   std::shared_ptr<Matrix> A(new Matrix);
   Vector b;
@@ -97,17 +93,17 @@ int main(int argc, char *argv[])
     bc.apply(b);
 
     // Solve the linear system (re-use the already factorized matrix A)
-    lu.solve(*u.vector(), b);
+    lu.solve(*u->vector(), b);
 
     // Save solution in VTK format
-    file << std::pair<const Function*, double>(&u, t);
+    file << std::pair<const Function*, double>(u.get(), t);
 
     // Move to next interval
-    p = t / T;
+    p = t/T;
     t += k;
   }
 
   // Plot solution
-  plot(u);
+  plot(*u);
   interactive();
 }
