@@ -38,6 +38,9 @@ using a discontinuous Galerkin formulation (interior penalty method).
 
 from dolfin import *
 
+# FIXME: Make mesh ghosted
+parameters["ghost_mode"] = "shared_facet"
+
 # Define class marking Dirichlet boundary (x = 0 or x = 1)
 class DirichletBoundary(SubDomain):
   def inside(self, x, on_boundary):
@@ -62,9 +65,9 @@ h = CellSize(mesh)
 h_avg = (h('+') + h('-'))/2
 
 # Define the source term f, Dirichlet term u0 and Neumann term g
-f = Expression('-100.0*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)')
-u0 = Expression('x[0] + 0.25*sin(2*pi*x[1])')
-g = Expression('(x[1] - 0.5)*(x[1] - 0.5)')
+f = Expression('-100.0*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)', degree=2)
+u0 = Expression('x[0] + 0.25*sin(2*pi*x[1])', degree=2)
+g = Expression('(x[1] - 0.5)*(x[1] - 0.5)', degree=2)
 
 # Mark facets of the mesh
 boundaries = FacetFunction('size_t', mesh, 0)
@@ -72,7 +75,7 @@ NeumanBoundary().mark(boundaries, 2)
 DirichletBoundary().mark(boundaries, 1)
 
 # Define outer surface measure aware of Dirichlet and Neumann boundaries
-ds = Measure('ds')[boundaries]
+ds = Measure('ds', domain=mesh, subdomain_data=boundaries)
 
 # Define parameters
 alpha = 4.0
@@ -91,13 +94,11 @@ L = v*f*dx - u0*dot(grad(v), n)*ds(1) + (gamma/h)*u0*v*ds(1) + g*v*ds(2)
 # Compute solution
 u = Function(V)
 solve(a == L, u)
-
-# Project solution to piecewise linears
-u_proj = project(u)
+print("Solution vector norm (0): {!r}".format(u.vector().norm("l2")))
 
 # Save solution to file
-file = File("poisson.pvd")
-file << u_proj
+#file = File("poisson.pvd")
+#file << u
 
 # Plot solution
-plot(u_proj, interactive=True)
+#plot(u, interactive=True)
