@@ -26,42 +26,42 @@ using namespace dolfin;
 
 // Function to compute the near nullspace for elasticity - it is made
 // up of the six rigid body modes
-dolfin::VectorSpaceBasis build_nullspace(const dolfin::FunctionSpace& V,
-                                         const GenericVector& x)
+dolfin::VectorSpaceBasis build_near_nullspace(const dolfin::FunctionSpace& V,
+                                              const GenericVector& x)
 {
-  dolfin_assert(V.mesh());
-  const dolfin::Mesh& mesh = *(V.mesh());
-
   // Get subspaces
-  dolfin::SubSpace V0(V, 0);
-  dolfin::SubSpace V1(V, 1);
-  dolfin::SubSpace V2(V, 2);
+  auto V0 = V.sub(0);
+  auto V1 = V.sub(1);
+  auto V2 = V.sub(2);
 
   // Create vectors for nullspace basis
-  std::vector<std::shared_ptr<dolfin::GenericVector> > basis(6);
+  std::vector<std::shared_ptr<dolfin::GenericVector>> basis(6);
   for (std::size_t i = 0; i < basis.size(); ++i)
     basis[i] = x.copy();
 
   // x0, x1, x2 translations
-  V0.dofmap()->set(*basis[0], 1.0);
-  V1.dofmap()->set(*basis[1], 1.0);
-  V2.dofmap()->set(*basis[2], 1.0);
+  V0->dofmap()->set(*basis[0], 1.0);
+  V1->dofmap()->set(*basis[1], 1.0);
+  V2->dofmap()->set(*basis[2], 1.0);
 
   // Rotations
-  V0.dofmap()->set_x(*basis[3], -1.0, 1, mesh);
-  V1.dofmap()->set_x(*basis[3],  1.0, 0, mesh);
+  V0->set_x(*basis[3], -1.0, 1);
+  V1->set_x(*basis[3],  1.0, 0);
 
-  V0.dofmap()->set_x(*basis[4],  1.0, 2, mesh);
-  V2.dofmap()->set_x(*basis[4], -1.0, 0, mesh);
+  V0->set_x(*basis[4],  1.0, 2);
+  V2->set_x(*basis[4], -1.0, 0);
 
-  V2.dofmap()->set_x(*basis[5],  1.0, 1, mesh);
-  V1.dofmap()->set_x(*basis[5], -1.0, 2, mesh);
+  V2->set_x(*basis[5],  1.0, 1);
+  V1->set_x(*basis[5], -1.0, 2);
 
   // Apply
   for (std::size_t i = 0; i < basis.size(); ++i)
     basis[i]->apply("add");
 
-  return dolfin::VectorSpaceBasis(basis);
+  // Create vector space and orthonormalize
+  VectorSpaceBasis vector_space(basis);
+  vector_space.orthonormalize();
+  return vector_space;
 }
 
 
@@ -99,19 +99,19 @@ int main()
   };
 
   // Read mesh
-  Mesh mesh("../pulley.xml.gz");
+  auto mesh = std::make_shared<Mesh>("../pulley.xml.gz");
 
   // Create right-hand side loading function
-  CentripetalLoading f;
+  auto f = std::make_shared<CentripetalLoading>();
 
   // Set elasticity parameters
   double E  = 10.0;
   double nu = 0.3;
-  Constant mu(E/(2.0*(1.0 + nu)));
-  Constant lambda(E*nu/((1.0 + nu)*(1.0 - 2.0*nu)));
+  auto mu = std::make_shared<Constant>(E/(2.0*(1.0 + nu)));
+  auto lambda = std::make_shared<Constant>(E*nu/((1.0 + nu)*(1.0 - 2.0*nu)));
 
   // Create function space
-  Elasticity::Form_a::TestSpace V(mesh);
+  auto V = std::make_shared<Elasticity::Form_a::TestSpace>(mesh);
 
   // Define variational problem
   Elasticity::Form_a a(V, V);
@@ -120,69 +120,81 @@ int main()
   L.f = f;
 
   // Set up boundary condition on inner surface
-  InnerSurface inner_surface;
-  Constant zero(0.0, 0.0, 0.0);
-  DirichletBC bc(V, zero, inner_surface);
+  auto inner_surface = std::make_shared<InnerSurface>();
+  auto zero = std::make_shared<Constant>(0.0, 0.0, 0.0);
+  auto bc = std::make_shared<DirichletBC>(V, zero, inner_surface);
 
   // Assemble system, applying boundary conditions and preserving
   // symmetry)
   PETScMatrix A;
   PETScVector b;
-  assemble_system(A, b, a, L, bc);
+  assemble_system(A, b, a, L, {bc});
 
   // Create solution function
-  Function u(V);
+  auto u = std::make_shared<Function>(V);
 
   // Create near null space basis (required for smoothed aggregation
   // AMG). The solution vector is passed so that it can be copied to
   // generate compatible vectors for the nullspace.
-  VectorSpaceBasis null_space = build_nullspace(V, *u.vector());
+  VectorSpaceBasis near_null_space = build_near_nullspace(*V, *u->vector());
+
+  // Attach near nullspace to matrix
+  A.set_near_nullspace(near_null_space);
 
   // Create PETSc smoothed aggregation AMG preconditioner
-  PETScPreconditioner pc("petsc_amg");
-  pc.parameters["report"] = true;
-  pc.set_nullspace(null_space);
+  auto pc = std::make_shared<PETScPreconditioner>("petsc_amg");
+
+  // Use Chebyshev smoothing for multigrid
+  PETScOptions::set("mg_levels_ksp_type", "chebyshev");
+  PETScOptions::set("mg_levels_pc_type", "jacobi");
+
+  // Improve estimate of eigenvalues for Chebyshev smoothing
+  PETScOptions::set("mg_levels_esteig_ksp_type", "cg");
+  PETScOptions::set("mg_levels_ksp_chebyshev_esteig_steps", 50);
 
   // Create CG PETSc linear solver and turn on convergence monitor
   PETScKrylovSolver solver("cg", pc);
   solver.parameters["monitor_convergence"] = true;
 
   // Solve
-  solver.solve(A, *(u.vector()), b);
+  solver.solve(A, *(u->vector()), b);
 
   // Extract solution components (deep copy)
-  Function ux = u[0];
-  Function uy = u[1];
-  Function uz = u[2];
-  std::cout << "Norm (u): " << u.vector()->norm("l2") << std::endl;
+  Function ux = (*u)[0];
+  Function uy = (*u)[1];
+  Function uz = (*u)[2];
+  std::cout << "Norm (u vector): " << u->vector()->norm("l2") << std::endl;
   std::cout << "Norm (ux, uy, uz): " << ux.vector()->norm("l2") << "  "
             << uy.vector()->norm("l2") << "  "
             << uz.vector()->norm("l2") << std::endl;
 
   // Save solution in VTK format
   File vtk_file("elasticity.pvd", "compressed");
-  vtk_file << u;
+  vtk_file << *u;
 
   // Extract stress and write in VTK format
-  Elasticity::Form_a_s::TestSpace W(mesh);
+  auto W = std::make_shared<Elasticity::Form_a_s::TestSpace>(mesh);
   Elasticity::Form_a_s a_s(W, W);
   Elasticity::Form_L_s L_s(W);
   L_s.mu = mu;
   L_s.lmbda = lambda;
   L_s.disp = u;
 
+  // Compute stress for visualisation
   Function stress(W);
-  LocalSolver local_solver;
-  local_solver.solve(*stress.vector(), a_s, L_s);
+  LocalSolver local_solver(std::shared_ptr<Form>(&a_s, NoDeleter()),
+                           std::shared_ptr<Form>(&L_s, NoDeleter()),
+                           LocalSolver::SolverType::Cholesky);
+  local_solver.solve_local_rhs(stress);
 
   File file_stress("stress.pvd");
   file_stress << stress;
 
   // Save colored mesh paritions in VTK format if running in parallel
-  if (dolfin::MPI::size(mesh.mpi_comm()) > 1)
+  if (dolfin::MPI::size(mesh->mpi_comm()) > 1)
   {
     CellFunction<std::size_t>
-      partitions(mesh, dolfin::MPI::rank(mesh.mpi_comm()));
+      partitions(mesh, dolfin::MPI::rank(mesh->mpi_comm()));
     File file("partitions.pvd");
     file << partitions;
   }
@@ -191,8 +203,8 @@ int main()
   plot(u, "Displacement", "displacement");
 
   // Displace mesh and plot displaced mesh
-  mesh.move(u);
-  plot(mesh, "Deformed mesh");
+  ALE::move(*mesh, *u);
+  plot(*mesh, "Deformed mesh");
 
   // Make plot windows interactive
   interactive();
