@@ -33,8 +33,8 @@ print('Maximal radius ratio:', qmax)
 
 # Show histogram using matplotlib
 hist = MeshQuality.radius_ratio_matplotlib_histogram(mesh)
-hist = hist.replace('import pylab', 'import matplotlib\n    matplotlib.use(\'Agg\')\n    import pylab')
-hist = hist.replace('pylab.show()', 'pylab.savefig("mesh-quality.pdf")')
+hist = hist.replace('    import matplotlib.pylab', '    import matplotlib\n    matplotlib.use(\'Agg\')\n    import matplotlib.pylab\n')
+hist = hist.replace('matplotlib.pylab.show()', 'matplotlib.pylab.savefig("mesh-quality.pdf")')
 print(hist)
 exec(hist)
 
