@@ -25,11 +25,11 @@ using namespace dolfin;
 int main()
 {
   // Create mesh
-  UnitSquareMesh mesh(20, 20);
+  auto mesh = std::make_shared<UnitSquareMesh>(20, 20);
 
   // Create MeshFunction to hold cell process rank
   CellFunction<std::size_t>
-    processes0(mesh, dolfin::MPI::rank(mesh.mpi_comm()));
+    processes0(mesh, dolfin::MPI::rank(mesh->mpi_comm()));
 
   // Output cell distribution to VTK file
   File file("processes.pvd");
@@ -37,22 +37,22 @@ int main()
 
   // Mark all cells on process 0 for refinement
   const CellFunction<bool>
-    marker(mesh, (dolfin::MPI::rank(mesh.mpi_comm()) == 0));
+    marker(mesh, (dolfin::MPI::rank(mesh->mpi_comm()) == 0));
 
   // Refine mesh, but keep all new cells on parent process
-  Mesh mesh0 = refine(mesh, marker, false);
+  auto mesh0 = std::make_shared<Mesh>(refine(*mesh, marker, false));
 
   // Create MeshFunction to hold cell process rank
   const CellFunction<std::size_t>
-    processes1(mesh0, dolfin::MPI::rank(mesh.mpi_comm()));
+    processes1(mesh0, dolfin::MPI::rank(mesh->mpi_comm()));
   file << processes1;
 
   // Refine mesh, but this time repartition the mesh after refinement
-  Mesh mesh1 = refine(mesh, marker, true);
+  auto mesh1 = std::make_shared<Mesh>(refine(*mesh, marker, false));
 
   // Create MeshFunction to hold cell process rank
   CellFunction<std::size_t>
-    processes2(mesh1, dolfin::MPI::rank(mesh.mpi_comm()));
+    processes2(mesh1, dolfin::MPI::rank(mesh->mpi_comm()));
   file << processes2;
 
   return 0;
