@@ -31,7 +31,7 @@ from dolfin import *
 #--- Demo of global DOLFIN parameters ---
 
 # Set some global DOLFIN parameters
-parameters["linear_algebra_backend"] = "uBLAS"
+parameters["linear_algebra_backend"] = "Eigen"
 
 # Print global DOLFIN parameters
 info(parameters, True)
@@ -70,10 +70,6 @@ solver_parameters.set_range("max_iterations", 0, 1000)
 solver_parameters["max_iterations"] = 500
 solver_parameters["relative_tolerance"] = 0.1
 
-# Can also access parameter values like this
-solver_parameters.max_iterations = 500
-solver_parameters.relative_tolerance = 0.1
-
 # Set solver parameters as nested parameters of application parameters
 application_parameters.add(solver_parameters)
 
@@ -85,11 +81,6 @@ foo = application_parameters["foo"]
 bar = application_parameters["bar"]
 tol = application_parameters["solver_parameters"]["tolerance"]
 
-# Can also access parameter values like this
-foo = application_parameters.foo
-bar = application_parameters.bar
-tol = application_parameters.solver_parameters.tolerance
-
 # Print parameter values
 print("foo =", foo)
 print("bar =", bar)
