@@ -23,7 +23,7 @@
 #ifndef __NO_DELETER_H
 #define __NO_DELETER_H
 
-#include <boost/shared_ptr.hpp>
+#include <memory>
 
 namespace dolfin
 {
@@ -33,15 +33,15 @@ namespace dolfin
   class NoDeleter
   {
   public:
-    void operator() (const void *p) {}
+    void operator() (const void *) {}
   };
 
   /// Helper function to construct shared pointer with NoDeleter with cleaner syntax
 
   template <typename T>
-  boost::shared_ptr<T> reference_to_no_delete_pointer(T& r)
+  std::shared_ptr<T> reference_to_no_delete_pointer(T& r)
   {
-    return boost::shared_ptr<T>(&r, NoDeleter());
+    return std::shared_ptr<T>(&r, NoDeleter());
   }
 
 }
