@@ -48,6 +48,9 @@ namespace dolfin
     /// Destructor
     virtual ~Variable();
 
+    /// Assignment operator
+    const Variable& operator=(const Variable& variable);
+
     /// Rename variable
     void rename(const std::string name, const std::string label);
 
