@@ -21,6 +21,10 @@
 // First added:  2011-10-15
 // Last changed: 2014-08-11
 
+#ifdef HAS_PETSC
+#include <petscversion.h>
+#endif
+
 #include "types.h"
 #include "defines.h"
 
@@ -30,6 +34,11 @@ std::string dolfin::dolfin_version()
   return std::string(DOLFIN_VERSION);
 }
 //-------------------------------------------------------------------------
+std::string dolfin::ufc_signature()
+{
+  return std::string(UFC_SIGNATURE);
+}
+//-------------------------------------------------------------------------
 std::string dolfin::git_commit_hash()
 {
   return std::string(DOLFIN_GIT_COMMIT_HASH);
@@ -67,23 +76,6 @@ bool dolfin::has_petsc()
 #endif
 }
 //-------------------------------------------------------------------------
-bool dolfin::has_petsc_snes()
-{
-#ifdef ENABLE_PETSC_SNES
-  return true;
-#else
-  return false;
-#endif
-}
-//-------------------------------------------------------------------------
-bool dolfin::has_petsc_tao()
-{
-#ifdef ENABLE_PETSC_TAO
-  return true;
-#else
-  return false;
-#endif
-}//-------------------------------------------------------------------------
 bool dolfin::has_slepc()
 {
 #ifdef HAS_SLEPC
@@ -147,9 +139,9 @@ bool dolfin::has_hdf5()
 #endif
 }
 //-------------------------------------------------------------------------
-bool dolfin::has_exodus()
+bool dolfin::has_vtk()
 {
-#ifdef HAS_VTK_EXODUS
+#ifdef HAS_VTK
   return true;
 #else
   return false;
