@@ -29,53 +29,19 @@
 #include "Assembler.h"
 #include "Form.h"
 #include "NonlinearVariationalProblem.h"
+#include "SystemAssembler.h"
 #include "NonlinearVariationalSolver.h"
 
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-NonlinearVariationalSolver::
-NonlinearVariationalSolver(NonlinearVariationalProblem& problem)
-  : _problem(reference_to_no_delete_pointer(problem))
-{
-  // Set parameters
-  parameters = default_parameters();
-}
-//-----------------------------------------------------------------------------
-NonlinearVariationalSolver::
-NonlinearVariationalSolver(std::shared_ptr<NonlinearVariationalProblem> problem)
+NonlinearVariationalSolver::NonlinearVariationalSolver(std::shared_ptr<NonlinearVariationalProblem> problem)
   : _problem(problem)
 {
   // Set parameters
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-std::pair<std::size_t, bool> NonlinearVariationalSolver::solve(const Function& lb,
-                                                               const Function& ub)
-{
-  return solve(lb.vector(), ub.vector());
-}
-//-----------------------------------------------------------------------------
-std::pair<std::size_t, bool> NonlinearVariationalSolver::solve(std::shared_ptr<const Function> lb,
-                                       std::shared_ptr<const Function> ub)
-{
-  return solve(*lb,*ub);
-}
-//-----------------------------------------------------------------------------
-std::pair<std::size_t, bool> NonlinearVariationalSolver::solve(const GenericVector& lb,
-                                                               const GenericVector& ub)
-{
-  return solve(reference_to_no_delete_pointer(lb),reference_to_no_delete_pointer(ub));
-}
-//-----------------------------------------------------------------------------
-std::pair<std::size_t, bool> NonlinearVariationalSolver::solve(std::shared_ptr<const GenericVector> lb,
-                                       std::shared_ptr<const GenericVector> ub)
-{
-  // Set bounds and solve
-  this->_problem->set_bounds(lb,ub);
-  return solve();
-}
-//-----------------------------------------------------------------------------
 std::pair<std::size_t, bool> NonlinearVariationalSolver::solve()
 {
   begin("Solving nonlinear variational problem.");
@@ -97,23 +63,16 @@ std::pair<std::size_t, bool> NonlinearVariationalSolver::solve()
                  "Needs PETSc to solve bound constrained problems");
   }
 #endif
+
   // Get problem data
   dolfin_assert(_problem);
-  std::shared_ptr<Function> u(_problem->solution());
+  auto u = _problem->solution();
 
-  // 'reset_jacobian' option is deprecated
-  if (parameters["reset_jacobian"].change_count() > 1)
-  {
-    deprecation("reset_jacobian parameter in NonlinearVariationalSolver",
-                "1.4.0", "1.5",
-                "reset_jacobian no longer has any effect.");
-  }
-
-  // Create nonlinear problem
+  // Create discrete nonlinear problem
   if (!nonlinear_problem)
   {
-    nonlinear_problem = std::shared_ptr<NonlinearDiscreteProblem>(new NonlinearDiscreteProblem(_problem,
-                                             reference_to_no_delete_pointer(*this)));
+    nonlinear_problem = std::make_shared<NonlinearDiscreteProblem>(_problem,
+                                                    reference_to_no_delete_pointer(*this));
   }
 
   std::pair<std::size_t, bool> ret;
@@ -125,6 +84,7 @@ std::pair<std::size_t, bool> NonlinearVariationalSolver::solve()
                    "solve nonlinear variational problem",
                    "Set the \"nonlinear_solver\" parameter to \"snes\" or remove bounds");
     }
+
     // Create Newton solver and set parameters
     if (!newton_solver)
       newton_solver = std::shared_ptr<NewtonSolver>(new NewtonSolver());
@@ -137,14 +97,14 @@ std::pair<std::size_t, bool> NonlinearVariationalSolver::solve()
     dolfin_assert(nonlinear_problem);
     ret = newton_solver->solve(*nonlinear_problem, *u->vector());
   }
-  #ifdef ENABLE_PETSC_SNES
+  #ifdef HAS_PETSC
   else if (std::string(parameters["nonlinear_solver"]) == "snes")
   {
     // Create SNES solver and set parameters
     if (!snes_solver)
     {
       // Create Newton solver and set parameters
-      snes_solver = std::shared_ptr<PETScSNESSolver>(new PETScSNESSolver());
+      snes_solver = std::make_shared<PETScSNESSolver>();
     }
     snes_solver->parameters.update(parameters("snes_solver"));
 
@@ -174,17 +134,15 @@ std::pair<std::size_t, bool> NonlinearVariationalSolver::solve()
 //-----------------------------------------------------------------------------
 // Implementation of NonlinearDiscreteProblem
 //-----------------------------------------------------------------------------
-NonlinearVariationalSolver::
-NonlinearDiscreteProblem::
-NonlinearDiscreteProblem(std::shared_ptr<NonlinearVariationalProblem> problem,
-                         std::shared_ptr<NonlinearVariationalSolver> solver)
+NonlinearVariationalSolver::NonlinearDiscreteProblem::
+NonlinearDiscreteProblem(std::shared_ptr<const NonlinearVariationalProblem> problem,
+                         std::shared_ptr<const NonlinearVariationalSolver> solver)
   : _problem(problem), _solver(solver)
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-NonlinearVariationalSolver::
-NonlinearDiscreteProblem::~NonlinearDiscreteProblem()
+NonlinearVariationalSolver::NonlinearDiscreteProblem::~NonlinearDiscreteProblem()
 {
   // Do nothing
 }
@@ -194,20 +152,17 @@ NonlinearDiscreteProblem::F(GenericVector& b, const GenericVector& x)
 {
   // Get problem data
   dolfin_assert(_problem);
-  std::shared_ptr<const Form> F(_problem->residual_form());
-  std::vector<std::shared_ptr<const DirichletBC> > bcs(_problem->bcs());
+  std::shared_ptr<const Form> F = _problem->residual_form();
+  std::shared_ptr<const Form> J = _problem->jacobian_form();
+  std::vector<std::shared_ptr<const DirichletBC>> bcs(_problem->bcs());
+
+  // Create assembler
+  dolfin_assert(J);
+  dolfin_assert(F);
+  SystemAssembler assembler(J, F, bcs);
 
   // Assemble right-hand side
-  dolfin_assert(F);
-  Assembler assembler;
-  assembler.assemble(b, *F);
-
-  // Apply boundary conditions
-  for (std::size_t i = 0; i < bcs.size(); i++)
-  {
-    dolfin_assert(bcs[i]);
-    bcs[i]->apply(b, x);
-  }
+  assembler.assemble(b, x);
 
   // Print vector
   dolfin_assert(_solver);
@@ -216,25 +171,23 @@ NonlinearDiscreteProblem::F(GenericVector& b, const GenericVector& x)
     info(b, true);
 }
 //-----------------------------------------------------------------------------
-void NonlinearVariationalSolver::NonlinearDiscreteProblem::J(GenericMatrix& A,
+void
+NonlinearVariationalSolver::NonlinearDiscreteProblem::J(GenericMatrix& A,
                                                         const GenericVector& x)
 {
   // Get problem data
   dolfin_assert(_problem);
-  std::shared_ptr<const Form> J(_problem->jacobian_form());
-  std::vector<std::shared_ptr<const DirichletBC> > bcs(_problem->bcs());
+  std::shared_ptr<const Form> J = _problem->jacobian_form();
+  std::shared_ptr<const Form> F = _problem->residual_form();
+  std::vector<std::shared_ptr<const DirichletBC>> bcs(_problem->bcs());
+
+  // Create assembler
+  dolfin_assert(J);
+  dolfin_assert(F);
+  SystemAssembler assembler(J, F, bcs);
 
   // Assemble left-hand side
-  dolfin_assert(J);
-  Assembler assembler;
-  assembler.assemble(A, *J);
-
-  // Apply boundary conditions
-  for (std::size_t i = 0; i < bcs.size(); i++)
-  {
-    dolfin_assert(bcs[i]);
-    bcs[i]->apply(A);
-  }
+  assembler.assemble(A);
 
   // Print matrix
   dolfin_assert(_solver);
