@@ -14,11 +14,11 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// First added:  2011-04-13
-// Last changed: 2013-06-23
 
-#include <boost/shared_ptr.hpp>
+#ifndef __POINT_SOURCE_H
+#define __POINT_SOURCE_H
+
+#include <memory>
 #include <dolfin/geometry/Point.h>
 
 namespace dolfin
@@ -39,13 +39,7 @@ namespace dolfin
   public:
 
     /// Create point source at given point of given magnitude
-    PointSource(const FunctionSpace& V,
-                const Point& p,
-                double magnitude=1.0);
-
-    /// Create point source at given point of given magnitude
-    PointSource(boost::shared_ptr<const FunctionSpace> V,
-                const Point& p,
+    PointSource(std::shared_ptr<const FunctionSpace> V, const Point& p,
                 double magnitude=1.0);
 
     /// Destructor
@@ -60,7 +54,7 @@ namespace dolfin
     void check_is_scalar(const FunctionSpace& V);
 
     // The function space
-    boost::shared_ptr<const FunctionSpace> _V;
+    std::shared_ptr<const FunctionSpace> _function_space;
 
     // The point
     Point _p;
@@ -71,3 +65,5 @@ namespace dolfin
   };
 
 }
+
+#endif
