@@ -143,7 +143,7 @@ namespace dolfin
     virtual void restrict(double* w,
                           const FiniteElement& element,
                           const Cell& dolfin_cell,
-                          const double* vertex_coordinates,
+                          const double* coordinate_dofs,
                           const ufc::cell& ufc_cell) const;
 
     /// Compute values at all mesh vertices.
@@ -156,6 +156,14 @@ namespace dolfin
     virtual void compute_vertex_values(std::vector<double>& vertex_values,
                                        const Mesh& mesh) const;
 
+    /// Return shared pointer to function space (NULL)
+    /// Expression does not have a FunctionSpace
+    ///
+    /// *Returns*
+    ///     _FunctionSpace_
+    ///         Return the shared pointer.
+    virtual std::shared_ptr<const FunctionSpace> function_space() const;
+
   protected:
 
     // Value shape
