@@ -28,6 +28,7 @@
 #include <dolfin/common/Array.h>
 #include <dolfin/common/Timer.h>
 #include <dolfin/common/utils.h>
+#include <dolfin/common/NoDeleter.h>
 #include <dolfin/fem/FiniteElement.h>
 #include <dolfin/fem/GenericDofMap.h>
 #include <dolfin/fem/DirichletBC.h>
@@ -217,34 +218,38 @@ Function& Function::operator[] (std::size_t i) const
   }
 }
 //-----------------------------------------------------------------------------
-FunctionAXPY Function::operator+(const Function& other) const
+FunctionAXPY Function::operator+(std::shared_ptr<const Function> other) const
 {
-  return FunctionAXPY(*this, other, FunctionAXPY::Direction::ADD_ADD);
+  return FunctionAXPY(reference_to_no_delete_pointer(*this), other,
+                      FunctionAXPY::Direction::ADD_ADD);
 }
 //-----------------------------------------------------------------------------
 FunctionAXPY Function::operator+(const FunctionAXPY& axpy) const
 {
-  return FunctionAXPY(axpy, *this, FunctionAXPY::Direction::ADD_ADD);
+  return FunctionAXPY(axpy, reference_to_no_delete_pointer(*this),
+                      FunctionAXPY::Direction::ADD_ADD);
 }
 //-----------------------------------------------------------------------------
-FunctionAXPY Function::operator-(const Function& other) const
+FunctionAXPY Function::operator-(std::shared_ptr<const Function> other) const
 {
-  return FunctionAXPY(*this, other, FunctionAXPY::Direction::ADD_SUB);
+  return FunctionAXPY(reference_to_no_delete_pointer(*this), other,
+                      FunctionAXPY::Direction::ADD_SUB);
 }
 //-----------------------------------------------------------------------------
 FunctionAXPY Function::operator-(const FunctionAXPY& axpy) const
 {
-  return FunctionAXPY(axpy, *this, FunctionAXPY::Direction::SUB_ADD);
+  return FunctionAXPY(axpy, reference_to_no_delete_pointer(*this),
+                      FunctionAXPY::Direction::SUB_ADD);
 }
 //-----------------------------------------------------------------------------
 FunctionAXPY Function::operator*(double scalar) const
 {
-  return FunctionAXPY(*this, scalar);
+  return FunctionAXPY(reference_to_no_delete_pointer(*this), scalar);
 }
 //-----------------------------------------------------------------------------
 FunctionAXPY Function::operator/(double scalar) const
 {
-  return FunctionAXPY(*this, 1.0/scalar);
+  return FunctionAXPY(reference_to_no_delete_pointer(*this), 1.0/scalar);
 }
 //-----------------------------------------------------------------------------
 void Function::operator=(const FunctionAXPY& axpy)
@@ -257,14 +262,20 @@ void Function::operator=(const FunctionAXPY& axpy)
   }
 
   // Make an initial assign and scale
+  dolfin_assert(axpy.pairs()[0].second);
   *this = *(axpy.pairs()[0].second);
   if (axpy.pairs()[0].first != 1.0)
     *_vector *= axpy.pairs()[0].first;
 
   // Start from item 2 and axpy
-  std::vector<std::pair<double, const Function*>>::const_iterator it;
+  std::vector<std::pair<double, std::shared_ptr<const Function>>>
+    ::const_iterator it;
   for (it = axpy.pairs().begin()+1; it != axpy.pairs().end(); it++)
+  {
+    dolfin_assert(it->second);
+    dolfin_assert(it->second->vector());
     _vector->axpy(it->first, *(it->second->vector()));
+  }
 }
 //-----------------------------------------------------------------------------
 std::shared_ptr<GenericVector> Function::vector()
