@@ -17,11 +17,8 @@
 //
 // Modified by Garth N. Wells, 2005-2010.
 // Modified by Kristian B. Oelgaard, 2007.
-// Modified by Martin Sandve Alnes, 2008.
+// Modified by Martin Sandve Alnes, 2008-2014.
 // Modified by Andre Massing, 2009.
-//
-// First added:  2003-11-28
-// Last changed: 2013-10-22
 
 #ifndef __FUNCTION_H
 #define __FUNCTION_H
@@ -67,19 +64,6 @@ namespace dolfin
   {
   public:
 
-    /// Create function on given function space
-    ///
-    /// *Arguments*
-    ///     V (_FunctionSpace_)
-    ///         The function space.
-    ///
-    /// *Example*
-    ///     .. code-block:: c++
-    ///
-    ///         Function u(V);
-    ///
-    explicit Function(const FunctionSpace& V);
-
     /// Create function on given function space (shared data)
     ///
     /// *Arguments*
@@ -100,17 +84,6 @@ namespace dolfin
     Function(std::shared_ptr<const FunctionSpace> V,
              std::shared_ptr<GenericVector> x);
 
-    /// Create function from vector of dofs stored to file
-    ///
-    /// *Arguments*
-    ///     V (_FunctionSpace_)
-    ///         The function space.
-    ///     filename_vector (std::string)
-    ///         The name of the file containing the vector.
-    ///     filename_dofdata (std::string)
-    ///         The name of the file containing the dofmap data.
-    Function(const FunctionSpace& V, std::string filename);
-
     /// Create function from vector of dofs stored to file (shared data)
     ///
     /// *Arguments*
@@ -178,7 +151,7 @@ namespace dolfin
     /// *Returns*
     ///     _FunctionAXPY_
     ///         Return a linear combination of Functions
-    FunctionAXPY operator+(const Function& other) const;
+    FunctionAXPY operator+(std::shared_ptr<const Function> other) const;
 
     /// Add operator with other linear combination of functions
     ///
@@ -187,14 +160,14 @@ namespace dolfin
     ///         Return a linear combination of Functions
     FunctionAXPY operator+(const FunctionAXPY& axpy) const;
 
-    /// Substraction operator with other function
+    /// Subtraction operator with other function
     ///
     /// *Returns*
     ///     _FunctionAXPY_
     ///         Return a linear combination of Functions
-    FunctionAXPY operator-(const Function& other) const;
+    FunctionAXPY operator-(std::shared_ptr<const Function> other) const;
 
-    /// Substraction operator with other linear combination of functions
+    /// Subtraction operator with other linear combination of functions
     ///
     /// *Returns*
     ///     _FunctionAXPY_
@@ -220,7 +193,11 @@ namespace dolfin
     /// *Returns*
     ///     _FunctionSpace_
     ///         Return the shared pointer.
-    std::shared_ptr<const FunctionSpace> function_space() const;
+    virtual std::shared_ptr<const FunctionSpace> function_space() const
+    {
+      dolfin_assert(_function_space);
+      return _function_space;
+    }
 
     /// Return vector of expansion coefficients (non-const version)
     ///
@@ -274,10 +251,8 @@ namespace dolfin
     ///         The cell.
     ///     ufc_cell (ufc::cell)
     ///         The ufc::cell.
-    void eval(Array<double>& values,
-              const Array<double>& x,
-              const Cell& dolfin_cell,
-              const ufc::cell& ufc_cell) const;
+    void eval(Array<double>& values, const Array<double>& x,
+              const Cell& dolfin_cell, const ufc::cell& ufc_cell) const;
 
     /// Interpolate function (on possibly non-matching meshes)
     ///
@@ -325,18 +300,6 @@ namespace dolfin
     virtual void eval(Array<double>& values, const Array<double>& x,
                       const ufc::cell& cell) const;
 
-    /// Evaluate function for given data (non-matching meshes)
-    ///
-    /// *Arguments*
-    ///     values (_Array_ <double>)
-    ///         The values at the point.
-    ///     x (_Array_ <double>)
-    ///         The coordinates of the point.
-    ///     cell (ufc::cell)
-    ///         The cell.
-    void non_matching_eval(Array<double>& values, const Array<double>& x,
-                           const ufc::cell& ufc_cell) const;
-
     /// Restrict function to local cell (compute expansion coefficients w)
     ///
     /// *Arguments*
@@ -351,7 +314,7 @@ namespace dolfin
     virtual void restrict(double* w,
                           const FiniteElement& element,
                           const Cell& dolfin_cell,
-                          const double* vertex_coordinates,
+                          const double* coordinate_dofs,
                           const ufc::cell& ufc_cell) const;
 
     /// Compute values at all mesh vertices
@@ -371,8 +334,21 @@ namespace dolfin
     ///         The values at all vertices.
     void compute_vertex_values(std::vector<double>& vertex_values);
 
-    /// Update off-process ghost coefficients
-    virtual void update() const;
+    /// Allow extrapolation when evaluating the Function
+    ///
+    /// *Arguments*
+    ///     allow_extrapolation (bool)
+    ///         Whether or not permit extrapolation.
+    void set_allow_extrapolation(bool allow_extrapolation)
+    { _allow_extrapolation = allow_extrapolation; }
+
+    /// Check if extrapolation is permitted when evaluating the Function
+    ///
+    /// *Returns*
+    ///     bool
+    ///         True if extrapolation is permitted, otherwise false
+    bool get_allow_extrapolation() const
+    { return _allow_extrapolation; }
 
   private:
 
@@ -381,18 +357,11 @@ namespace dolfin
     friend class FunctionAssigner;
 
     // Collection of sub-functions which share data with the function
-    mutable boost::ptr_map<std::size_t, Function> sub_functions;
-
-    // Compute lists of off-process dofs
-    void compute_off_process_dofs() const;
+    mutable boost::ptr_map<std::size_t, Function> _sub_functions;
 
     // Initialize vector
     void init_vector();
 
-    // Get coefficients from the vector(s)
-    void compute_ghost_indices(std::pair<std::size_t, std::size_t> range,
-                               std::vector<la_index>& ghost_indices) const;
-
     // The function space
     std::shared_ptr<const FunctionSpace> _function_space;
 
@@ -400,7 +369,7 @@ namespace dolfin
     std::shared_ptr<GenericVector> _vector;
 
     // True if extrapolation should be allowed
-    bool allow_extrapolation;
+    bool _allow_extrapolation;
 
   };
 
