@@ -34,7 +34,7 @@ namespace dolfin
 
     /// NB: this code is experimental, just for testing, and will generally not
     /// work with anything else
-    UnitHexMesh(std::size_t nx, std::size_t ny, std::size_t nz);
+    UnitHexMesh(MPI_Comm comm, std::size_t nx, std::size_t ny, std::size_t nz);
 
   };
 
