@@ -31,11 +31,10 @@
 #include <dolfin/common/Timer.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/mesh/LocalMeshData.h>
+#include "CSRGraph.h"
 #include "GraphBuilder.h"
 #include "SCOTCH.h"
 
-#include <dolfin/common/dolfin_common.h>
-
 #ifdef HAS_SCOTCH
 extern "C"
 {
@@ -49,29 +48,36 @@ using namespace dolfin;
 #ifdef HAS_SCOTCH
 
 //-----------------------------------------------------------------------------
-void SCOTCH::compute_partition(
-  const MPI_Comm mpi_comm,
-  std::vector<std::size_t>& cell_partition,
-  std::map<std::size_t, dolfin::Set<unsigned int>>& ghost_procs,
-  const LocalMeshData& mesh_data)
+void SCOTCH::compute_partition(const MPI_Comm mpi_comm,
+                               std::vector<int>& cell_partition,
+                               std::map<std::int64_t, std::vector<int>>& ghost_procs,
+                               const boost::multi_array<std::int64_t, 2>& cell_vertices,
+                               const std::vector<std::size_t>& cell_weight,
+                               const std::int64_t num_global_vertices,
+                               const std::int64_t num_global_cells,
+                               const CellType& cell_type)
 {
   // Create data structures to hold graph
-  std::vector<std::set<std::size_t>> local_graph;
-  std::set<std::size_t> ghost_vertices;
+  CSRGraph<SCOTCH_Num> csr_graph;
+  std::set<std::int64_t> ghost_vertices;
 
-  // Compute local dual graph
-  GraphBuilder::compute_dual_graph(mpi_comm, mesh_data, local_graph,
-                                   ghost_vertices);
+  // Build dual graph. Use scoping to clean up memory
+  {
+    // Compute dual graph (for this parition)
+    std::vector<std::vector<std::size_t>> local_graph;
+    GraphBuilder::compute_dual_graph(mpi_comm, cell_vertices, cell_type,
+                                     num_global_vertices, local_graph,
+                                     ghost_vertices);
+
+    csr_graph = CSRGraph<SCOTCH_Num>(MPI_COMM_SELF, local_graph);
+  }
 
   // Compute partitions
-  partition(mpi_comm, local_graph, mesh_data.cell_weight,
-            ghost_vertices, mesh_data.global_cell_indices,
-            mesh_data.num_global_cells, cell_partition, ghost_procs);
-
+  partition(mpi_comm, csr_graph, cell_weight, ghost_vertices,
+            num_global_cells, cell_partition, ghost_procs);
 }
 //-----------------------------------------------------------------------------
-std::vector<int> SCOTCH::compute_gps(const Graph& graph,
-                                     std::size_t num_passes)
+std::vector<int> SCOTCH::compute_gps(const Graph& graph, std::size_t num_passes)
 {
   // Create strategy string for Gibbs-Poole-Stockmeyer ordering
   std::string strategy = "g{pass= " + std::to_string(num_passes) + "}";
@@ -92,7 +98,7 @@ void SCOTCH::compute_reordering(const Graph& graph,
                                 std::vector<int>& inverse_permutation,
                                 std::string scotch_strategy)
 {
-  Timer timer("SCOTCH graph ordering");
+  Timer timer("Compute SCOTCH graph re-ordering");
 
   // Number of local graph vertices (cells)
   const SCOTCH_Num vertnbr = graph.size();
@@ -117,6 +123,10 @@ void SCOTCH::compute_reordering(const Graph& graph,
     edgetab.insert(edgetab.end(), vertex->begin(), vertex->end());
   }
 
+  // Shrink vectors to hopefully recover an unused memory
+  verttab.shrink_to_fit();
+  edgetab.shrink_to_fit();
+
   // Create SCOTCH graph
   SCOTCH_Graph scotch_graph;
 
@@ -132,6 +142,7 @@ void SCOTCH::compute_reordering(const Graph& graph,
   }
 
   // Build SCOTCH graph
+  Timer timer1("SCOTCH: call SCOTCH_graphBuild");
   if (SCOTCH_graphBuild(&scotch_graph, baseval,
                         vertnbr, &verttab[0], &verttab[1], NULL, NULL,
                         edgenbr, &edgetab[0], NULL))
@@ -140,6 +151,7 @@ void SCOTCH::compute_reordering(const Graph& graph,
                  "partition mesh using SCOTCH",
                  "Error building SCOTCH graph");
   }
+  timer1.stop();
 
   // Check graph data for consistency
   /*
@@ -158,8 +170,6 @@ void SCOTCH::compute_reordering(const Graph& graph,
   SCOTCH_stratInit(&strat);
 
   // Set SCOTCH strategy (if provided)
-  //SCOTCH_stratGraphOrderBuild(&strat, SCOTCH_STRATQUALITY, 0, 0);
-  //SCOTCH_stratGraphOrderBuild(&strat, SCOTCH_STRATSPEED, 0, 0);
   if (!scotch_strategy.empty())
     SCOTCH_stratGraphOrder(&strat, scotch_strategy.c_str());
 
@@ -172,6 +182,7 @@ void SCOTCH::compute_reordering(const Graph& graph,
   SCOTCH_randomReset();
 
   // Compute re-ordering
+  Timer timer2("SCOTCH: call SCOTCH_graphOrder");
   if (SCOTCH_graphOrder(&scotch_graph, &strat, permutation_indices.data(),
                         inverse_permutation_indices.data(), NULL, NULL, NULL))
   {
@@ -179,6 +190,7 @@ void SCOTCH::compute_reordering(const Graph& graph,
                  "re-order graph using SCOTCH",
                  "Error during re-ordering");
   }
+  timer2.stop();
 
   // Clean up SCOTCH objects
   SCOTCH_graphExit(&scotch_graph);
@@ -193,17 +205,18 @@ void SCOTCH::compute_reordering(const Graph& graph,
             inverse_permutation_indices.end(), inverse_permutation.begin());
 }
 //-----------------------------------------------------------------------------
+template<typename T>
 void SCOTCH::partition(
   const MPI_Comm mpi_comm,
-  const std::vector<std::set<std::size_t>>& local_graph,
+  CSRGraph<T>& local_graph,
   const std::vector<std::size_t>& node_weights,
-  const std::set<std::size_t>& ghost_vertices,
-  const std::vector<std::size_t>& global_cell_indices,
+  const std::set<std::int64_t>& ghost_vertices,
   const std::size_t num_global_vertices,
-  std::vector<std::size_t>& cell_partition,
-  std::map<std::size_t, dolfin::Set<unsigned int>>& ghost_procs)
+  std::vector<int>& cell_partition,
+  std::map<std::int64_t, std::vector<int>>& ghost_procs)
 {
-  Timer timer("Partition graph (calling SCOTCH)");
+  log(PROGRESS, "Compute graph partition using PT-SCOTCH");
+  Timer timer("Compute graph partition (SCOTCH)");
 
   // C-style array indexing
   const SCOTCH_Num baseval = 0;
@@ -220,28 +233,9 @@ void SCOTCH::partition(
   const SCOTCH_Num vertlocnbr = local_graph.size();
   const std::size_t vertgstnbr = vertlocnbr + ghost_vertices.size();
 
-  // Data structures for graph input to SCOTCH (add 1 for case that
-  // local graph size is zero)
-  std::vector<SCOTCH_Num> vertloctab;
-  vertloctab.reserve(local_graph.size() + 1);
-  std::vector<SCOTCH_Num> edgeloctab;
-
-  // Build local graph input for SCOTCH
-  // (number of local + ghost graph vertices (cells),
-  // number of local edges + edges connecting to ghost vertices)
-  SCOTCH_Num edgelocnbr = 0;
-  vertloctab.push_back((SCOTCH_Num) 0);
-  std::vector<std::set<std::size_t>>::const_iterator vertex;
-  for(vertex = local_graph.begin(); vertex != local_graph.end(); ++vertex)
-  {
-    edgelocnbr += vertex->size();
-    vertloctab.push_back(vertloctab.back() + vertex->size());
-    edgeloctab.insert(edgeloctab.end(), vertex->begin(), vertex->end());
-  }
-
-  // Handle case that local graph size is zero
-  if (edgeloctab.empty())
-    edgeloctab.resize(1);
+  // Get graph data
+  std::vector<SCOTCH_Num>& edgeloctab = local_graph.edges();
+  std::vector<SCOTCH_Num>& vertloctab = local_graph.nodes();
 
   // Global data ---------------------------------
 
@@ -276,27 +270,24 @@ void SCOTCH::partition(
                  "Error initializing SCOTCH graph");
   }
 
-  SCOTCH_Num* veloloctab;
+
+  // Handle cell weights (if any)
   std::vector<SCOTCH_Num> vload;
-  if (node_weights.size() == 0)
-    veloloctab = NULL;
-  else
-  {
-    vload.resize(node_weights.size());
-    std::copy(node_weights.begin(), node_weights.end(), vload.begin());
-    veloloctab = vload.data();
-  }
+  if (!node_weights.empty())
+    vload.assign(node_weights.begin(), node_weights.end());
 
   // Build SCOTCH distributed graph
+  Timer timer1("SCOTCH: call SCOTCH_dgraphBuild");
   if (SCOTCH_dgraphBuild(&dgrafdat, baseval, vertlocnbr, vertlocnbr,
-                              &vertloctab[0], NULL, veloloctab, NULL,
-                              edgelocnbr, edgelocnbr,
-                              &edgeloctab[0], NULL, NULL) )
+                         vertloctab.data(), NULL, vload.data(), NULL,
+                         edgeloctab.size(), edgeloctab.size(),
+                         edgeloctab.data(), NULL, NULL) )
   {
     dolfin_error("SCOTCH.cpp",
                  "partition mesh using SCOTCH",
                  "Error building SCOTCH graph");
   }
+  timer1.stop();
 
   // Check graph data for consistency
   #ifdef DEBUG
@@ -311,14 +302,15 @@ void SCOTCH::partition(
   // Number of partitions (set equal to number of processes)
   const SCOTCH_Num npart = num_processes;
 
-  // Partitioning strategy
+  // Initialise partitioning strategy
   SCOTCH_Strat strat;
   SCOTCH_stratInit(&strat);
 
-  // Set strategy (SCOTCH uses very cryptic strings for this, and they
-  // can change between versions)
-  //std::string strategy = "b{sep=m{asc=b{bnd=q{strat=f},org=q{strat=f}},low=q{strat=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}},seq=q{strat=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}}},seq=b{job=t,map=t,poli=S,sep=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}}}";
-  //SCOTCH_stratDgraphMap (&strat, strategy.c_str());
+  // Set SCOTCH strategy
+  //SCOTCH_stratDgraphMapBuild(&strat, SCOTCH_STRATDEFAULT, npart, npart, 0.05);
+  SCOTCH_stratDgraphMapBuild(&strat, SCOTCH_STRATSPEED, npart, npart, 0.05);
+  //SCOTCH_stratDgraphMapBuild(&strat, SCOTCH_STRATQUALITY, npart, npart, 0.05);
+  //SCOTCH_stratDgraphMapBuild(&strat, SCOTCH_STRATSCALABILITY, npart, npart, 0.15);
 
   // Resize vector to hold cell partition indices with enough extra
   // space for ghost cell partition information too When there are no
@@ -332,12 +324,25 @@ void SCOTCH::partition(
   SCOTCH_randomReset();
 
   // Partition graph
+  Timer timer2("SCOTCH: call SCOTCH_dgraphPart");
   if (SCOTCH_dgraphPart(&dgrafdat, npart, &strat, _cell_partition.data()))
   {
     dolfin_error("SCOTCH.cpp",
                  "partition mesh using SCOTCH",
                  "Error during partitioning");
   }
+  timer2.stop();
+
+  /*
+  // Write SCOTCH strategy to file
+  if (dolfin::MPI::rank(MPI_COMM_WORLD) == 0)
+  {
+    FILE* fp;
+    fp = fopen("test.txt", "w");
+    SCOTCH_stratSave(&strat, fp);
+    fclose(fp);
+  }
+  */
 
   // Exchange halo with cell_partition data for ghosts
   // FIXME: check MPI type compatibility with SCOTCH_Num. Getting this
@@ -346,7 +351,7 @@ void SCOTCH::partition(
   MPI_Datatype MPI_SCOTCH_Num;
   if (sizeof(SCOTCH_Num) == 4)
     MPI_SCOTCH_Num = MPI_INT;
-  else if (sizeof(SCOTCH_Num)==8)
+  else if (sizeof(SCOTCH_Num) == 8)
     MPI_SCOTCH_Num = MPI_LONG_LONG_INT;
 
   // Double check size is correct
@@ -354,6 +359,7 @@ void SCOTCH::partition(
   MPI_Type_size(MPI_SCOTCH_Num, &tsize);
   dolfin_assert(tsize == sizeof(SCOTCH_Num));
 
+  Timer timer3("SCOTCH: call SCOTCH_dgraphHalo");
   if (SCOTCH_dgraphHalo(&dgrafdat, (void *)_cell_partition.data(),
                         MPI_SCOTCH_Num))
   {
@@ -361,16 +367,20 @@ void SCOTCH::partition(
                  "partition mesh using SCOTCH",
                  "Error during halo exchange");
   }
+  timer3.stop();
 
   // Get SCOTCH's locally indexed graph
+  Timer timer4("Get SCOTCH graph data");
   SCOTCH_Num* edge_ghost_tab;
   SCOTCH_dgraphData(&dgrafdat,
                     NULL, NULL, NULL, NULL, NULL, NULL,
                     NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                     &edge_ghost_tab, NULL, (MPI_Comm *)&mpi_comm);
+  timer4.stop();
 
   // Iterate through SCOTCH's local compact graph to find partition
   // boundaries and save to map
+  Timer timer5("Extract partition boundaries from SCOTCH graph");
   for(SCOTCH_Num i = 0; i < vertlocnbr; ++i)
   {
     const std::size_t proc_this =  _cell_partition[i];
@@ -382,18 +392,25 @@ void SCOTCH::partition(
         auto map_it = ghost_procs.find(i);
         if (map_it == ghost_procs.end())
         {
-          dolfin::Set<unsigned int> sharing_processes;
-          // Owning process goes first into dolfin::Set
-          // (unordered set) so will always be first.
-          sharing_processes.insert(proc_this);
-          sharing_processes.insert(proc_other);
+          std::vector<int> sharing_processes;
+
+          // Owning process always goes in first to vector
+          sharing_processes.push_back(proc_this);
+          sharing_processes.push_back(proc_other);
           ghost_procs.insert(std::make_pair(i, sharing_processes));
         }
         else
-          map_it->second.insert(proc_other);
+        {
+          // Add to vector if not already there
+          auto it = std::find(map_it->second.begin(), map_it->second.end(),
+                              proc_other);
+          if (it == map_it->second.end())
+            map_it->second.push_back(proc_other);
+        }
       }
     }
   }
+  timer5.stop();
 
   // Clean up SCOTCH objects
   SCOTCH_dgraphExit(&dgrafdat);
@@ -408,19 +425,21 @@ void SCOTCH::partition(
 //-----------------------------------------------------------------------------
 #else
 //-----------------------------------------------------------------------------
-void SCOTCH::compute_partition(
-  const MPI_Comm mpi_comm,
-  std::vector<std::size_t>& cell_partition,
-  std::map<std::size_t, dolfin::Set<unsigned int>>& ghost_procs,
-  const LocalMeshData& mesh_data)
+void SCOTCH::compute_partition(const MPI_Comm mpi_comm,
+                               std::vector<int>& cell_partition,
+                               std::map<std::int64_t, std::vector<int>>& ghost_procs,
+                               const boost::multi_array<std::int64_t, 2>& cell_vertices,
+                               const std::vector<std::size_t>& cell_weight,
+                               const std::int64_t num_global_vertices,
+                               const std::int64_t num_global_cells,
+                               const CellType& cell_type)
 {
   dolfin_error("SCOTCH.cpp",
                "partition mesh using SCOTCH",
                "DOLFIN has been configured without support for SCOTCH");
 }
 //-----------------------------------------------------------------------------
-std::vector<int> SCOTCH::compute_gps(const Graph& graph,
-                                     std::size_t num_passes)
+std::vector<int> SCOTCH::compute_gps(const Graph& graph, std::size_t num_passes)
 {
   dolfin_error("SCOTCH.cpp",
                "re-order graph using SCOTCH",
@@ -429,8 +448,7 @@ std::vector<int> SCOTCH::compute_gps(const Graph& graph,
 }
 //-----------------------------------------------------------------------------
 std::vector<int>
-SCOTCH::compute_reordering(const Graph& graph,
-                           std::string scotch_strategy)
+SCOTCH::compute_reordering(const Graph& graph, std::string scotch_strategy)
 {
   dolfin_error("SCOTCH.cpp",
                "re-order graph using SCOTCH",
@@ -449,14 +467,14 @@ void SCOTCH::compute_reordering(const Graph& graph,
                "DOLFIN has been configured without support for SCOTCH");
 }
 //-----------------------------------------------------------------------------
+template<typename T>
 void SCOTCH::partition(const MPI_Comm mpi_comm,
-                       const std::vector<std::set<std::size_t>>& local_graph,
+                       CSRGraph<T>& local_graph,
                        const std::vector<std::size_t>& node_weights,
-                       const std::set<std::size_t>& ghost_vertices,
-                       const std::vector<std::size_t>& global_cell_indices,
+                       const std::set<std::int64_t>& ghost_vertices,
                        const std::size_t num_global_vertices,
-                       std::vector<std::size_t>& cell_partition,
-                       std::map<std::size_t, dolfin::Set<unsigned int>>& ghost_procs)
+                       std::vector<int>& cell_partition,
+                       std::map<std::int64_t, std::vector<int>>& ghost_procs)
 {
   dolfin_error("SCOTCH.cpp",
                "partition mesh using SCOTCH",
