@@ -24,7 +24,7 @@
 
 #include <fstream>
 #include <dolfin/common/MPI.h>
-#include <dolfin/log/dolfin_log.h>
+#include <dolfin/log/log.h>
 #include "GenericFile.h"
 
 using namespace dolfin;
@@ -113,6 +113,11 @@ void GenericFile::operator>> (Parameters& parameters)
   read_not_impl("Parameters");
 }
 //-----------------------------------------------------------------------------
+void GenericFile::operator>> (Table& table)
+{
+  read_not_impl("Table");
+}
+//-----------------------------------------------------------------------------
 void GenericFile::operator>> (std::vector<int>& x)
 {
   read_not_impl("std::vector<int>");
@@ -143,19 +148,22 @@ void GenericFile::operator>> (std::map<std::size_t, double>& map)
   read_not_impl("std::map<std::size_t, double>");
 }
 //-----------------------------------------------------------------------------
-void GenericFile::operator>> (std::map<std::size_t, std::vector<int> >& array_map)
+void GenericFile::operator>> (std::map<std::size_t,
+                              std::vector<int>>& array_map)
 {
-  read_not_impl("std::map<std::size_t, std::vector<int> >");
+  read_not_impl("std::map<std::size_t, std::vector<int>>");
 }
 //-----------------------------------------------------------------------------
-void GenericFile::operator>> (std::map<std::size_t, std::vector<std::size_t> >& array_map)
+void GenericFile::operator>> (std::map<std::size_t,
+                              std::vector<std::size_t>>& array_map)
 {
-  read_not_impl("std::map<std::size_t, std::vector<std::size_t> >");
+  read_not_impl("std::map<std::size_t, std::vector<std::size_t>>");
 }
 //-----------------------------------------------------------------------------
-void GenericFile::operator>> (std::map<std::size_t, std::vector<double> >& array_map)
+void GenericFile::operator>> (std::map<std::size_t,
+                              std::vector<double>>& array_map)
 {
-  read_not_impl("std::map<std::size_t, std::vector<double> >");
+  read_not_impl("std::map<std::size_t, std::vector<double>>");
 }
 //-----------------------------------------------------------------------------
 void GenericFile::operator>> (Function& u)
@@ -267,6 +275,11 @@ void GenericFile::operator<< (const Parameters& parameters)
   write_not_impl("Parameters");
 }
 //-----------------------------------------------------------------------------
+void GenericFile::operator<< (const Table& table)
+{
+  write_not_impl("Table");
+}
+//-----------------------------------------------------------------------------
 void GenericFile::operator<< (const std::vector<int>& x)
 {
   read_not_impl("std::vector<int>");
@@ -298,29 +311,29 @@ void GenericFile::operator<< (const std::map<std::size_t, double>& map)
 }
 //-----------------------------------------------------------------------------
 void GenericFile::operator<< (const std::map<std::size_t,
-                              std::vector<int> >& array_map)
+                              std::vector<int>>& array_map)
 {
-  read_not_impl("std::map<std::size_t, std::vector<int> >");
+  read_not_impl("std::map<std::size_t, std::vector<int>>");
 }
 //-----------------------------------------------------------------------------
 void GenericFile::operator<< (const std::map<std::size_t,
-                              std::vector<std::size_t> >& array_map)
+                              std::vector<std::size_t>>& array_map)
 {
-  read_not_impl("std::map<std::size_t, std::vector<std::size_t> >");
+  read_not_impl("std::map<std::size_t, std::vector<std::size_t>>");
 }
 //-----------------------------------------------------------------------------
 void GenericFile::operator<< (const std::map<std::size_t,
-                              std::vector<double> >& array_map)
+                              std::vector<double>>& array_map)
 {
-  read_not_impl("std::map<std::size_t, std::vector<double> >");
+  read_not_impl("std::map<std::size_t, std::vector<double>>");
 }
 //-----------------------------------------------------------------------------
-void GenericFile::read()
+void GenericFile::_read()
 {
   opened_read = true;
 }
 //-----------------------------------------------------------------------------
-void GenericFile::write(std::size_t process_number)
+void GenericFile::_write(std::size_t process_number)
 {
   // pvd files should only be cleared by one process
   if (_filetype == "VTK" && process_number > 0)
