@@ -106,7 +106,7 @@ void RAWFile::ResultsWrite(const Function& u) const
   std::size_t cell_based_dim = 1;
   for (std::size_t i = 0; i < rank; i++)
     cell_based_dim *= mesh.topology().dim();
-  if (dofmap.max_cell_dimension() == cell_based_dim)
+  if (dofmap.max_element_dofs() == cell_based_dim)
     data_type = "cell";
 
   // Open file
@@ -180,8 +180,8 @@ void RAWFile::rawNameUpdate(const int counter)
   fileid.fill('0');
   fileid.width(6);
 
-  filestart.assign(_filename, 0, _filename.find("."));
-  extension.assign(_filename, _filename.find("."), _filename.size());
+  filestart.assign(_filename, 0, _filename.find_last_of("."));
+  extension.assign(_filename, _filename.find_last_of("."), _filename.size());
 
   fileid << counter;
   newfilename << filestart << fileid.str() << ".raw";
