@@ -1,4 +1,4 @@
-// Copyright (C) 2010 Garth N. Wells
+// Copyright (C) 2010-2016 Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -17,10 +17,8 @@
 //
 // Modified by Anders Logg 2011
 // Modified by Johannes Ring 2012
-//
-// First added:  2010-07-19
-// Last changed: 2012-09-14
 
+#include <cstdint>
 #include <fstream>
 #include <ostream>
 #include <sstream>
@@ -61,7 +59,8 @@ void VTKWriter::write_cell_data(const Function& u, std::string filename,
   dolfin_assert(u.function_space()->dofmap());
   const Mesh& mesh = *u.function_space()->mesh();
   const GenericDofMap& dofmap = *u.function_space()->dofmap();
-  const std::size_t num_cells = mesh.num_cells();
+  const std::size_t tdim = mesh.topology().dim();
+  const std::size_t num_cells = mesh.topology().ghost_offset(tdim);
 
   std::string encode_string;
   if (!binary)
@@ -89,8 +88,8 @@ void VTKWriter::write_cell_data(const Function& u, std::string filename,
   if (rank == 0)
   {
     fp << "<CellData  Scalars=\"" << u.name() << "\"> " << std::endl;
-    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name() << "\"  format=\""
-       << encode_string <<"\">";
+    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name()
+       << "\"  format=\"" << encode_string <<"\">";
   }
   else if (rank == 1)
   {
@@ -127,12 +126,14 @@ void VTKWriter::write_cell_data(const Function& u, std::string filename,
   for (CellIterator cell(mesh); !cell.end(); ++cell)
   {
     // Tabulate dofs
-    const std::vector<dolfin::la_index>& dofs = dofmap.cell_dofs(cell->index());
-    for(std::size_t i = 0; i < dofmap.cell_dimension(cell->index()); ++i)
+    const ArrayView<const dolfin::la_index>
+      dofs = dofmap.cell_dofs(cell->index());
+    for(std::size_t i = 0; i < dofmap.num_element_dofs(cell->index()); ++i)
       dof_set.push_back(dofs[i]);
 
     // Add local dimension to cell offset and increment
-    *(cell_offset + 1) = *(cell_offset) + dofmap.cell_dimension(cell->index());
+    *(cell_offset + 1)
+      = *(cell_offset) + dofmap.num_element_dofs(cell->index());
     ++cell_offset;
   }
 
@@ -231,7 +232,7 @@ std::string VTKWriter::base64_cell_data(const Mesh& mesh,
 void VTKWriter::write_ascii_mesh(const Mesh& mesh, std::size_t cell_dim,
                                  std::string filename)
 {
-  const std::size_t num_cells = mesh.topology().size(cell_dim);
+  const std::size_t num_cells = mesh.topology().ghost_offset(cell_dim);
   const std::size_t num_cell_vertices = mesh.type().num_vertices(cell_dim);
 
   // Get VTK cell type
@@ -262,10 +263,14 @@ void VTKWriter::write_ascii_mesh(const Mesh& mesh, std::size_t cell_dim,
   file << "<Cells>" << std::endl;
   file << "<DataArray  type=\"UInt32\"  Name=\"connectivity\"  format=\""
        << "ascii" << "\">";
+
+  std::unique_ptr<CellType>
+    celltype(CellType::create(mesh.type().entity_type(cell_dim)));
+  const std::vector<std::int8_t> perm = celltype->vtk_mapping();
   for (MeshEntityIterator c(mesh, cell_dim); !c.end(); ++c)
   {
-    for (VertexIterator v(*c); !v.end(); ++v)
-      file << v->index() << " ";
+    for (unsigned int i = 0; i != c->num_entities(0); ++i)
+      file << c->entities(0)[perm[i]] << " ";
     file << " ";
   }
   file << "</DataArray>" << std::endl;
@@ -296,7 +301,7 @@ void VTKWriter::write_base64_mesh(const Mesh& mesh, std::size_t cell_dim,
   const std::size_t num_cell_vertices = mesh.type().num_vertices(cell_dim);
 
   // Get VTK cell type
-  const boost::uint8_t _vtk_cell_type = vtk_cell_type(mesh, cell_dim);
+  const std::uint8_t _vtk_cell_type = vtk_cell_type(mesh, cell_dim);
 
   // Open file
   std::ofstream file(filename.c_str(), std::ios::app);
@@ -330,12 +335,16 @@ void VTKWriter::write_base64_mesh(const Mesh& mesh, std::size_t cell_dim,
   file << "<DataArray  type=\"UInt32\"  Name=\"connectivity\"  format=\""
        << "binary" << "\">" << std::endl;
   const int size = num_cells*num_cell_vertices;
-  std::vector<boost::uint32_t> cell_data(size);
-  std::vector<boost::uint32_t>::iterator cell_entry = cell_data.begin();
+  std::vector<std::uint32_t> cell_data(size);
+  std::vector<std::uint32_t>::iterator cell_entry = cell_data.begin();
+
+  std::unique_ptr<CellType>
+    celltype(CellType::create(mesh.type().entity_type(cell_dim)));
+  const std::vector<std::int8_t> perm = celltype->vtk_mapping();
   for (MeshEntityIterator c(mesh, cell_dim); !c.end(); ++c)
   {
-    for (VertexIterator v(*c); !v.end(); ++v)
-      *cell_entry++ = v->index();
+    for (unsigned int i = 0; i != c->num_entities(0); ++i)
+      *cell_entry++ = c->entities(0)[perm[i]];
   }
 
   // Create encoded stream
@@ -345,8 +354,8 @@ void VTKWriter::write_base64_mesh(const Mesh& mesh, std::size_t cell_dim,
   // Write offset into connectivity array for the end of each cell
   file << "<DataArray  type=\"UInt32\"  Name=\"offsets\"  format=\""
        << "binary" << "\">" << std::endl;
-  std::vector<boost::uint32_t> offset_data(num_cells*num_cell_vertices);
-  std::vector<boost::uint32_t>::iterator offset_entry = offset_data.begin();
+  std::vector<std::uint32_t> offset_data(num_cells*num_cell_vertices);
+  std::vector<std::uint32_t>::iterator offset_entry = offset_data.begin();
   for (std::size_t offsets = 1; offsets <= num_cells; offsets++)
     *offset_entry++ = offsets*num_cell_vertices;
 
@@ -357,8 +366,8 @@ void VTKWriter::write_base64_mesh(const Mesh& mesh, std::size_t cell_dim,
   // Write cell type
   file << "<DataArray  type=\"UInt8\"  Name=\"types\"  format=\"" << "binary"
        << "\">" << std::endl;
-  std::vector<boost::uint8_t> type_data(num_cells);
-  std::vector<boost::uint8_t>::iterator type_entry = type_data.begin();
+  std::vector<std::uint8_t> type_data(num_cells);
+  std::vector<std::uint8_t>::iterator type_entry = type_data.begin();
   for (std::size_t types = 0; types < num_cells; types++)
     *type_entry++ = _vtk_cell_type;
 
@@ -372,30 +381,20 @@ void VTKWriter::write_base64_mesh(const Mesh& mesh, std::size_t cell_dim,
   file.close();
 }
 //----------------------------------------------------------------------------
-boost::uint8_t VTKWriter::vtk_cell_type(const Mesh& mesh,
-                                        std::size_t cell_dim)
+std::uint8_t VTKWriter::vtk_cell_type(const Mesh& mesh,
+                                      std::size_t cell_dim)
 {
   // Get cell type
-  CellType::Type cell_type = mesh.type().cell_type();
-  if (mesh.topology().dim() == cell_dim)
-    cell_type = mesh.type().cell_type();
-  else if (mesh.topology().dim() - 1 == cell_dim)
-    cell_type = mesh.type().facet_type();
-  else if (cell_dim == 1)
-    cell_type = CellType::interval;
-  else if (cell_dim == 0)
-    cell_type = CellType::point;
-  else
-  {
-    dolfin_error("VTKWriter.cpp",
-                 "write data to VTK file",
-                 "Can only handle cells, cell facets or points with VTK output for now");
-  }
+  CellType::Type cell_type = mesh.type().entity_type(cell_dim);
 
   // Determine VTK cell type
-  boost::uint8_t vtk_cell_type = 0;
+  std::uint8_t vtk_cell_type = 0;
   if (cell_type == CellType::tetrahedron)
     vtk_cell_type = 10;
+  else if (cell_type == CellType::hexahedron)
+    vtk_cell_type = 12;
+  else if (cell_type == CellType::quadrilateral)
+    vtk_cell_type = 9;
   else if (cell_type == CellType::triangle)
     vtk_cell_type = 5;
   else if (cell_type == CellType::interval)
