@@ -18,16 +18,15 @@
 // Modified by Anders Logg 2011
 //
 // First added:  2002-12-06
-// Last changed: 2013-05-25
+// Last changed: 2014-02-06
 
 #include <map>
+#include <memory>
 #include <iomanip>
 #include <iostream>
+#include <string>
 #include <vector>
 #include <boost/format.hpp>
-#include <boost/lexical_cast.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/shared_ptr.hpp>
 
 #include "pugixml.hpp"
 
@@ -37,6 +36,7 @@
 #include "dolfin/la/GenericVector.h"
 #include "dolfin/mesh/Cell.h"
 #include "dolfin/mesh/CellType.h"
+#include "dolfin/mesh/LocalMeshData.h"
 #include "dolfin/mesh/Mesh.h"
 #include "dolfin/mesh/MeshData.h"
 #include "dolfin/mesh/MeshEditor.h"
@@ -92,7 +92,7 @@ void XMLMesh::read_mesh(Mesh& mesh, const pugi::xml_node mesh_node)
   const std::size_t gdim = mesh_node.attribute("dim").as_uint();
 
   // Get topological dimension
-  boost::scoped_ptr<CellType> cell_type(CellType::create(cell_type_str));
+  std::unique_ptr<CellType> cell_type(CellType::create(cell_type_str));
   const std::size_t tdim = cell_type->dim();
 
   // Create mesh for editing
@@ -105,7 +105,7 @@ void XMLMesh::read_mesh(Mesh& mesh, const pugi::xml_node mesh_node)
 
   // Get number of vertices and init editor
   const std::size_t num_vertices = xml_vertices.attribute("size").as_uint();
-  editor.init_vertices(num_vertices);
+  editor.init_vertices_global(num_vertices, num_vertices);
 
   // Iterate over vertices and add to mesh
   Point p;
@@ -125,13 +125,13 @@ void XMLMesh::read_mesh(Mesh& mesh, const pugi::xml_node mesh_node)
 
   // Get number of cells and init editor
   const std::size_t num_cells = xml_cells.attribute("size").as_uint();
-  editor.init_cells(num_cells);
+  editor.init_cells_global(num_cells, num_cells);
 
   // Create list of vertex index attribute names
   const unsigned int num_vertices_per_cell = cell_type->num_vertices(tdim);
   std::vector<std::string> v_str(num_vertices_per_cell);
   for (std::size_t i = 0; i < num_vertices_per_cell; ++i)
-    v_str[i] = "v" + boost::lexical_cast<std::string, unsigned int>(i);
+    v_str[i] = "v" + std::to_string(i);
 
   // Iterate over cells and add to mesh
   std::vector<std::size_t> v(num_vertices_per_cell);
@@ -167,7 +167,8 @@ void XMLMesh::read_data(MeshData& data, const Mesh& mesh,
     if (node_name == "mesh_function")
     {
       // Create MeshFunction to read data into
-      MeshFunction<std::size_t> mf(mesh);
+      auto _mesh = reference_to_no_delete_pointer(mesh);
+      MeshFunction<std::size_t> mf(_mesh);
 
       // Read  MeshFunction
       //const std::string data_type = it->attribute("type").value();
@@ -214,7 +215,8 @@ void XMLMesh::read_data(MeshData& data, const Mesh& mesh,
       else if (data_set_type == "mesh_function")
       {
         // Create MeshFunction to read data into
-        MeshFunction<std::size_t> mf(mesh);
+        auto _mesh = reference_to_no_delete_pointer(mesh);
+        MeshFunction<std::size_t> mf(_mesh);
 
         // Read  MeshFunction
         const std::string data_type = data_set.attribute("type").value();
@@ -285,7 +287,7 @@ void XMLMesh::read_domains(MeshDomains& domains, const Mesh& mesh,
     mesh.init(dim);
 
     // Read data into a mesh value collection
-    boost::shared_ptr<const Mesh> _mesh = reference_to_no_delete_pointer(mesh);
+    std::shared_ptr<const Mesh> _mesh = reference_to_no_delete_pointer(mesh);
     MeshValueCollection<std::size_t> mvc(_mesh);
     XMLMeshValueCollection::read(mvc, type, *it);
 
@@ -317,6 +319,73 @@ void XMLMesh::read_domains(MeshDomains& domains, const Mesh& mesh,
   }
 }
 //-----------------------------------------------------------------------------
+void XMLMesh::read_domain_data(LocalMeshData& mesh_data,
+                               const pugi::xml_node xml_dolfin)
+{
+  // Get mesh node
+  const pugi::xml_node mesh_node = xml_dolfin.child("mesh");
+  if (!mesh_node)
+  {
+    dolfin_error("XMLMesh.cpp",
+                 "read mesh from XML file",
+                 "Not a DOLFIN XML Mesh file");
+  }
+
+
+  // Check if we have any domains
+  const pugi::xml_node xml_domains = mesh_node.child("domains");
+  if (!xml_domains)
+  {
+    mesh_data.domain_data.clear();
+    return;
+  }
+
+  // Iterate over data
+  for (pugi::xml_node_iterator xml_domain = xml_domains.begin();
+       xml_domain != xml_domains.end(); ++xml_domain)
+  {
+    // Check that node is <mesh_value_collection>
+    const std::string node_name = xml_domain->name();
+    if (node_name != "mesh_value_collection")
+    {
+      dolfin_error("XMLMesh.cpp",
+                   "read mesh domains from XML file",
+                   "Expecting XML node <mesh_value_collection> but got <%s>",
+                   node_name.c_str());
+    }
+
+    // Get attributes
+    const std::string type = xml_domain->attribute("type").value();
+    const std::size_t dim = xml_domain->attribute("dim").as_uint();
+
+    // Check that the type is uint
+    if (type != "uint")
+    {
+      dolfin_error("XMLMesh.cpp",
+                   "read mesh domains from XML file",
+                   "Mesh domains must be marked as uint, not %s",
+                   type.c_str());
+    }
+
+    // Get domain data
+    std::vector<std::pair<std::pair<std::size_t, std::size_t>, std::size_t>>&
+      domain_data = mesh_data.domain_data[dim] ;
+
+    pugi::xml_node_iterator xml_domain_dim;
+    for (xml_domain_dim = xml_domain->begin();
+         xml_domain_dim != xml_domain->end(); ++xml_domain_dim)
+    {
+      const std::size_t cell_index
+        = xml_domain_dim->attribute("cell_index").as_uint();
+      const std::size_t local_entity
+          = xml_domain_dim->attribute("local_entity").as_uint();
+      const std::size_t value = xml_domain_dim->attribute("value").as_uint();
+
+      domain_data.push_back({{cell_index, local_entity}, value});
+    }
+  }
+}
+//-----------------------------------------------------------------------------
 void XMLMesh::read_array_uint(std::vector<std::size_t>& array,
                               const pugi::xml_node xml_array)
 {
@@ -449,10 +518,11 @@ void XMLMesh::write_data(const Mesh& mesh, const MeshData& data,
 
   // Write arrays
   typedef std::vector<std::map<std::string,
-                               std::vector<std::size_t> > >::const_iterator array_iterator_d;
+                               std::vector<std::size_t>>>::const_iterator array_iterator_d;
   typedef std::map<std::string,
-                   std::vector<std::size_t> >::const_iterator array_iterator;
-  for (array_iterator_d it_d = data._arrays.begin(); it_d != data._arrays.end(); ++it_d)
+                   std::vector<std::size_t>>::const_iterator array_iterator;
+  for (array_iterator_d it_d = data._arrays.begin();
+       it_d != data._arrays.end(); ++it_d)
   {
     const std::size_t dim =  it_d - data._arrays.begin();
     for (array_iterator it = it_d->begin(); it != it_d->end(); ++it)
@@ -461,7 +531,7 @@ void XMLMesh::write_data(const Mesh& mesh, const MeshData& data,
       std::string name = it->first;
       const std::vector<std::size_t>& array = it->second;
 
-      // Check data lenght
+      // Check data length
       if (array.size() != mesh.num_entities(dim))
       {
         dolfin_error("XMLMesh.cpp",
@@ -470,7 +540,8 @@ void XMLMesh::write_data(const Mesh& mesh, const MeshData& data,
       }
 
       // Copy data into MeshFunction
-      MeshFunction<std::size_t> mf(mesh, dim);
+      auto _mesh = reference_to_no_delete_pointer(mesh);
+      MeshFunction<std::size_t> mf(_mesh, dim);
       mf.rename(name, name);
       for (std::size_t i = 0; i < mf.size(); ++i)
         mf[i] = array[i];
@@ -498,7 +569,8 @@ void XMLMesh::write_domains(const Mesh& mesh, const MeshDomains& domains,
     {
       const std::map<std::size_t, std::size_t>& domain = domains.markers(d);
 
-      MeshValueCollection<std::size_t> collection(mesh, d);
+      auto _mesh = reference_to_no_delete_pointer(mesh);
+      MeshValueCollection<std::size_t> collection(_mesh, d);
       std::map<std::size_t, std::size_t>::const_iterator it;
       for (it = domain.begin(); it != domain.end(); ++it)
         collection.set_value(it->first, it->second);
