@@ -90,8 +90,10 @@ double CoordinateMatrix::norm(std::string norm_type) const
 {
   if (norm_type != "frobenius")
   {
-    error("Do not know to compute %s norm for CoordinateMatrix",
-          norm_type.c_str());
+    dolfin_error("CoordinateMatrix.cpp",
+                 "compute matrix norm",
+                 "Do not know to compute %s norm for CoordinateMatrix",
+                 norm_type.c_str());
   }
 
   double _norm = 0.0;
