@@ -25,12 +25,16 @@
 #ifndef __GENERIC_TENSOR_H
 #define __GENERIC_TENSOR_H
 
+#include <cstdint>
 #include <exception>
-#include <typeinfo>
 #include <memory>
-#include <dolfin/log/log.h>
+#include <typeinfo>
+#include <utility>
+
+#include <dolfin/common/ArrayView.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/types.h>
+#include <dolfin/log/log.h>
 #include "LinearAlgebraObject.h"
 
 namespace dolfin
@@ -63,31 +67,42 @@ namespace dolfin
     virtual std::size_t size(std::size_t dim) const = 0;
 
     /// Return local ownership range
-    virtual std::pair<std::size_t, std::size_t>
-      local_range(std::size_t dim) const = 0;
+    virtual std::pair<std::int64_t, std::int64_t>
+    local_range(std::size_t dim) const = 0;
 
     /// Get block of values
     virtual void get(double* block, const dolfin::la_index* num_rows,
                      const dolfin::la_index * const * rows) const = 0;
 
-    /// Set block of values
+    /// Set block of values using global indices
     virtual void set(const double* block, const dolfin::la_index* num_rows,
                      const dolfin::la_index * const * rows) = 0;
 
-    /// Add block of values
+    /// Set block of values using local indices
+    virtual void set_local(const double* block,
+                           const dolfin::la_index* num_rows,
+                           const dolfin::la_index * const * rows) = 0;
+
+    /// Add block of values using global indices
     virtual
       void add(const double* block,
-           const std::vector<const std::vector<dolfin::la_index>* >& rows) = 0;
+           const std::vector<ArrayView<const dolfin::la_index>>& rows) = 0;
 
-    /// Add block of values
-    virtual
-      void add(const double* block,
-               const std::vector<std::vector<dolfin::la_index> >& rows) = 0;
+    /// Add block of values using local indices
+    virtual void add_local(
+      const double* block,
+      const std::vector<ArrayView<const dolfin::la_index>>& rows) = 0;
 
-    /// Add block of values
+
+    /// Add block of values using global indices
     virtual void add(const double* block, const dolfin::la_index* num_rows,
                      const dolfin::la_index * const * rows) = 0;
 
+    /// Add block of values using local indices
+    virtual void add_local(const double* block,
+                           const dolfin::la_index* num_rows,
+                           const dolfin::la_index * const * rows) = 0;
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero() = 0;
 
@@ -95,7 +110,7 @@ namespace dolfin
     virtual void apply(std::string mode) = 0;
 
     /// Return MPI communicator
-    virtual MPI_Comm mpi_comm() const = 0;
+    //virtual MPI_Comm mpi_comm() const = 0;
 
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const = 0;
