@@ -30,29 +30,39 @@
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-LUSolver::LUSolver(std::string method)
+LUSolver::LUSolver(MPI_Comm comm, std::string method)
 {
-  // Initialize solver
-  init(method);
+  init(comm, method);
 }
 //-----------------------------------------------------------------------------
-LUSolver::LUSolver(std::shared_ptr<const GenericLinearOperator> A,
+LUSolver::LUSolver(std::string method) : LUSolver(MPI_COMM_WORLD, method)
+{
+  // Do nothing
+}
+//-----------------------------------------------------------------------------
+LUSolver::LUSolver(MPI_Comm comm,
+                   std::shared_ptr<const GenericLinearOperator> A,
                    std::string method)
 {
   // Initialize solver
-  init(method);
+  init(comm, method);
 
   // Set operator
   set_operator(A);
 }
 //-----------------------------------------------------------------------------
+LUSolver::LUSolver(std::shared_ptr<const GenericLinearOperator> A,
+                   std::string method) : LUSolver(MPI_COMM_WORLD, A, method)
+{
+  // Do nothing
+}
+//-----------------------------------------------------------------------------
 LUSolver::~LUSolver()
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-void
-LUSolver::set_operator(std::shared_ptr<const GenericLinearOperator> A)
+void LUSolver::set_operator(std::shared_ptr<const GenericLinearOperator> A)
 {
   dolfin_assert(solver);
   solver->parameters.update(parameters);
@@ -69,7 +79,7 @@ std::size_t LUSolver::solve(GenericVector& x, const GenericVector& b)
 }
 //-----------------------------------------------------------------------------
 std::size_t LUSolver::solve(const GenericLinearOperator& A, GenericVector& x,
-                             const GenericVector& b)
+                            const GenericVector& b)
 {
   dolfin_assert(solver);
 
@@ -98,14 +108,13 @@ std::size_t LUSolver::solve_transpose(const GenericLinearOperator& A,
   return solver->solve_transpose(A, x, b);
 }
 //-----------------------------------------------------------------------------
-void LUSolver::init(std::string method)
+void LUSolver::init(MPI_Comm comm, std::string method)
 {
   // Get default linear algebra factory
   DefaultFactory factory;
 
   // Get list of available methods
-  std::vector<std::pair<std::string, std::string> >
-    methods = factory.lu_solver_methods();
+  std::map<std::string, std::string> methods = factory.lu_solver_methods();
 
   // Check that method is available
   if (!LinearSolver::in_list(method, methods))
@@ -121,7 +130,7 @@ void LUSolver::init(std::string method)
   parameters = dolfin::parameters("lu_solver");
 
   // Initialize solver
-  solver = factory.create_lu_solver(method);
+  solver = factory.create_lu_solver(comm, method);
   solver->parameters.update(parameters);
 }
 //-----------------------------------------------------------------------------
