@@ -25,14 +25,15 @@
 #ifndef __MATRIX_H
 #define __MATRIX_H
 
-#include <boost/tuple/tuple.hpp>
-#include <boost/shared_ptr.hpp>
+#include <memory>
+#include <tuple>
 #include "DefaultFactory.h"
 #include "GenericMatrix.h"
 
 namespace dolfin
 {
 
+  class GenericVector;
   class TensorLayout;
 
   /// This class provides the default DOLFIN matrix class,
@@ -43,16 +44,16 @@ namespace dolfin
   public:
 
     /// Create empty matrix
-    Matrix()
+    Matrix(MPI_Comm comm=MPI_COMM_WORLD)
     {
       DefaultFactory factory;
-      matrix = factory.create_matrix();
+      matrix = factory.create_matrix(comm);
     }
 
     /// Copy constructor
     Matrix(const Matrix& A) : matrix(A.matrix->copy()) {}
 
-    /// Create a Vector from a GenericVetor
+    /// Create a Vector from a GenericVector
     Matrix(const GenericMatrix& A) : matrix(A.copy()) {}
 
     /// Destructor
@@ -64,15 +65,23 @@ namespace dolfin
     virtual void init(const TensorLayout& tensor_layout)
     { matrix->init(tensor_layout); }
 
+    /// Return true if matrix is empty
+    virtual bool empty() const
+    { return matrix->empty(); }
+
     /// Return size of given dimension
     virtual std::size_t size(std::size_t dim) const
     { return matrix->size(dim); }
 
     /// Return local ownership range
-    virtual std::pair<std::size_t, std::size_t>
+    virtual std::pair<std::int64_t, std::int64_t>
       local_range(std::size_t dim) const
     { return matrix->local_range(dim); }
 
+    /// Return number of non-zero entries in matrix (collective)
+    virtual std::size_t nnz() const
+    { return matrix->nnz(); }
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero()
     { matrix->zero(); }
@@ -81,6 +90,10 @@ namespace dolfin
     virtual void apply(std::string mode)
     { matrix->apply(mode); }
 
+    /// Return MPI communicator
+    MPI_Comm mpi_comm() const
+    { return matrix->mpi_comm(); }
+
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const
     { return "<Matrix wrapper of " + matrix->str(verbose) + ">"; }
@@ -88,17 +101,17 @@ namespace dolfin
     //--- Implementation of the GenericMatrix interface ---
 
     /// Return copy of matrix
-    virtual boost::shared_ptr<GenericMatrix> copy() const
+    virtual std::shared_ptr<GenericMatrix> copy() const
     {
-      boost::shared_ptr<Matrix> A(new Matrix(*this));
+      std::shared_ptr<Matrix> A(new Matrix(*this));
       return A;
     }
 
     /// Resize vector y such that is it compatible with matrix for
-    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// multiplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
     /// case, size and layout are important.
-    virtual void resize(GenericVector& y, std::size_t dim) const
-    { matrix->resize(y, dim); }
+    virtual void init_vector(GenericVector& y, std::size_t dim) const
+    { matrix->init_vector(y, dim); }
 
     /// Get block of values
     virtual void get(double* block,
@@ -106,18 +119,30 @@ namespace dolfin
                      std::size_t n, const dolfin::la_index* cols) const
     { matrix->get(block, m, rows, n, cols); }
 
-    /// Set block of values
+    /// Set block of values using global indices
     virtual void set(const double* block,
                      std::size_t m, const dolfin::la_index* rows,
                      std::size_t n, const dolfin::la_index* cols)
     { matrix->set(block, m, rows, n, cols); }
 
-    /// Add block of values
+    /// Set block of values using local indices
+    virtual void set_local(const double* block,
+                           std::size_t m, const dolfin::la_index* rows,
+                           std::size_t n, const dolfin::la_index* cols)
+    { matrix->set_local(block, m, rows, n, cols); }
+
+    /// Add block of values using global indices
     virtual void add(const double* block,
                      std::size_t m, const dolfin::la_index* rows,
                      std::size_t n, const dolfin::la_index* cols)
     { matrix->add(block, m, rows, n, cols); }
 
+    /// Add block of values using local indices
+    virtual void add_local(const double* block,
+                           std::size_t m, const dolfin::la_index* rows,
+                           std::size_t n, const dolfin::la_index* cols)
+    { matrix->add_local(block, m, rows, n, cols); }
+
     /// Add multiple of given matrix (AXPY operation)
     virtual void axpy(double a, const GenericMatrix& A,
                       bool same_nonzero_pattern)
@@ -142,10 +167,18 @@ namespace dolfin
     virtual void zero(std::size_t m, const dolfin::la_index* rows)
     { matrix->zero(m, rows); }
 
-    /// Set given rows to identity matrix
+    /// Set given rows (local row indices) to zero
+    virtual void zero_local(std::size_t m, const dolfin::la_index* rows)
+    { matrix->zero_local(m, rows); }
+
+    /// Set given rows (global row indices) to identity matrix
     virtual void ident(std::size_t m, const dolfin::la_index* rows)
     { matrix->ident(m, rows); }
 
+    /// Set given rows (local row indices) to identity matrix
+    virtual void ident_local(std::size_t m, const dolfin::la_index* rows)
+    { matrix->ident_local(m, rows); }
+
     // Matrix-vector product, y = Ax
     virtual void mult(const GenericVector& x, GenericVector& y) const
     { matrix->mult(x, y); }
@@ -154,6 +187,14 @@ namespace dolfin
     virtual void transpmult(const GenericVector& x, GenericVector& y) const
     { matrix->transpmult(x, y); }
 
+    /// Get diagonal of a matrix
+    virtual void get_diagonal(GenericVector& x) const
+    { matrix->get_diagonal(x); }
+
+    /// Set diagonal of a matrix
+    virtual void set_diagonal(const GenericVector& x)
+    { matrix->set_diagonal(x); }
+
     /// Multiply matrix by given number
     virtual const Matrix& operator*= (double a)
     { *matrix *= a; return *this; }
@@ -170,12 +211,6 @@ namespace dolfin
     virtual bool is_symmetric(double tol) const
     { return matrix->is_symmetric(tol); }
 
-    /// Return pointers to underlying compressed storage data.
-    /// See GenericMatrix for documentation.
-    virtual boost::tuples::tuple<const std::size_t*, const std::size_t*,
-      const double*, int> data() const
-    { return matrix->data(); }
-
     //--- Special functions ---
 
     /// Return linear algebra backend factory
@@ -192,10 +227,10 @@ namespace dolfin
     virtual GenericMatrix* instance()
     { return matrix.get(); }
 
-    virtual boost::shared_ptr<const LinearAlgebraObject> shared_instance() const
+    virtual std::shared_ptr<const LinearAlgebraObject> shared_instance() const
     { return matrix; }
 
-    virtual boost::shared_ptr<LinearAlgebraObject> shared_instance()
+    virtual std::shared_ptr<LinearAlgebraObject> shared_instance()
     { return matrix; }
 
     //--- Special Matrix functions ---
@@ -207,7 +242,7 @@ namespace dolfin
   private:
 
     // Pointer to concrete implementation
-    boost::shared_ptr<GenericMatrix> matrix;
+    std::shared_ptr<GenericMatrix> matrix;
 
   };
 
