@@ -46,23 +46,26 @@ namespace dolfin
     virtual ~PETScFactory() {}
 
     /// Create empty matrix
-    std::shared_ptr<GenericMatrix> create_matrix() const;
+    std::shared_ptr<GenericMatrix> create_matrix(MPI_Comm comm) const;
 
     /// Create empty vector
-    std::shared_ptr<GenericVector> create_vector() const;
+    std::shared_ptr<GenericVector> create_vector(MPI_Comm comm) const;
 
     /// Create empty tensor layout
     std::shared_ptr<TensorLayout> create_layout(std::size_t rank) const;
 
     /// Create empty linear operator
-    std::shared_ptr<GenericLinearOperator> create_linear_operator() const;
+    std::shared_ptr<GenericLinearOperator>
+      create_linear_operator(MPI_Comm comm) const;
 
     /// Create LU solver
-    std::shared_ptr<GenericLUSolver> create_lu_solver(std::string method) const;
+    std::shared_ptr<GenericLUSolver> create_lu_solver(MPI_Comm comm,
+                                                      std::string method) const;
 
     /// Create Krylov solver
     std::shared_ptr<GenericLinearSolver>
-    create_krylov_solver(std::string method,
+    create_krylov_solver(MPI_Comm comm,
+                         std::string method,
                          std::string preconditioner) const;
 
     /// Return a list of available LU solver methods
