@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2005 Johan Jansson
+// Copyright (C) 2004-2015 Johan Jansson and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -19,9 +19,6 @@
 // Modified by Johan Hoffman 2005
 // Modified by Andy R. Terrel 2005
 // Modified by Garth N. Wells 2005-2010
-//
-// First added:  2005-12-02
-// Last changed: 2014-07-09
 
 #ifndef __DOLFIN_PETSC_KRYLOV_SOLVER_H
 #define __DOLFIN_PETSC_KRYLOV_SOLVER_H
@@ -29,8 +26,9 @@
 #ifdef HAS_PETSC
 
 #include <map>
-#include <petscksp.h>
 #include <memory>
+#include <string>
+#include <petscksp.h>
 #include <dolfin/common/types.h>
 #include "GenericLinearSolver.h"
 #include "PETScObject.h"
@@ -49,37 +47,51 @@ namespace dolfin
   class PETScSNESSolver;
   class VectorSpaceBasis;
 
-  /// This class implements Krylov methods for linear systems
-  /// of the form Ax = b. It is a wrapper for the Krylov solvers
-  /// of PETSc.
+  /// This class implements Krylov methods for linear systems of the
+  /// form Ax = b. It is a wrapper for the Krylov solvers of PETSc.
 
   class PETScKrylovSolver : public GenericLinearSolver, public PETScObject
   {
   public:
 
-    /// Create Krylov solver for a particular method and names
+    /// Norm types used in convergence testing. Not all solvers types
+    /// support all norm types (see
+    /// http://www.mcs.anl.gov/petsc/petsc-current/docs/manualpages/KSP/KSPSetNormType.html). Note
+    /// that 'default' is a reserved keyword, so we use 'default_norm'
+    enum class norm_type {none, default_norm, preconditioned, unpreconditioned, natural};
+
+    /// Create Krylov solver for a particular method and named
     /// preconditioner
-    PETScKrylovSolver(std::string method = "default",
-                      std::string preconditioner = "default");
+    PETScKrylovSolver(MPI_Comm comm,
+                      std::string method="default",
+                      std::string preconditioner="default");
+
+    /// Create Krylov solver for a particular method and named
+    /// preconditioner
+    PETScKrylovSolver(std::string method="default",
+                      std::string preconditioner="default");
 
     /// Create Krylov solver for a particular method and
-    /// PETScPreconditioner
-    PETScKrylovSolver(std::string method, PETScPreconditioner& preconditioner);
+    /// PETScPreconditioner (shared_ptr version)
+    PETScKrylovSolver(MPI_Comm comm,
+                      std::string method,
+                      std::shared_ptr<PETScPreconditioner> preconditioner);
 
     /// Create Krylov solver for a particular method and
     /// PETScPreconditioner (shared_ptr version)
     PETScKrylovSolver(std::string method,
-		      std::shared_ptr<PETScPreconditioner> preconditioner);
+                      std::shared_ptr<PETScPreconditioner> preconditioner);
 
     /// Create Krylov solver for a particular method and
-    /// PETScPreconditioner
-    PETScKrylovSolver(std::string method,
-                      PETScUserPreconditioner& preconditioner);
+    /// PETScPreconditioner (shared_ptr version)
+    PETScKrylovSolver(MPI_Comm comm,
+                      std::string method,
+                      std::shared_ptr<PETScUserPreconditioner> preconditioner);
 
     /// Create Krylov solver for a particular method and
     /// PETScPreconditioner (shared_ptr version)
     PETScKrylovSolver(std::string method,
-		    std::shared_ptr<PETScUserPreconditioner> preconditioner);
+                      std::shared_ptr<PETScUserPreconditioner> preconditioner);
 
     /// Create solver wrapper of a PETSc KSP object
     explicit PETScKrylovSolver(KSP ksp);
@@ -90,24 +102,10 @@ namespace dolfin
     /// Set operator (matrix)
     void set_operator(std::shared_ptr<const GenericLinearOperator> A);
 
-    /// Set operator (matrix)
-    void set_operator(std::shared_ptr<const PETScBaseMatrix> A);
-
     /// Set operator (matrix) and preconditioner matrix
     void set_operators(std::shared_ptr<const GenericLinearOperator> A,
                        std::shared_ptr<const GenericLinearOperator> P);
 
-    /// Set operator (matrix) and preconditioner matrix
-    void set_operators(std::shared_ptr<const PETScBaseMatrix> A,
-                       std::shared_ptr<const PETScBaseMatrix> P);
-
-    /// Set null space of the operator (matrix). This is used to solve
-    /// singular systems
-    void set_nullspace(const VectorSpaceBasis& nullspace);
-
-    /// Get operator (matrix)
-    const PETScBaseMatrix& get_operator() const;
-
     /// Solve linear system Ax = b and return number of iterations
     std::size_t solve(GenericVector& x, const GenericVector& b);
 
@@ -118,41 +116,82 @@ namespace dolfin
     std::size_t solve(const GenericLinearOperator& A, GenericVector& x,
                       const GenericVector& b);
 
-    /// Solve linear system Ax = b and return number of iterations
-    std::size_t solve(const PETScBaseMatrix& A, PETScVector& x,
-                      const PETScVector& b);
+    /// Use nonzero intial guess for solution function
+    /// (nonzero_guess=true, the solution vector x will not be zeroed
+    /// before the solver starts)
+    void set_nonzero_guess(bool nonzero_guess);
+
+    /// Reuse preconditioner if true, even if matrix operator changes
+    /// (by default preconditioner will be re-built if the matrix
+    /// changes)
+    void set_reuse_preconditioner(bool reuse_pc);
+
+    /// Set tolerances (relative residual, alsolute residial, maximum
+    /// number of iterations)
+    void set_tolerances(double relative, double absolute, double diverged,
+                        int max_iter);
+
+    /// Set norm type used in convergence testing - not all solvers
+    /// types support all norm types
+    void set_norm_type(norm_type type);
+
+    /// Get norm type used in convergence testing
+    norm_type get_norm_type() const;
+
+    /// Monitor residual at each iteration
+    void monitor(bool monitor_convergence);
+
+    /// Sets the prefix used by PETSc when searching the PETSc options
+    /// database
+    void set_options_prefix(std::string options_prefix);
+
+    /// Returns the prefix used by PETSc when searching the PETSc
+    /// options database
+    std::string get_options_prefix() const;
+
+    /// Set options from PETSc options database
+    void set_from_options() const;
 
     /// Return informal string representation (pretty-print)
     std::string str(bool verbose) const;
 
+    /// Return MPI communicator
+    MPI_Comm mpi_comm() const;
+
     /// Return PETSc KSP pointer
     KSP ksp() const;
 
     /// Return a list of available solver methods
-    static std::vector<std::pair<std::string, std::string> > methods();
+    static std::map<std::string, std::string> methods();
 
-    /// Return a list of available preconditioners
-    static std::vector<std::pair<std::string, std::string> > preconditioners();
-
-    /// Set options prefix
-    void set_options_prefix(std::string prefix);
+    /// Return a list of available named preconditioners
+    static std::map<std::string, std::string> preconditioners();
 
     /// Default parameter values
     static Parameters default_parameters();
 
+    /// Return parameter type: "krylov_solver" or "lu_solver"
+    std::string parameter_type() const
+    { return "krylov_solver"; }
+
     friend class PETScSNESSolver;
     friend class PETScTAOSolver;
 
   private:
 
-    // Initialize KSP solver
-    void init(const std::string& method);
+    // Return norm_type enum for norm string
+    static PETScKrylovSolver::norm_type get_norm_type(std::string norm);
 
-    // Set PETSc operators
-    //void set_petsc_operators();
+    // Set operator (matrix)
+    void _set_operator(std::shared_ptr<const PETScBaseMatrix> A);
 
-    // Set options that affect KSP object
-    void set_petsc_ksp_options();
+    // Set operator (matrix) and preconditioner matrix
+    void _set_operators(std::shared_ptr<const PETScBaseMatrix> A,
+                        std::shared_ptr<const PETScBaseMatrix> P);
+
+    // Solve linear system Ax = b and return number of iterations
+    std::size_t _solve(const PETScBaseMatrix& A, PETScVector& x,
+                       const PETScVector& b);
 
     // Report the number of iterations
     void write_report(int num_iterations, KSPConvergedReason reason);
@@ -164,8 +203,7 @@ namespace dolfin
     static const std::map<std::string, const KSPType> _methods;
 
     // Available solvers descriptions
-    static const std::vector<std::pair<std::string, std::string> >
-      _methods_descr;
+    static const std::map<std::string, std::string> _methods_descr;
 
     // PETSc solver pointer
     KSP _ksp;
@@ -182,12 +220,6 @@ namespace dolfin
     // Matrix used to construct the preconditioner
     std::shared_ptr<const PETScBaseMatrix> _matP;
 
-    // Null space vectors
-    std::vector<PETScVector> _nullspace;
-
-    // PETSc null space
-    MatNullSpace petsc_nullspace;
-
     bool preconditioner_set;
 
   };
