@@ -46,7 +46,7 @@ namespace dolfin
   public:
 
     /// Constructor
-    PETScLinearOperator();
+    explicit PETScLinearOperator(MPI_Comm comm);
 
     //--- Implementation of the GenericLinearOperator interface ---
 
@@ -56,6 +56,10 @@ namespace dolfin
     /// Compute matrix-vector product y = Ax
     virtual void mult(const GenericVector& x, GenericVector& y) const;
 
+    /// Return MPI communicator
+    virtual MPI_Comm  mpi_comm() const
+    { return PETScBaseMatrix::mpi_comm(); }
+
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const;
 
