@@ -14,88 +14,72 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// First added:  2012-02-24
-// Last changed:
+
+#include <memory>
 
 #include <dolfin/log/log.h>
 #include <dolfin/log/LogStream.h>
+#include "IndexMap.h"
 #include "SparsityPattern.h"
 #include "TensorLayout.h"
 
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-TensorLayout::TensorLayout(std::size_t pdim, bool sparsity_pattern)
-  : primary_dim(pdim), block_size(0), _mpi_comm(MPI_COMM_NULL)
+TensorLayout::TensorLayout(std::size_t pdim, Sparsity sparsity_pattern)
+  : primary_dim(pdim), _mpi_comm(MPI_COMM_NULL)
 {
   // Create empty sparsity pattern
-  if (sparsity_pattern)
-    _sparsity_pattern.reset(new SparsityPattern(primary_dim));
+  if (sparsity_pattern == TensorLayout::Sparsity::SPARSE)
+    _sparsity_pattern = std::make_shared<SparsityPattern>(primary_dim);
 }
 //-----------------------------------------------------------------------------
 TensorLayout::TensorLayout(const MPI_Comm mpi_comm,
-                           const std::vector<std::size_t>& dims,
-                           std::size_t pdim, std::size_t bs,
-                           const std::vector<std::pair<std::size_t,
-                           std::size_t>>& ownership_range,
-                           bool sparsity_pattern)
-  : primary_dim(pdim), block_size(bs), local_to_global_map(dims.size()),
-    _mpi_comm(mpi_comm), _shape(dims), _ownership_range(ownership_range)
+             std::vector<std::shared_ptr<const IndexMap>> index_maps,
+             std::size_t pdim,
+             Sparsity sparsity_pattern,
+             Ghosts ghosted)
+  : primary_dim(pdim), _mpi_comm(mpi_comm), _index_maps(index_maps),
+    _ghosted(ghosted)
 {
+  if (sparsity_pattern == TensorLayout::Sparsity::SPARSE)
+    _sparsity_pattern = std::make_shared<SparsityPattern>(primary_dim);
+
   // Only rank 2 sparsity patterns are supported
-  dolfin_assert(!(sparsity_pattern && dims.size() != 2));
-
-  // Check that dimensions match
-  dolfin_assert(dims.size() == ownership_range.size());
-
-  // Create empty sparsity pattern
-  if (sparsity_pattern)
-    _sparsity_pattern.reset(new SparsityPattern(primary_dim));
+  dolfin_assert(!(_sparsity_pattern && index_maps.size() != 2));
 }
 //-----------------------------------------------------------------------------
 void TensorLayout::init(
   const MPI_Comm mpi_comm,
-  const std::vector<std::size_t>& dims, std::size_t bs,
-  const std::vector<std::pair<std::size_t, std::size_t>>& ownership_range)
+  const std::vector<std::shared_ptr<const IndexMap>> index_maps,
+  const Ghosts ghosted)
 {
   // Only rank 2 sparsity patterns are supported
-  dolfin_assert(!(_sparsity_pattern && dims.size() != 2));
+  dolfin_assert(!(_sparsity_pattern && index_maps.size() != 2));
 
-  // Check that dimensions match
-  dolfin_assert(dims.size() == ownership_range.size());
-
-  local_to_global_map.resize(dims.size());
-
-  // Store MPI communicator
+  // Store everything
+  _index_maps = index_maps;
   _mpi_comm = mpi_comm;
-
-  // Store dimensions
-  _shape = dims;
-
-  // Store block size
-  block_size = bs;
-
-  // Store ownership range
-  _ownership_range = ownership_range;
+  _ghosted = ghosted;
 }
 //-----------------------------------------------------------------------------
 std::size_t TensorLayout::rank() const
 {
-  return _shape.size();
+  return _index_maps.size();
 }
 //-----------------------------------------------------------------------------
 std::size_t TensorLayout::size(std::size_t i) const
 {
-  dolfin_assert(i < _shape.size());
-  return _shape[i];
+  dolfin_assert(i < _index_maps.size());
+  return _index_maps[i]->size(IndexMap::MapSize::GLOBAL);
 }
 //-----------------------------------------------------------------------------
 std::pair<std::size_t, std::size_t>
 TensorLayout::local_range(std::size_t dim) const
 {
-  dolfin_assert(dim < 2);
-  return _ownership_range[dim];
+  dolfin_assert(dim < _index_maps.size());
+  return _index_maps[dim]->local_range();
+
 }
 //-----------------------------------------------------------------------------
 std::string TensorLayout::str(bool verbose) const
@@ -104,8 +88,8 @@ std::string TensorLayout::str(bool verbose) const
   s << "<TensorLayout for tensor of rank " << rank() << ">" << std::endl;
   for (std::size_t i = 0; i < rank(); i++)
   {
-    s << " Local range for dim " << i << ": [" << _ownership_range[i].first
-        << ", " << _ownership_range[i].second << ")" << std::endl;
+    s << " Local range for dim " << i << ": [" << local_range(i).first
+      << ", " << local_range(i).second << ")" << std::endl;
   }
   return s.str();
 }
