@@ -10,12 +10,10 @@
 
 #include <dolfin/la/GenericTensor.h>
 #include <dolfin/la/GenericMatrix.h>
-#include <dolfin/la/GenericSparsityPattern.h>
 #include <dolfin/la/GenericVector.h>
 #include <dolfin/la/VectorSpaceBasis.h>
 #include <dolfin/la/GenericLinearSolver.h>
 #include <dolfin/la/GenericLUSolver.h>
-#include <dolfin/la/GenericPreconditioner.h>
 
 #include <dolfin/la/PETScOptions.h>
 #include <dolfin/la/PETScObject.h>
@@ -33,10 +31,10 @@
 #include <dolfin/la/PETScKrylovSolver.h>
 #include <dolfin/la/PETScLUSolver.h>
 #include <dolfin/la/BelosKrylovSolver.h>
-#include <dolfin/la/MUMPSLUSolver.h>
-#include <dolfin/la/PaStiXLUSolver.h>
+#include <dolfin/la/TrilinosPreconditioner.h>
+#include <dolfin/la/MueluPreconditioner.h>
+#include <dolfin/la/Ifpack2Preconditioner.h>
 
-#include <dolfin/la/STLMatrix.h>
 #include <dolfin/la/CoordinateMatrix.h>
 #include <dolfin/la/EigenVector.h>
 #include <dolfin/la/PETScVector.h>
@@ -45,12 +43,14 @@
 #include <dolfin/la/TensorLayout.h>
 #include <dolfin/la/SparsityPattern.h>
 
+#include <dolfin/la/IndexMap.h>
+
 #include <dolfin/la/GenericLinearAlgebraFactory.h>
 #include <dolfin/la/DefaultFactory.h>
 #include <dolfin/la/EigenFactory.h>
 #include <dolfin/la/PETScUserPreconditioner.h>
 #include <dolfin/la/PETScFactory.h>
-#include <dolfin/la/STLFactory.h>
+#include <dolfin/la/TpetraFactory.h>
 #include <dolfin/la/SLEPcEigenSolver.h>
 #include <dolfin/la/Vector.h>
 #include <dolfin/la/Matrix.h>
