@@ -1,4 +1,4 @@
-// Copyright (C) 2003-2007 Anders Logg
+// Copyright (C) 2003-2016 Anders Logg
 //
 // This file is part of DOLFIN.
 //
@@ -18,7 +18,7 @@
 // Thanks to Jim Tilander for many helpful hints.
 //
 // First added:  2003-03-13
-// Last changed: 2007-05-11
+// Last changed: 2016-06-10
 
 #ifndef __LOG_MANAGER_H
 #define __LOG_MANAGER_H
@@ -28,20 +28,12 @@
 namespace dolfin
 {
 
-  /* FIXME: logging in destructors may fail at exit because the logger instance
-     may already be destroyed ("static initialization order fiasco"). The same
-     may happen at startup, if logging from constructors of static objects. The
-     logger instance should be converted to a heap-allocated (and never
-     deleted) object, with an accessor function. Like in SubSystemsManager, but
-     maybe with a static pointer rather than a static object in the accessor
-     function. */
-
   class LogManager
   {
   public:
 
     // Singleton instance of logger
-    static Logger logger;
+    static Logger& logger();
 
   };
 
