@@ -23,9 +23,7 @@
 // First added:  2006-06-21
 // Last changed: 2014-02-06
 
-#include <dolfin/common/timing.h>
-
-#include <dolfin/log/dolfin_log.h>
+#include <dolfin/log/log.h>
 #include "BoundaryMesh.h"
 #include "Cell.h"
 #include "Facet.h"
@@ -85,17 +83,17 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
 
   // Shared vertices for full mesh
   // FIXME: const_cast
-  const std::map<unsigned int, std::set<unsigned int> > &
+  const std::map<std::int32_t, std::set<unsigned int>>&
     shared_vertices = const_cast<Mesh&>(mesh).topology().shared_entities(0);
 
   // Shared vertices for boundary mesh
-  std::map<unsigned int, std::set<unsigned int> > shared_boundary_vertices;
+  std::map<std::int32_t, std::set<unsigned int>> shared_boundary_vertices;
   if (exterior)
   {
     // Extract shared vertices if vertex is identified as part of globally
     // exterior facet.
     std::vector<std::size_t> boundary_global_indices;
-    for (std::map<unsigned int, std::set<unsigned int> >::const_iterator
+    for (std::map<std::int32_t, std::set<unsigned int>>::const_iterator
         sv_it=shared_vertices.begin(); sv_it != shared_vertices.end(); ++sv_it)
     {
       std::size_t local_mesh_index = sv_it->first;
@@ -115,7 +113,7 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
     }
 
     // Distribute all shared boundary vertices
-    std::vector<std::vector<std::size_t> > boundary_global_indices_all;
+    std::vector<std::vector<std::size_t>> boundary_global_indices_all;
     MPI::all_gather(mesh.mpi_comm(), boundary_global_indices,
                      boundary_global_indices_all);
 
@@ -178,7 +176,8 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
   std::size_t num_owned_vertices = 0;
   std::size_t num_boundary_cells = 0;
 
-  MeshFunction<bool> boundary_facet(mesh, D - 1, false);
+  MeshFunction<bool> boundary_facet(reference_to_no_delete_pointer(mesh),
+                                    D - 1, false);
   for (FacetIterator f(mesh); !f.end(); ++f)
   {
     // Boundary facets are connected to exactly one cell
@@ -206,7 +205,7 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
             // Determine "owner" of global_mesh_index
             std::size_t owner = my_rank;
 
-            std::map<unsigned int, std::set<unsigned int> >::const_iterator
+            std::map<std::int32_t, std::set<unsigned int>>::const_iterator
               other_processes_it
               = shared_boundary_vertices.find(local_mesh_index);
             if (other_processes_it != shared_boundary_vertices.end() && D > 1)
@@ -249,15 +248,18 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
   */
 
   // Specify number of vertices and cells
-  editor.init_vertices_global(num_boundary_vertices, MPI::sum(mesh.mpi_comm(),
-                                                              num_owned_vertices));
+  editor.init_vertices_global(num_boundary_vertices,
+                              MPI::sum(mesh.mpi_comm(), num_owned_vertices));
   editor.init_cells_global(num_boundary_cells, MPI::sum(mesh.mpi_comm(),
                                                         num_boundary_cells));
 
   // Write vertex map
   MeshFunction<std::size_t>& vertex_map = boundary.entity_map(0);
   if (num_boundary_vertices > 0)
-    vertex_map.init(boundary, 0, num_boundary_vertices);
+  {
+    vertex_map.init(reference_to_no_delete_pointer(boundary), 0,
+                    num_boundary_vertices);
+  }
   std::map<std::size_t, std::size_t>::const_iterator it;
   for (it = boundary_vertices.begin(); it != boundary_vertices.end(); ++it)
     vertex_map[it->second] = it->first;
@@ -273,7 +275,7 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
   // Set global indices of owned vertices, request global indices for
   // vertices owned elsewhere
   std::map<std::size_t, std::size_t> global_indices;
-  std::vector<std::vector<std::size_t> > request_global_indices(num_processes);
+  std::vector<std::vector<std::size_t>> request_global_indices(num_processes);
 
   std::size_t current_index = start_index;
   for (std::size_t local_boundary_index = 0;
@@ -291,12 +293,12 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
   }
 
   // Send and receive requests from other processes
-  std::vector<std::vector<std::size_t> > global_index_requests(num_processes);
+  std::vector<std::vector<std::size_t>> global_index_requests(num_processes);
   MPI::all_to_all(mesh.mpi_comm(), request_global_indices,
                   global_index_requests);
 
   // Find response to requests of global indices
-  std::vector<std::vector<std::size_t> > respond_global_indices(num_processes);
+  std::vector<std::vector<std::size_t>> respond_global_indices(num_processes);
   for (std::size_t i = 0; i < num_processes; i++)
   {
     const std::size_t N = global_index_requests[i].size();
@@ -308,7 +310,7 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
   }
 
   // Scatter responses back to requesting processes
-  std::vector<std::vector<std::size_t> > global_index_responses(num_processes);
+  std::vector<std::vector<std::size_t>> global_index_responses(num_processes);
   MPI::all_to_all(mesh.mpi_comm(), respond_global_indices,
                   global_index_responses);
 
@@ -352,7 +354,10 @@ void BoundaryComputation::compute_boundary(const Mesh& mesh,
   // Create cells (facets) and map between boundary mesh cells and facets parent
   MeshFunction<std::size_t>& cell_map = boundary.entity_map(D - 1);
   if (num_boundary_cells > 0)
-    cell_map.init(boundary, D - 1, num_boundary_cells);
+  {
+    cell_map.init(reference_to_no_delete_pointer(boundary), D - 1,
+                  num_boundary_cells);
+  }
   std::vector<std::size_t>
     cell(boundary.type().num_vertices(boundary.topology().dim()));
   std::size_t current_cell = 0;
