@@ -222,7 +222,7 @@ double IntervalCell::facet_area(const Cell& cell, std::size_t facet) const
 }
 //-----------------------------------------------------------------------------
 void IntervalCell::order(Cell& cell,
-                         const std::vector<std::size_t>&
+                         const std::vector<std::int64_t>&
                          local_to_global_vertex_indices) const
 {
   // Sort i - j for i > j: 1 - 0
