@@ -94,7 +94,7 @@ namespace dolfin
     double facet_area(const Cell& cell, std::size_t facet) const;
 
     /// Order entities locally
-    void order(Cell& cell, const std::vector<std::size_t>&
+    void order(Cell& cell, const std::vector<std::int64_t>&
                local_to_global_vertex_indices) const;
 
     /// Check whether given point collides with cell
