@@ -39,6 +39,7 @@
 #include <dolfin/geometry/BoundingBoxTree.h>
 #include "BoundaryMesh.h"
 #include "Cell.h"
+#include "DistributedMeshTools.h"
 #include "Facet.h"
 #include "LocalMeshData.h"
 #include "MeshColoring.h"
@@ -222,6 +223,12 @@ void Mesh::init() const
       init(d0, d1);
 }
 //-----------------------------------------------------------------------------
+void Mesh::init_global(std::size_t dim) const
+{
+  init(dim);
+  DistributedMeshTools::number_entities(*this, dim);
+}
+//-----------------------------------------------------------------------------
 void Mesh::clean()
 {
   const std::size_t D = topology().dim();
