@@ -25,16 +25,15 @@
 #define __MESH_DATA_H
 
 #include <map>
+#include <memory>
 #include <string>
 #include <vector>
-#include <memory>
 #include <dolfin/common/Variable.h>
 
 namespace dolfin
 {
 
   class Mesh;
-  template <typename T> class MeshFunction;
 
   /// The class MeshData is a container for auxiliary mesh data,
   /// represented either as arrays or maps. Each dataset is identified
@@ -91,29 +90,6 @@ namespace dolfin
 
     //--- Creation of data ---
 
-    /// This function is no longer supported. It is retained to print
-    /// an informative error message. It will be removed in the
-    /// future.
-    std::shared_ptr<MeshFunction<std::size_t> >
-      create_mesh_function(std::string name);
-
-    /// This function is no longer supported. It is retained to print
-    /// an informative error message. It will be removed in the
-    /// future.
-    std::shared_ptr<MeshFunction<std::size_t> >
-      create_mesh_function(std::string name, std::size_t dim);
-
-    /// Create empty array (vector) with given name
-    ///
-    /// *Arguments*
-    ///     name (std::string)
-    ///         The name of the array.
-    ///
-    /// *Returns*
-    ///     std::vector<std::size_t>
-    ///         The array.
-    //std::vector<std::size_t>& create_array(std::string name);
-
     /// Create array (vector) with given name and size
     ///
     /// *Arguments*
@@ -125,18 +101,10 @@ namespace dolfin
     /// *Returns*
     ///     std::vector<std::size_t>
     ///         The array.
-    //private:
     std::vector<std::size_t>& create_array(std::string name, std::size_t dim);
 
-    //public:
     //--- Retrieval of data ---
 
-    /// This function is no longer supported. It is retained to print
-    /// an informative error message. It will be removed in the
-    /// future.
-    std::shared_ptr<MeshFunction<std::size_t> >
-      mesh_function(const std::string name) const;
-
     /// Return array with given name (returning zero if data is not
     /// available)
     ///
@@ -195,7 +163,7 @@ namespace dolfin
     // A map from named mesh array data to vector for dim
     std::vector<std::map<std::string, std::vector<std::size_t> > > _arrays;
 
-    // List of depcrecated named data
+    // List of deprecated named data
     std::vector<std::string> _deprecated_names;
 
   };
