@@ -23,12 +23,11 @@
 
 #include <vector>
 #include "CellType.h"
+#include "Mesh.h"
 
 namespace dolfin
 {
 
-  class CellType;
-  class Mesh;
   class Point;
 
   /// A simple mesh editor for creating simplicial meshes in 1D, 2D
@@ -53,15 +52,16 @@ namespace dolfin
     ///         The topological dimension.
     ///     gdim (std::size_t)
     ///         The geometrical dimension.
-    ///
+    ///     degree (std::size_t)
+    ///         The polynomial degree.
     /// *Example*
     ///     .. code-block:: c++
     ///
     ///         Mesh mesh;
     ///         MeshEditor editor;
-    ///         editor.open(mesh, 2, 2);
+    ///         editor.open(mesh, 2, 2, 1);
     ///
-    void open(Mesh& mesh, std::size_t tdim, std::size_t gdim);
+    void open(Mesh& mesh, std::size_t tdim, std::size_t gdim, std::size_t degree=1);
 
     /// Open mesh of given cell type, topological and geometrical dimension
     ///
@@ -74,7 +74,10 @@ namespace dolfin
     ///         The topological dimension.
     ///     gdim (std::size_t)
     ///         The geometrical dimension.
-    void open(Mesh& mesh, CellType::Type type, std::size_t tdim, std::size_t gdim);
+    ///     degree (std::size_t)
+    ///         The polynomial degree.
+    void open(Mesh& mesh, CellType::Type type, std::size_t tdim,
+              std::size_t gdim, std::size_t degree=1);
 
     /// Open mesh of given cell type, topological and geometrical dimension
     ///
@@ -87,7 +90,10 @@ namespace dolfin
     ///         The topological dimension.
     ///     gdim (std::size_t)
     ///         The geometrical dimension.
-    void open(Mesh& mesh, std::string type, std::size_t tdim, std::size_t gdim);
+    ///     degree (std::size_t)
+    ///         The polynomial degree.
+    void open(Mesh& mesh, std::string type, std::size_t tdim,
+              std::size_t gdim, std::size_t degree=1);
 
     /// Specify number of vertices (serial version)
     ///
@@ -106,6 +112,14 @@ namespace dolfin
     void init_vertices(std::size_t num_vertices)
     { init_vertices_global(num_vertices, num_vertices); }
 
+    /// Initialise entities in MeshGeometry
+    ///
+    /// Create required Edges and Faces for the current polynomial degree
+    /// in the mesh topology, so that points can be added for them.
+    /// In order to initialise entities, cells must all be added first.
+    ///
+    void init_entities();
+
     /// Specify number of vertices (distributed version)
     ///
     /// *Arguments*
@@ -113,6 +127,8 @@ namespace dolfin
     ///         The number of vertices on this process.
     ///     num_global_vertices (std::size_t)
     ///         The number of vertices in distributed mesh.
+    ///     degree (std::size_t)
+    ///         The polynomial degree of the mesh.
     ///
     /// *Example*
     ///     .. code-block:: c++
@@ -157,7 +173,8 @@ namespace dolfin
     ///         editor.open(mesh, 2, 2);
     ///         editor.init_cells(2, 6);
     ///
-    void init_cells_global(std::size_t num_local_cells, std::size_t num_global_cells);
+    void init_cells_global(std::size_t num_local_cells,
+                           std::size_t num_global_cells);
 
     /// Add vertex v at given point p
     ///
@@ -234,6 +251,10 @@ namespace dolfin
     void add_vertex_global(std::size_t local_index, std::size_t global_index,
                            const std::vector<double>& x);
 
+    /// Add a point in a given entity of dimension entity_dim
+    void add_entity_point(std::size_t entity_dim, std::size_t order,
+                          std::size_t index, const Point& p);
+
     /// Add cell with given vertices (1D)
     ///
     /// *Arguments*
@@ -256,7 +277,8 @@ namespace dolfin
     ///         The second vertex (local index).
     ///     v2 (std::vector<std::size_t>)
     ///         The third vertex (local index).
-    void add_cell(std::size_t c, std::size_t v0, std::size_t v1, std::size_t v2);
+    void add_cell(std::size_t c, std::size_t v0, std::size_t v1,
+                  std::size_t v2);
 
     /// Add cell with given vertices (3D)
     ///
@@ -274,14 +296,27 @@ namespace dolfin
     void add_cell(std::size_t c, std::size_t v0, std::size_t v1,
                   std::size_t v2, std::size_t v3);
 
-    /// Add cell with given vertices
+    /// Add cell with given vertices (non-templated version for Python
+    /// interface)
     ///
     /// *Arguments*
     ///     c (std::size_t)
     ///         The cell (index).
     ///     v (std::vector<std::size_t>)
     ///         The vertex indices (local indices)
-    void add_cell(std::size_t c, const std::vector<std::size_t>& v);
+    void add_cell(std::size_t c, const std::vector<std::size_t>& v)
+    { add_cell(c, c, v); }
+
+    /// Add cell with given vertices
+    ///
+    /// *Arguments*
+    ///     c (std::size_t)
+    ///         The cell (index).
+    ///     v (typename T)
+    ///         The vertex indices (local indices)
+    template<typename T>
+    void add_cell(std::size_t c, const T& v)
+    { add_cell(c, c, v); }
 
     /// Add cell with given vertices
     ///
@@ -292,8 +327,23 @@ namespace dolfin
     ///         The global (user) cell index.
     ///     v (std::vector<std::size_t>)
     ///         The vertex indices (local indices)
+    template<typename T>
     void add_cell(std::size_t local_index, std::size_t global_index,
-                  const std::vector<std::size_t>& v);
+                  const T& v)
+    {
+
+      // dolfin_assert(v.size() == _tdim + 1);
+
+      // Check vertices
+      check_vertices(v);
+
+      // Add cell
+      add_cell_common(local_index, _tdim);
+
+      // Set data
+      _mesh->_topology(_tdim, 0).set(local_index, v);
+      _mesh->_topology.set_global_index(_tdim, local_index, global_index);
+    }
 
     /// Close mesh, finish editing, and order entities locally
     ///
@@ -329,7 +379,20 @@ namespace dolfin
     void clear();
 
     // Check that vertices are in range
-    void check_vertices(const std::vector<std::size_t>& v) const;
+    template <typename T>
+    void check_vertices(const T& v) const
+    {
+      for (std::size_t i = 0; i < v.size(); ++i)
+      {
+        if (_num_vertices > 0 && v[i] >= _num_vertices)
+        {
+          dolfin_error("MeshEditor.cpp",
+                       "add cell using mesh editor",
+                       "Vertex index (%d) out of range [0, %d)", v[i],
+                       _num_vertices);
+        }
+      }
+    }
 
     // The mesh
     Mesh* _mesh;
