@@ -77,16 +77,6 @@ namespace dolfin
     ///         The mesh associated with the collection.
     ///     dim (std::size_t)
     ///         The mesh entity dimension for the mesh value collection.
-    MeshValueCollection(const Mesh& mesh, std::size_t dim);
-
-    /// Create a mesh value collection of entities of given dimension
-    /// on a given mesh (shared_ptr version)
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh associated with the collection.
-    ///     dim (std::size_t)
-    ///         The mesh entity dimension for the mesh value collection.
     MeshValueCollection(std::shared_ptr<const Mesh> mesh, std::size_t dim);
 
     /// Create a mesh value collection from a file.
@@ -99,7 +89,7 @@ namespace dolfin
     ///         The XML file name.
     ///     dim (std::size_t)
     ///         The mesh entity dimension for the mesh value collection.
-    MeshValueCollection(const Mesh& mesh, const std::string filename);
+    MeshValueCollection(std::shared_ptr<const Mesh> mesh, const std::string filename);
 
     /// Destructor
     ~MeshValueCollection() {}
@@ -121,17 +111,7 @@ namespace dolfin
     MeshValueCollection<T>&
       operator=(const MeshValueCollection<T>& mesh_value_collection);
 
-    /// Initialise MeshValueCollection wirth mesh and dimension
-    ///
-    /// *Arguments*
-    ///     mesh (_mesh))
-    ///         The mesh on which the value collection is defined
-    ///     dim (std::size_t)
-    ///         The mesh entity dimension for the mesh value collection.
-    void init(const Mesh& mesh, std::size_t dim);
-
-    /// Initialise MeshValueCollection wirth mesh and dimension
-    /// (shared_ptr version)
+    /// Initialise MeshValueCollection with mesh and dimension
     ///
     /// *Arguments*
     ///     mesh (_mesh))
@@ -277,34 +257,25 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshValueCollection<T>::MeshValueCollection(std::shared_ptr<const Mesh>
-                                                mesh) : _mesh(mesh), _dim(-1)
+  MeshValueCollection<T>::MeshValueCollection(std::shared_ptr<const Mesh>
+                                              mesh) : _mesh(mesh), _dim(-1)
   {
     // Do nothing
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshValueCollection<T>::MeshValueCollection(const Mesh& mesh,
-                                                std::size_t dim)
-    : Variable("m", "unnamed MeshValueCollection"),
-    _mesh(reference_to_no_delete_pointer(mesh)), _dim(dim)
-  {
-    // Do nothing
-  }
-  //---------------------------------------------------------------------------
-  template <typename T>
-    MeshValueCollection<T>::MeshValueCollection(std::shared_ptr<const Mesh>
-                                                mesh, std::size_t dim)
+  MeshValueCollection<T>::MeshValueCollection(std::shared_ptr<const Mesh>
+                                              mesh, std::size_t dim)
     : Variable("m", "unnamed MeshValueCollection"), _mesh(mesh), _dim(dim)
   {
     // Do nothing
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshValueCollection<T>::MeshValueCollection(const MeshFunction<T>&
-                                                mesh_function)
+  MeshValueCollection<T>::MeshValueCollection(const MeshFunction<T>&
+                                              mesh_function)
     : Variable("m", "unnamed MeshValueCollection"), _mesh(mesh_function.mesh()),
-    _dim(mesh_function.dim())
+      _dim(mesh_function.dim())
   {
     dolfin_assert(_mesh);
     const std::size_t D = _mesh->topology().dim();
@@ -316,7 +287,7 @@ namespace dolfin
            ++cell_index)
       {
         const std::pair<std::size_t, std::size_t> key(cell_index, 0);
-        _values.insert(std::make_pair(key, mesh_function[cell_index]));
+        _values.insert({key, mesh_function[cell_index]});
       }
     }
     else
@@ -341,17 +312,17 @@ namespace dolfin
           // Insert into map
           const std::pair<std::size_t, std::size_t> key(cell.index(),
                                                         local_entity);
-          _values.insert(std::make_pair(key, mesh_function[entity_index]));
+          _values.insert({key, mesh_function[entity_index]});
         }
       }
     }
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshValueCollection<T>::MeshValueCollection(const Mesh& mesh,
+    MeshValueCollection<T>::MeshValueCollection(std::shared_ptr<const Mesh> mesh,
                                                 const std::string filename)
     : Variable("m", "unnamed MeshValueCollection"),
-    _mesh(reference_to_no_delete_pointer(mesh)), _dim(-1)
+      _mesh(mesh), _dim(-1)
   {
     File file(filename);
     file >> *this;
@@ -359,14 +330,13 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshValueCollection<T>&
-    MeshValueCollection<T>::operator=(const MeshFunction<T>& mesh_function)
+  MeshValueCollection<T>&
+  MeshValueCollection<T>::operator=(const MeshFunction<T>& mesh_function)
   {
     _mesh = mesh_function.mesh();
     _dim = mesh_function.dim();
 
     dolfin_assert(_mesh);
-
     const std::size_t D = _mesh->topology().dim();
 
     // FIXME: Use iterators
@@ -378,7 +348,7 @@ namespace dolfin
            ++cell_index)
       {
         const std::pair<std::size_t, std::size_t> key(cell_index, 0);
-        _values.insert(std::make_pair(key, mesh_function[cell_index]));
+        _values.insert({key, mesh_function[cell_index]});
       }
     }
     else
@@ -403,7 +373,7 @@ namespace dolfin
           // Insert into map
           const std::pair<std::size_t, std::size_t> key(cell.index(),
                                                         local_entity);
-          _values.insert(std::make_pair(key, mesh_function[entity_index]));
+          _values.insert({key, mesh_function[entity_index]});
         }
       }
     }
@@ -412,9 +382,9 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshValueCollection<T>&
-    MeshValueCollection<T>::operator=(const MeshValueCollection<T>&
-                                      mesh_value_collection)
+  MeshValueCollection<T>&
+  MeshValueCollection<T>::operator=(const MeshValueCollection<T>&
+                                    mesh_value_collection)
   {
     _mesh = mesh_value_collection._mesh;
     _dim = mesh_value_collection.dim();
@@ -424,17 +394,8 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    void MeshValueCollection<T>::init(const Mesh& mesh, std::size_t dim)
-  {
-    mesh.init(dim);
-    _mesh = reference_to_no_delete_pointer(mesh);
-    _dim = dim;
-    _values.clear();
-  }
-  //---------------------------------------------------------------------------
-  template <typename T>
-    void MeshValueCollection<T>::init(std::shared_ptr<const Mesh> mesh,
-                                      std::size_t dim)
+  void MeshValueCollection<T>::init(std::shared_ptr<const Mesh> mesh,
+                                    std::size_t dim)
   {
     mesh->init(dim);
     _mesh = mesh;
@@ -443,7 +404,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    void MeshValueCollection<T>::init(std::size_t dim)
+  void MeshValueCollection<T>::init(std::size_t dim)
   {
     dolfin_assert(_mesh);
     dolfin_assert(_dim < 0);
@@ -451,7 +412,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    std::size_t MeshValueCollection<T>::dim() const
+  std::size_t MeshValueCollection<T>::dim() const
   {
     dolfin_assert(_dim >= 0);
     return _dim;
@@ -470,7 +431,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    std::shared_ptr<const Mesh> MeshValueCollection<T>::mesh() const
+  std::shared_ptr<const Mesh> MeshValueCollection<T>::mesh() const
   {
     dolfin_assert(_mesh);
     return _mesh;
@@ -486,12 +447,12 @@ namespace dolfin
     {
       dolfin_error("MeshValueCollection.h",
                    "set value",
-                   "A mesh has not been associcated with this MeshValueCollection");
+                   "A mesh has not been associated with this MeshValueCollection");
     }
 
     const std::pair<std::size_t, std::size_t> pos(cell_index, local_entity);
     std::pair<typename std::map<std::pair<std::size_t, std::size_t>, T>::iterator, bool>
-      it = _values.insert(std::make_pair(pos, value));
+      it = _values.insert({pos, value});
 
     // If an item with same key already exists the value has not been
     // set and we need to update it
@@ -509,7 +470,7 @@ namespace dolfin
     {
       dolfin_error("MeshValueCollection.h",
                    "set value",
-                   "A mesh has not been associcated with this MeshValueCollection");
+                   "A mesh has not been associated with this MeshValueCollection");
     }
 
     dolfin_assert(_dim >= 0);
@@ -522,7 +483,7 @@ namespace dolfin
       const std::pair<std::size_t, std::size_t> pos(entity_index, 0);
       std::pair<typename std::map<std::pair<std::size_t,
         std::size_t>, T>::iterator, bool> it;
-      it = _values.insert(std::make_pair(pos, value));
+      it = _values.insert({pos, value});
 
       // If an item with same key already exists the value has not been
       // set and we need to update it
@@ -549,7 +510,7 @@ namespace dolfin
     const std::pair<std::size_t, std::size_t> pos(cell.index(), local_entity);
     std::pair<typename std::map<std::pair<std::size_t,
       std::size_t>, T>::iterator, bool> it;
-    it = _values.insert(std::make_pair(pos, value));
+    it = _values.insert({pos, value});
 
     // If an item with same key already exists the value has not been
     // set and we need to update it
@@ -590,7 +551,7 @@ namespace dolfin
   //---------------------------------------------------------------------------
   template <typename T>
   const std::map<std::pair<std::size_t, std::size_t>, T>&
-    MeshValueCollection<T>::values() const
+  MeshValueCollection<T>::values() const
   {
     return _values;
   }
