@@ -60,21 +60,11 @@ namespace dolfin
     void create_entities(boost::multi_array<unsigned int, 2>& e,
                          std::size_t dim, const unsigned int* v) const;
 
-    /// Regular refinement of cell
-    void refine_cell(Cell& cell, MeshEditor& editor,
-                     std::size_t& current_cell) const;
-
-    /// Irregular refinement of cell
-    void refine_cellIrregular(Cell& cell, MeshEditor& editor,
-                              std::size_t& current_cell,
-                              std::size_t refinement_rule,
-                              std::size_t* marked_edges) const;
-
     /// Compute volume of tetrahedron
     double volume(const MeshEntity& tetrahedron) const;
 
-    /// Compute diameter of tetrahedron
-    double diameter(const MeshEntity& tetrahedron) const;
+    /// Compute circumradius of tetrahedron
+    double circumradius(const MeshEntity& tetrahedron) const;
 
     /// Compute squared distance to given point
     double squared_distance(const Cell& cell, const Point& point) const;
@@ -93,7 +83,7 @@ namespace dolfin
     double facet_area(const Cell& cell, std::size_t facet) const;
 
     /// Order entities locally
-    void order(Cell& cell, const std::vector<std::size_t>&
+    void order(Cell& cell, const std::vector<std::int64_t>&
                local_to_global_vertex_indices) const;
 
     /// Check whether given point collides with cell
@@ -110,8 +100,8 @@ namespace dolfin
     std::string description(bool plural) const;
 
     /// Mapping of DOLFIN/UFC vertex ordering to VTK/XDMF ordering
-    std::vector<unsigned int> vtk_mapping() const
-    { return std::vector<unsigned int> {0, 1, 2, 3}; }
+    std::vector<std::int8_t> vtk_mapping() const
+    { return {0, 1, 2, 3}; }
 
   private:
 
