@@ -16,7 +16,7 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // First added:  2013-02-15
-// Last changed: 2014-10-01
+// Last changed: 2016-04-27
 
 #ifndef __POINTINTEGRALSOLVER_H
 #define __POINTINTEGRALSOLVER_H
@@ -47,7 +47,7 @@ namespace dolfin
     /// Constructor
     /// FIXME: Include version where one can pass a Solver and/or
     /// Parameters
-    PointIntegralSolver(std::shared_ptr<MultiStageScheme> scheme);
+    explicit PointIntegralSolver(std::shared_ptr<MultiStageScheme> scheme);
 
     /// Destructor
     ~PointIntegralSolver();
@@ -72,9 +72,8 @@ namespace dolfin
       // Set parameters for NewtonSolver
       Parameters pn("newton_solver");
       pn.add("maximum_iterations", 40);
-      pn.add("recompute_jacobian_for_linear_problems", false);
       pn.add("always_recompute_jacobian", false);
-      pn.add("recompute_jacobian_each_solve", false);
+      pn.add("recompute_jacobian_each_solve", true);
       pn.add("relaxation_parameter", 1., 0., 1.);
       pn.add("relative_tolerance", 1e-10, 1e-20, 2.);
       pn.add("absolute_tolerance", 1e-15, 1e-20, 2.);
@@ -100,39 +99,26 @@ namespace dolfin
 
     // Return number of computations of jacobian
     std::size_t num_jacobian_computations() const
-    {
-      return _num_jacobian_computations;
-    }
+    { return _num_jacobian_computations; }
 
   private:
 
-    // Convergence criteria for simplified Newton solver
-    enum convergence_criteria_t
-    {
-
-      converged,
-      too_slow,
-      exceeds_max_iter,
-      diverge
-
-    };
-
     // In-place LU factorization of jacobian matrix
     void _lu_factorize(std::vector<double>& A);
 
     // Forward backward substitution, assume that mat is already
     // in place LU factorized
     void _forward_backward_subst(const std::vector<double>& A,
-				 const std::vector<double>& b,
-				 std::vector<double>& x) const;
+                                 const std::vector<double>& b,
+                                 std::vector<double>& x) const;
 
     // Compute jacobian using passed UFC form
     void _compute_jacobian(std::vector<double>& jac,
                            const std::vector<double>& u,
-			   unsigned int local_vert, UFC& loc_ufc,
-			   const Cell& cell, const ufc::cell& ufc_cell,
-			   int coefficient_index,
-			   const std::vector<double>& vertex_coordinates);
+                           unsigned int local_vert, UFC& loc_ufc,
+                           const Cell& cell, const ufc::cell& ufc_cell,
+                           int coefficient_index,
+                           const std::vector<double>& coordinate_dofs);
 
     // Compute the norm of a vector
     double _norm(const std::vector<double>& vec) const;
@@ -148,23 +134,23 @@ namespace dolfin
     // Solve an explicit stage
     void _solve_explicit_stage(std::size_t vert_ind, unsigned int stage,
                                const ufc::cell& ufc_cell,
-			       const std::vector<double>& vertex_coordinates);
+                               const std::vector<double>& coordinate_dofs);
 
     // Solve an implicit stage
     void _solve_implicit_stage(std::size_t vert_ind, unsigned int stage,
-			       const Cell& cell, const ufc::cell& ufc_cell,
-			       const std::vector<double>& vertex_coordinates);
+                               const Cell& cell, const ufc::cell& ufc_cell,
+                               const std::vector<double>& coordinate_dofs);
 
     void
       _simplified_newton_solve(std::size_t vert_ind, unsigned int stage,
                                const Cell& cell, const ufc::cell& ufc_cell,
-                               const std::vector<double>& vertex_coordinates);
+                               const std::vector<double>& coordinate_dofs);
 
     // The MultiStageScheme
     std::shared_ptr<MultiStageScheme> _scheme;
 
     // Reference to mesh
-    const Mesh& _mesh;
+    std::shared_ptr<const Mesh> _mesh;
 
     // The dofmap (Same for all stages and forms)
     const GenericDofMap& _dofmap;
@@ -184,14 +170,14 @@ namespace dolfin
 
     // Vertex map between vertices, cells and corresponding local
     // vertex
-    std::vector<std::pair<std::size_t, unsigned int> > _vertex_map;
+    std::vector<std::pair<std::size_t, unsigned int>> _vertex_map;
 
     // Local to global dofs used when solution is fanned out to global
     // vector
     std::vector<dolfin::la_index> _local_to_global_dofs;
 
     // Local stage solutions
-    std::vector<std::vector<double> > _local_stage_solutions;
+    std::vector<std::vector<double>> _local_stage_solutions;
 
     // Local solutions
     std::vector<double> _u0;
@@ -200,19 +186,19 @@ namespace dolfin
     std::vector<double> _dx;
 
     // UFC objects, one for each form
-    std::vector<std::vector<std::shared_ptr<UFC> > > _ufcs;
+    std::vector<std::vector<std::shared_ptr<UFC>>> _ufcs;
 
     // UFC objects for the last form
     std::shared_ptr<UFC> _last_stage_ufc;
 
     // Solution coefficient index in form
-    std::vector<std::vector<int> > _coefficient_index;
+    std::vector<std::vector<int>> _coefficient_index;
 
     // Flag which is set to false once the jacobian has been computed
     std::vector<bool> _recompute_jacobian;
 
     // Jacobians/LU factorized jacobians matrices
-    std::vector<std::vector<double> > _jacobians;
+    std::vector<std::vector<double>> _jacobians;
 
     // Variable used in the estimation of the error of the newton
     // iteration for the first iteration (important for linear
