@@ -18,12 +18,13 @@
 // First added:  2014-06-22
 // Last changed: 2014-07-27
 
-#ifdef ENABLE_PETSC_TAO
+#ifdef HAS_PETSC
 
 #include <map>
 #include <string>
 #include <utility>
 #include <petscsys.h>
+#include <petscversion.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/Timer.h>
 #include <dolfin/la/KrylovSolver.h>
@@ -38,7 +39,7 @@
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-const std::map<std::string, std::pair<std::string, const TaoType> >
+const std::map<std::string, std::pair<std::string, const TaoType>>
 PETScTAOSolver::_methods
 = { {"default", {"Default TAO method (ntl or tron)", TAOTRON}},
     {"tron",    {"Newton Trust Region method", TAOTRON}},
@@ -51,9 +52,9 @@ PETScTAOSolver::_methods
     {"cg",      {"Nonlinear conjugate gradient method", TAOCG}},
     {"nm",      {"Nelder-Mead algorithm", TAONM}} };
 //-----------------------------------------------------------------------------
-std::vector<std::pair<std::string, std::string> > PETScTAOSolver::methods()
+std::vector<std::pair<std::string, std::string>> PETScTAOSolver::methods()
 {
-  std::vector<std::pair<std::string, std::string> > available_methods;
+  std::vector<std::pair<std::string, std::string>> available_methods;
   for (auto it = _methods.begin(); it != _methods.end(); ++it)
     available_methods.push_back(std::make_pair(it->first, it->second.first));
   return available_methods;
@@ -77,15 +78,8 @@ Parameters PETScTAOSolver::default_parameters()
   p.add("linear_solver"          , "default");
   p.add("preconditioner"         , "default");
 
-  std::set<std::string> line_searches;
-  line_searches.insert("default");
-  line_searches.insert("unit");
-  line_searches.insert("more-thuente");
-  line_searches.insert("gpcg");
-  line_searches.insert("armijo");
-  line_searches.insert("owarmijo");
-  line_searches.insert("ipm");
-
+  std::set<std::string> line_searches = {"default", "unit", "more-thuente",
+                                         "gpcg", "armijo", "owarmijo", "ipm"};
   p.add("line_search", "default", line_searches);
 
   p.add(KrylovSolver::default_parameters());
@@ -93,12 +87,28 @@ Parameters PETScTAOSolver::default_parameters()
   return p;
 }
 //-----------------------------------------------------------------------------
-PETScTAOSolver::PETScTAOSolver(const std::string tao_type,
-                               const std::string ksp_type,
-                               const std::string pc_type) : _tao(NULL)
+PETScTAOSolver::PETScTAOSolver(MPI_Comm comm) : _tao(nullptr),
+                                                _has_bounds(false)
 {
   // Create TAO object
-  TaoCreate(PETSC_COMM_WORLD, &_tao);
+  PetscErrorCode ierr = TaoCreate(comm, &_tao);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoCreate");
+
+  // Set parameter values
+  parameters = default_parameters();
+
+  // Update parameters when tao/ksp/pc_types are explictly given
+  update_parameters("default", "default", "default");
+}
+//-----------------------------------------------------------------------------
+PETScTAOSolver::PETScTAOSolver(const std::string tao_type,
+                               const std::string ksp_type,
+                               const std::string pc_type) : _tao(NULL),
+                                                            _has_bounds(false)
+{
+  // Create TAO object
+  PetscErrorCode ierr = TaoCreate(PETSC_COMM_WORLD, &_tao);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoCreate");
 
   // Set parameter values
   parameters = default_parameters();
@@ -114,8 +124,8 @@ PETScTAOSolver::~PETScTAOSolver()
 }
 //-----------------------------------------------------------------------------
 void PETScTAOSolver::update_parameters(const std::string tao_type,
-                                  const std::string ksp_type,
-                                  const std::string pc_type)
+                                       const std::string ksp_type,
+                                       const std::string pc_type)
 {
   // Update parameters when tao/ksp/pc_types are explictly given
   if (tao_type != "default")
@@ -131,6 +141,7 @@ void PETScTAOSolver::update_parameters(const std::string tao_type,
 void PETScTAOSolver::set_tao(const std::string tao_type)
 {
   dolfin_assert(_tao);
+  PetscErrorCode ierr;
 
   // Check that the requested method is known
   if (_methods.count(tao_type) == 0)
@@ -142,36 +153,42 @@ void PETScTAOSolver::set_tao(const std::string tao_type)
 
   // In case of an unconstrained minimisation problem, set the TAO
   // method to TAONTL
-  if (!has_bounds && tao_type == "default")
-    TaoSetType(_tao, TAONTL);
+  if (!_has_bounds && tao_type == "default")
+  {
+    ierr = TaoSetType(_tao, TAONTL);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetType");
+  }
   else
   {
     // Set solver type
     std::map<std::string, std::pair<std::string,
-                                    const TaoType> >::const_iterator it;
+                                    const TaoType>>::const_iterator it;
     it = _methods.find(tao_type);
     dolfin_assert(it != _methods.end());
-    TaoSetType(_tao, it->second.second);
+    ierr = TaoSetType(_tao, it->second.second);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetType");
   }
 }
 //-----------------------------------------------------------------------------
-std::size_t PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
-                                  GenericVector& x,
-                                  const GenericVector& lb,
-                                  const GenericVector& ub)
+std::pair<std::size_t, bool>
+PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
+                      GenericVector& x,
+                      const GenericVector& lb,
+                      const GenericVector& ub)
 {
   // Bound-constrained minimisation problem
-  has_bounds = true;
+  _has_bounds = true;
 
   return solve(optimisation_problem, x.down_cast<PETScVector>(),
                lb.down_cast<PETScVector>(), ub.down_cast<PETScVector>());
 }
 //-----------------------------------------------------------------------------
-std::size_t PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
-                                  GenericVector& x)
+std::pair<std::size_t, bool>
+PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
+                      GenericVector& x)
 {
   // Unconstrained minimisation problem
-  has_bounds = false;
+  _has_bounds = false;
   PETScVector lb, ub;
 
   return solve(optimisation_problem, x.down_cast<PETScVector>(), lb, ub);
@@ -181,7 +198,7 @@ void PETScTAOSolver::init(OptimisationProblem& optimisation_problem,
                           PETScVector& x)
 {
   // Unconstrained minimisation problem
-  has_bounds = false;
+  _has_bounds = false;
   PETScVector lb, ub;
   init(optimisation_problem, x.down_cast<PETScVector>(), lb, ub);
 }
@@ -192,6 +209,7 @@ void PETScTAOSolver::init(OptimisationProblem& optimisation_problem,
                           const PETScVector& ub)
 {
   Timer timer("PETSc TAO solver init");
+  PetscErrorCode ierr;
 
   // Form the optimisation problem object
   _tao_ctx.optimisation_problem = &optimisation_problem;
@@ -210,22 +228,34 @@ void PETScTAOSolver::init(OptimisationProblem& optimisation_problem,
   dolfin_assert(_matH.mat());
 
   // Set initial vector
-  TaoSetInitialVector(_tao, x.vec());
+  ierr = TaoSetInitialVector(_tao, x.vec());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetInitialVector");
 
   // Set the bounds in case of a bound-constrained minimisation problem
-  if (has_bounds)
-    TaoSetVariableBounds(_tao, lb.vec(), ub.vec());
+  if (_has_bounds)
+  {
+    ierr = TaoSetVariableBounds(_tao, lb.vec(), ub.vec());
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetVariableBounds");
+  }
 
   // Set the objective function, gradient and Hessian evaluation routines
-  TaoSetObjectiveAndGradientRoutine(_tao, FormFunctionGradient, &_tao_ctx);
-  TaoSetHessianRoutine(_tao, _matH.mat(), _matH.mat(), FormHessian, &_tao_ctx);
+  ierr = TaoSetObjectiveAndGradientRoutine(_tao, FormFunctionGradient, &_tao_ctx);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetObjectiveAndGradientRoutine");
+  ierr = TaoSetHessianRoutine(_tao, _matH.mat(), _matH.mat(), FormHessian, &_tao_ctx);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetHessianRoutine");
 
   // Clear previous monitors
-  TaoCancelMonitors(_tao);
+  ierr = TaoCancelMonitors(_tao);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoCancelMonitors");
 
   // Set the monitor
   if (parameters["monitor_convergence"])
-    TaoSetMonitor(_tao, TaoDefaultMonitor, PETSC_NULL, PETSC_NULL);
+  {
+    ierr = TaoSetMonitor(_tao, TaoDefaultMonitor,
+                         PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)_tao)),
+                         NULL);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetMonitor");
+  }
 
   // Check for any TAO command line options
   std::string prefix = std::string(parameters["options_prefix"]);
@@ -236,27 +266,33 @@ void PETScTAOSolver::init(OptimisationProblem& optimisation_problem,
     char lastchar = *prefix.rbegin();
     if (lastchar != '_')
       prefix += "_";
-    TaoSetOptionsPrefix(_tao, prefix.c_str());
+    ierr = TaoSetOptionsPrefix(_tao, prefix.c_str());
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetOptionsPrefix");
   }
-  TaoSetFromOptions(_tao);
+  ierr = TaoSetFromOptions(_tao);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetFromOptions");
 
   // Set the convergence test
-  TaoSetConvergenceTest(_tao, TaoConvergenceTest, this);
+  ierr = TaoSetConvergenceTest(_tao, TaoConvergenceTest, this);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetConvergenceTest");
 }
 //-----------------------------------------------------------------------------
-std::size_t PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
-                                  PETScVector& x,
-                                  const PETScVector& lb,
-                                  const PETScVector& ub)
+std::pair<std::size_t, bool>
+PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
+                      PETScVector& x,
+                      const PETScVector& lb,
+                      const PETScVector& ub)
 {
   Timer timer("PETSc TAO solver execution");
+  PetscErrorCode ierr;
 
   // Initialise the TAO solver
   PETScVector x_copy(x);
   init(optimisation_problem, x_copy, lb, ub);
 
   // Solve
-  TaoSolve(_tao);
+  ierr = TaoSolve(_tao);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSolve");
 
   // Get the solution vector
   x.zero();
@@ -267,15 +303,20 @@ std::size_t PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
 
   // Print the report on convergence and methods used
   if (parameters["report"])
-    TaoView(_tao, PETSC_VIEWER_STDOUT_WORLD);
+  {
+    ierr = TaoView(_tao, PETSC_VIEWER_STDOUT_WORLD);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoView");
+  }
 
   // Check for convergence
   TaoConvergedReason reason;
-  TaoGetConvergedReason(_tao, &reason);
+  ierr = TaoGetConvergedReason(_tao, &reason);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoGetConvergedReason");
 
   // Get the number of iterations
-  PetscInt num_iterations = 0;
-  TaoGetMaximumIterations(_tao, &num_iterations);
+  PetscInt its = 0;
+  ierr = TaoGetIterationNumber(_tao, &its);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoGetIterationNumber");
 
   // Report number of iterations
   if (reason >= 0)
@@ -285,11 +326,12 @@ std::size_t PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
     bool error_on_nonconvergence = parameters["error_on_nonconvergence"];
     if (error_on_nonconvergence)
     {
-      TaoView(_tao, PETSC_VIEWER_STDOUT_WORLD);
+      ierr = TaoView(_tao, PETSC_VIEWER_STDOUT_WORLD);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "TaoView");
       dolfin_error("PETScTAOSolver.cpp",
                    "solve nonlinear optimisation problem",
                    "Solution failed to converge in %i iterations (TAO reason %d)",
-                   num_iterations, reason);
+                   its, reason);
     }
     else
     {
@@ -298,7 +340,7 @@ std::size_t PETScTAOSolver::solve(OptimisationProblem& optimisation_problem,
     }
   }
 
-  return num_iterations;
+  return std::make_pair(its, reason > 0);
 }
 //-----------------------------------------------------------------------------
 PetscErrorCode PETScTAOSolver::FormFunctionGradient(Tao tao, Vec x,
@@ -346,13 +388,21 @@ PetscErrorCode PETScTAOSolver::TaoConvergenceTest(Tao tao, void *ctx)
   PetscInt its;
   PetscReal f, gnorm, cnorm, xdiff;
   TaoConvergedReason reason;
-  TaoGetSolutionStatus(tao, &its, &f, &gnorm, &cnorm, &xdiff, &reason);
+  PetscErrorCode ierr;
+  ierr = TaoGetSolutionStatus(tao, &its, &f, &gnorm, &cnorm, &xdiff, &reason);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoGetSolutionStatus");
 
   // We enforce Tao to do at least one iteration
   if (its < 1)
-    TaoSetConvergedReason(tao, TAO_CONTINUE_ITERATING);
+  {
+    ierr = TaoSetConvergedReason(tao, TAO_CONTINUE_ITERATING);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetConvergedReason");
+  }
   else
-    TaoDefaultConvergenceTest(tao, &ctx);
+  {
+    ierr = TaoDefaultConvergenceTest(tao, &ctx);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoDefaultConvergenceTest");
+  }
 
   return 0;
 }
@@ -360,36 +410,50 @@ PetscErrorCode PETScTAOSolver::TaoConvergenceTest(Tao tao, void *ctx)
 void PETScTAOSolver::set_tao_options()
 {
   dolfin_assert(_tao);
+  PetscErrorCode ierr;
 
   // Set the TAO solver
   set_tao(parameters["method"]);
 
   // Set tolerances
-  TaoSetTolerances(_tao, parameters["function_absolute_tol"],
-                         parameters["function_relative_tol"],
-                         parameters["gradient_absolute_tol"],
-                         parameters["gradient_relative_tol"],
-                         parameters["gradient_t_tol"]);
+  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
+  ierr = TaoSetTolerances(_tao, parameters["function_absolute_tol"],
+                                parameters["function_relative_tol"],
+                                parameters["gradient_absolute_tol"],
+                                parameters["gradient_relative_tol"],
+                                parameters["gradient_t_tol"]);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetTolerances");
+  #else
+  ierr = TaoSetTolerances(_tao, parameters["gradient_absolute_tol"],
+                                parameters["gradient_relative_tol"],
+                                parameters["gradient_t_tol"]);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetTolerances");
+  #endif
 
   // Set TAO solver maximum iterations
   int maxits = parameters["maximum_iterations"];
-  TaoSetMaximumIterations(_tao, maxits);
+  ierr = TaoSetMaximumIterations(_tao, maxits);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetMaximumIterations");
 
   // Set TAO line search
   const std::string line_search_type = parameters["line_search"];
   if (line_search_type != "default")
   {
     TaoLineSearch linesearch;
-    TaoGetLineSearch(_tao, &linesearch);
-    TaoLineSearchSetType(linesearch, line_search_type.c_str());
+    ierr = TaoGetLineSearch(_tao, &linesearch);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoGetLineSearch");
+    ierr = TaoLineSearchSetType(linesearch, line_search_type.c_str());
+    if (ierr != 0) petsc_error(ierr, __FILE__, "TaoLineSearchSetType");
   }
 }
 //-----------------------------------------------------------------------------
 void PETScTAOSolver::set_ksp_options()
 {
   dolfin_assert(_tao);
+  PetscErrorCode ierr;
   KSP ksp;
-  TaoGetKSP(_tao, &ksp);
+  ierr = TaoGetKSP(_tao, &ksp);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "TaoGetKSP");
   const std::string ksp_type  = parameters["linear_solver"];
   const std::string pc_type = parameters["preconditioner"];
 
@@ -397,7 +461,8 @@ void PETScTAOSolver::set_ksp_options()
   if (ksp)
   {
     PC pc;
-    KSPGetPC(ksp, &pc);
+    ierr = KSPGetPC(ksp, &pc);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGetPC");
 
     if (ksp_type == "default")
     {
@@ -410,14 +475,16 @@ void PETScTAOSolver::set_ksp_options()
       std::map<std::string, const KSPType>::const_iterator ksp_pair
         = PETScKrylovSolver::_methods.find(ksp_type);
       dolfin_assert(ksp_pair != PETScKrylovSolver::_methods.end());
-      KSPSetType(ksp, ksp_pair->second);
+      ierr = KSPSetType(ksp, ksp_pair->second);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetType");
 
       if (pc_type != "default")
       {
         std::map<std::string, const PCType>::const_iterator pc_pair
           = PETScPreconditioner::_methods.find(pc_type);
         dolfin_assert(pc_pair != PETScPreconditioner::_methods.end());
-        PCSetType(pc, pc_pair->second);
+        ierr = PCSetType(pc, pc_pair->second);
+        if (ierr != 0) petsc_error(ierr, __FILE__, "PCSetType");
       }
     }
     else if (ksp_type == "lu" || PETScLUSolver::_methods.count(ksp_type) != 0)
@@ -444,7 +511,7 @@ void PETScTAOSolver::set_ksp_options()
           #else
           lu_method = "petsc";
           warning("Using PETSc native LU solver. Consider configuring PETSc with an efficient LU solver (e.g. UMFPACK, MUMPS).");
-#endif
+          #endif
         }
         else
         {
@@ -456,17 +523,20 @@ void PETScTAOSolver::set_ksp_options()
           lu_method = "mumps";
           #else
           dolfin_error("PETScTAOSolver.cpp",
-                     "solve linear system using PETSc LU solver",
-                     "No suitable solver for parallel LU found. Consider configuring PETSc with MUMPS or SuperLU_dist");
+                       "solve linear system using PETSc LU solver",
+                       "No suitable solver for parallel LU found. Consider configuring PETSc with MUMPS or SuperLU_dist");
           #endif
         }
       }
-      KSPSetType(ksp, KSPPREONLY);
-      PCSetType(pc, PCLU);
+      ierr = KSPSetType(ksp, KSPPREONLY);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetType");
+      ierr = PCSetType(pc, PCLU);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "PCSetType");
       std::map<std::string, const MatSolverPackage>::const_iterator lu_pair
         = PETScLUSolver::_methods.find(lu_method);
       dolfin_assert(lu_pair != PETScLUSolver::_methods.end());
-      PCFactorSetMatSolverPackage(pc, lu_pair->second);
+      ierr = PCFactorSetMatSolverPackage(pc, lu_pair->second);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "PCFactorSetMatSolverPackage");
     }
     else     // Unknown KSP method
     {
@@ -478,30 +548,52 @@ void PETScTAOSolver::set_ksp_options()
     // In any case, set the KSP options specified by the user
     Parameters krylov_parameters = parameters("krylov_solver");
 
-    // GMRES restart parameter
-    const int gmres_restart = krylov_parameters("gmres")["restart"];
-    KSPGMRESSetRestart(ksp, gmres_restart);
-
     // Non-zero initial guess
-    const bool nonzero_guess = krylov_parameters["nonzero_initial_guess"];
+    bool nonzero_guess = false;
+    if (krylov_parameters["nonzero_initial_guess"].is_set())
+      nonzero_guess = krylov_parameters["nonzero_initial_guess"];
     if (nonzero_guess)
-      KSPSetInitialGuessNonzero(ksp, PETSC_TRUE);
+    {
+      ierr = KSPSetInitialGuessNonzero(ksp, PETSC_TRUE);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetInitialGuessNonzero");
+    }
     else
-      KSPSetInitialGuessNonzero(ksp, PETSC_FALSE);
+    {
+      ierr = KSPSetInitialGuessNonzero(ksp, PETSC_FALSE);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetInitialGuessNonzero");
+    }
 
     // KSP monitor
-    if (krylov_parameters["monitor_convergence"])
-      KSPMonitorSet(ksp, KSPMonitorTrueResidualNorm, 0, 0);
-
-    // Get integer tolerances (to take care of casting to PetscInt)
-    const int max_iter = krylov_parameters["maximum_iterations"];
+    if (krylov_parameters["monitor_convergence"].is_set())
+    {
+      if (krylov_parameters["monitor_convergence"])
+      {
+        #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
+        ierr = TaoSetMonitor(_tao, TaoDefaultMonitor,
+                             PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)_tao)),
+                             NULL);
+        if (ierr != 0) petsc_error(ierr, __FILE__, "TaoSetMonitor");
+        #else
+        PetscViewer viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ksp));
+        PetscViewerFormat format = PETSC_VIEWER_DEFAULT;
+        PetscViewerAndFormat *vf;
+        ierr = PetscViewerAndFormatCreate(viewer,format,&vf);
+        ierr = KSPMonitorSet(ksp,
+                         (PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*)) KSPMonitorTrueResidualNorm,
+                         vf,
+                         (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
+        if (ierr != 0) petsc_error(ierr, __FILE__, "KSPMonitorSet");
+        #endif
+      }
+    }
 
     // Set tolerances
-    KSPSetTolerances(ksp,
-                     krylov_parameters["relative_tolerance"],
-                     krylov_parameters["absolute_tolerance"],
-                     krylov_parameters["divergence_limit"],
-                     max_iter);
+    const double rtol = krylov_parameters["relative_tolerance"].is_set() ? (double)krylov_parameters["relative_tolerance"] : PETSC_DEFAULT;
+    const double atol = krylov_parameters["absolute_tolerance"].is_set() ? (double)krylov_parameters["absolute_tolerance"] : PETSC_DEFAULT;
+    const double dtol = krylov_parameters["divergence_limit"].is_set() ? (double)krylov_parameters["divergence_limit"] : PETSC_DEFAULT;
+    const int max_it  = krylov_parameters["maximum_iterations"].is_set() ? (int)krylov_parameters["maximum_iterations"] : PETSC_DEFAULT;
+    ierr = KSPSetTolerances(ksp, rtol, atol, dtol, max_it);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetTolerances");
   }
   else
   {
@@ -509,4 +601,5 @@ void PETScTAOSolver::set_ksp_options()
   }
 }
 //-----------------------------------------------------------------------------
+
 #endif
