@@ -47,8 +47,10 @@ const std::map<std::string, const KSPType> TAOLinearBoundSolver::_ksp_methods
     {"minres",     KSPMINRES},
     {"tfqmr",      KSPTFQMR},
     {"richardson", KSPRICHARDSON},
+    #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 7 && PETSC_VERSION_RELEASE == 1
     {"nash",       KSPNASH},
     {"stcg",       KSPSTCG},
+    #endif
     {"bicgstab",   KSPBCGS} };
 //-----------------------------------------------------------------------------
 // Mapping from method string to description
@@ -312,6 +314,7 @@ void TAOLinearBoundSolver::set_ksp(std::string ksp_type)
     if (ierr != 0) petsc_error(ierr, __FILE__, "TaoGetKSP");
     if (ksp)
     {
+      dolfin_assert(_ksp_methods.find(ksp_type) != _ksp_methods.end());
       ierr = KSPSetType(ksp, _ksp_methods.find(ksp_type)->second);
       if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetType");
     }
