@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """Main module for DOLFIN"""
 
 # Special module to handle import issues
@@ -13,9 +14,9 @@ PROGRESS = 16
 del set_handler, set_level
 
 # Import names from the compiled cpp modules
-import cpp
+from . import cpp
 from dolfin.cpp import *
-from dolfin.cpp import __version__, __swigversion__
+from dolfin.cpp import __version__, __swigversion__, __pythonversion__
 
 # Reset dl open flags
 import sys
@@ -36,12 +37,12 @@ except:
     supported_elements_for_plotting = []
 
 # Imports from pure Python modules
-from dolfin.functions import *
 from dolfin.common import *
+from dolfin.functions import *
 from dolfin.compilemodules import *
 from dolfin.fem import *
 from dolfin.mesh import *
 from dolfin.multistage import *
 
 # Handling of deprecated interfaces
-from deprecation import *
+from .deprecation import *
