@@ -2,7 +2,7 @@
 """This module provides functionality to compile PyDOLFIN compatible
 extension modules."""
 
-# Copyright (C) 2009 Johan Hake
+# Copyright (C) 2009-2016 Johan Hake
 #
 # This file is part of DOLFIN.
 #
@@ -20,23 +20,21 @@ extension modules."""
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Johannes Ring, 2011
-# Modifie by Mikael Mortensen, 2014
-#
-# First added:  2009-08-15
-# Last changed: 2014-02-14
+# Modified by Mikael Mortensen, 2014
+# Modified by Martin Sandve Alnæs, 2016
 
 import sys
-import os
 import re
-import numpy
-import instant
 import hashlib
-import types
+
+import instant
+from ufl.utils.sorting import canonicalize_metadata
 from dolfin_utils.cppparser import *
 
 # Import PyDOLFIN
 import dolfin
 import dolfin.cpp as cpp
+from dolfin.cpp import parameters
 
 from dolfin.compilemodules.jit import mpi_jit_decorator
 from dolfin.compilemodules.swigimportinfo import *
@@ -50,6 +48,9 @@ __all__ = ["compile_extension_module",
 # version)
 _interface_version = 3
 
+# Add max_signature_length to parameter system (don't need this in C++)
+cpp.parameters.add("max_signature_length", 0)
+
 # A list of supported math builtins
 _cpp_math_builtins = [
     # <cmath> functions: from http://www.cplusplus.com/reference/cmath/
@@ -72,8 +73,8 @@ _math_dolfin = [
     # functions from dolfin::math:
     "sqr", "ipow", "rand", "near", "DOLFIN_EPS", "DOLFIN_PI", "pi"]
 
-_all_math = list(set(_math_builtins).difference(_cpp_math_builtins)) + \
-            _math_dolfin + _cpp_math_builtins
+_all_math = (list(set(_math_builtins).difference(_cpp_math_builtins))
+            + _math_dolfin + _cpp_math_builtins)
 
 math_header = """
 // cmath functions
@@ -126,6 +127,7 @@ import_array();
 %%shared_ptr(ufc::function)
 %%shared_ptr(ufc::dofmap)
 %%shared_ptr(ufc::finite_element)
+%%shared_ptr(ufc::coordinate_mapping)
 %%shared_ptr(ufc::function)
 %%shared_ptr(ufc::form)
 %%ignore ufc::integral;
@@ -164,6 +166,7 @@ del tmp, get_swigversion
 re_symbol = re.compile(r"(?:[a-zA-Z_]*::)*[\w\.]+")
 re_number = re.compile("[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?")
 
+
 def expression_to_code_fragments(expr, arguments, generic_function_members=None):
     "A help function which extract a dict with code snippets from an expression"
 
@@ -227,6 +230,7 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
     # Return the code fragments
     return fragments, variables
 
+
 def expect_list_of(argtype, arg, argname):
     if arg is None:
         return []
@@ -240,6 +244,7 @@ def expect_list_of(argtype, arg, argname):
                      "Provide a 'tuple' or 'list' with '%s', for the "\
                      "'%s' argument" % (argtype.__name__, argname))
 
+
 def expect_arg(argtype, arg, argname):
     # Check the type of the argument
     if isinstance(arg, argtype):
@@ -250,6 +255,7 @@ def expect_arg(argtype, arg, argname):
                      "Provide a '%s', for the '%s' argument" % \
                      (argtype.__name__, argname))
 
+
 def check_swig_version(compiled_module):
 
     # Check swig version of compiled module
@@ -261,6 +267,7 @@ def check_swig_version(compiled_module):
                          "version: '%s' != '%s' " % \
                          (cpp.__swigversion__, compiled_module.swigversion))
 
+
 @mpi_jit_decorator
 def compile_extension_module(code, module_name="",
                              additional_declarations="",
@@ -289,7 +296,7 @@ def compile_extension_module(code, module_name="",
         additional_system_headers :
             System headers needed to compile the generated can be included
             using this argument. The headers are passed using a list of 'str'
-            
+
         mpi_comm
             An mpi communicator used to determine what processors
             should compile the module. By default mpi_world_comm is
@@ -352,15 +359,20 @@ def compile_extension_module(code, module_name="",
         code = "\n".join(codelines)
 
     # Create unique module name for this application run
-    if module_name is "":
-        module_name = "dolfin_compile_code_%s" % \
-                      hashlib.sha1((repr(code) +
+    if not module_name:
+        module_signature = hashlib.sha1((repr(code) +
                                    dolfin.__version__ +
                                    str(_interface_version) +
+                                   sys.version +
                                    additional_declarations +
                                    str(additional_system_headers) +
-                                   repr(instant_kwargs)).encode("utf-8")
-                                   ).hexdigest()
+                                   repr(canonicalize_metadata(instant_kwargs))
+                                   ).encode("utf-8")
+                                ).hexdigest()
+        max_signature_length = parameters["max_signature_length"]
+        if max_signature_length:
+            module_signature = module_signature[:max_signature_length]
+        module_name = "dolfin_%s" % module_signature
 
     # Extract dolfin dependencies and class names
     used_types, declared_types = parse_and_extract_type_info(code)
@@ -406,7 +418,7 @@ def compile_extension_module(code, module_name="",
         for f in ["dolfin/common/constants.h", "dolfin/common/types.h"]:
             if f not in dependencies["common"]:
                 dependencies["common"].append(f)
-        
+
     # Sort the dependencies
     dependencies = sort_submodule_dependencies(dependencies, submodule_info)
 
@@ -414,7 +426,7 @@ def compile_extension_module(code, module_name="",
                   build_swig_import_info(dependencies, submodule_info, "dolfin.cpp.")
 
     # Extract header info
-    dolfin_system_headers = [header for header in file_dependencies \
+    dolfin_system_headers = [header for header in file_dependencies
                              if not "pre.i" in header]
 
     # Check the handed import files
@@ -423,52 +435,41 @@ def compile_extension_module(code, module_name="",
     # Check cache
     compiled_module = instant.import_module(module_name)
 
-    if compiled_module:
-        # Check that the swig version of the compiled module is the same as
-        # dolfin was compiled with
-        check_swig_version(compiled_module)
-        return compiled_module
+    if not compiled_module:
+        # Configure instant before building
+        sys.stdout.flush()
+        dolfin.info("Calling DOLFIN just-in-time (JIT) compiler, this may take some time.")
 
-    sys.stdout.flush()
-    dolfin.info("Calling DOLFIN just-in-time (JIT) compiler, this may take some time.")
+        # Add various system headers
+        instant_kwargs["system_headers"] = ["cmath", "iostream","complex",
+                                            "stdexcept","numpy/arrayobject.h",
+                                            "memory",
+                                            "dolfin/common/types.h",
+                                            "dolfin/math/basic.h",
+                                        ] + instant_kwargs.get("system_headers", [])
+        instant_kwargs["system_headers"] += dolfin_system_headers
+        instant_kwargs["system_headers"] += additional_system_headers
 
-    # Configure instant and add additional system headers
+        # Add cmake packages
+        instant_kwargs["cmake_packages"]  = ["DOLFIN"] + \
+                                            instant_kwargs.get("cmake_packages", [])
+        instant_kwargs["signature"] = module_name
 
-    # Add dolfin system headers
-    instant_kwargs["system_headers"] = ["cmath", "iostream","complex",
-                                        "stdexcept","numpy/arrayobject.h",
-                                        "memory",
-                                        "dolfin/common/types.h",
-                                        "dolfin/math/basic.h"] + \
-                                        instant_kwargs.get("system_headers", [])
-    instant_kwargs["system_headers"] += dolfin_system_headers
+        # Add additional swig declarations
+        dolfin_import_statement = "\n".join(import_lines)
+        shared_ptr_declarations = extract_shared_ptr_declaration(
+            declared_types, used_dolfin_types, shared_ptr_classes)
+        additional_declarations = _additional_declarations % {
+            "additional_declarations": additional_declarations,
+            "shared_ptr_declarations": shared_ptr_declarations,
+            "dolfin_import_statement": dolfin_import_statement,
+            }
 
-    # Add user specified system headers
-    instant_kwargs["system_headers"] += additional_system_headers
-
-    # Add cmake packages
-    instant_kwargs["cmake_packages"]  = ["DOLFIN"] + \
-                                        instant_kwargs.get("cmake_packages", [])
-    instant_kwargs["signature"] = module_name
-
-    declaration_strs = {"additional_declarations":""}
-    declaration_strs["dolfin_import_statement"] = \
-                    "\n".join(import_lines)
-
-    # Add any provided additional declarations
-    if additional_declarations is not None:
-        declaration_strs["additional_declarations"] += additional_declarations
-
-    # Add any shared_ptr declarations
-    declaration_strs["shared_ptr_declarations"] = \
-        extract_shared_ptr_declaration(declared_types, used_dolfin_types, \
-                                       shared_ptr_classes)
-
-    # Compile extension module with instant
-    compiled_module = instant.build_module(\
-        code              = code,
-        additional_declarations = _additional_declarations % declaration_strs,
-        **instant_kwargs)
+        # Compile extension module with instant
+        compiled_module = instant.build_module(
+            code=code,
+            additional_declarations=additional_declarations,
+            **instant_kwargs)
 
     sys.stdout.flush()
 
@@ -478,6 +479,7 @@ def compile_extension_module(code, module_name="",
 
     return compiled_module
 
+
 def extract_shared_ptr_declaration(declared_types, used_dolfin_types, shared_ptr_classes):
     " Extract any declaration for shared_ptr"
     # Check if there are any classes that is derived from any of the
@@ -494,7 +496,6 @@ def extract_shared_ptr_declaration(declared_types, used_dolfin_types, shared_ptr
     used_shared_ptr_types.extend(derived for derived, bases in list(declared_types.items())
                                  if any(base in shared_ptr_classes for base in bases))
 
-    shared_ptr_declarations = "\n".join([shared_ptr_format % derived \
+    shared_ptr_declarations = "\n".join([shared_ptr_format % derived
                                          for derived in used_shared_ptr_types])
-
     return shared_ptr_declarations
