@@ -2,7 +2,7 @@
 """This module defines a jit function that wraps the jit function for
 the chosen form compiler."""
 
-# Copyright (C) 2008-2014 Johan Hake
+# Copyright (C) 2008-2016 Johan Hake
 #
 # This file is part of DOLFIN.
 #
@@ -21,20 +21,23 @@ the chosen form compiler."""
 #
 # Modified by Anders Logg, 2008-2009.
 # Modified by Garth N. Wells, 2011.
-# Modified by Martin Sandve Alnæs, 2014.
+# Modified by Martin Sandve Alnæs, 2014-2016.
 
 from __future__ import print_function
 
 __all__ = ["jit"]
 
 from functools import wraps
+import sys
+import traceback
+
+import six
+import ufl
 
 # Import SWIG-generated extension module (DOLFIN C++)
 import dolfin.cpp as cpp
 from dolfin.cpp import MPI, info, parameters, warning, info
 
-import ufl
-import six
 
 def mpi_jit_decorator(local_jit, *args, **kwargs):
     """A decorator for jit compilation
@@ -119,9 +122,16 @@ def jit(ufl_object, form_compiler_parameters=None, mpi_comm=None):
     p = ffc.default_parameters()
     p.update(parameters["form_compiler"])
     p.update(form_compiler_parameters or {})
-    
+
     # Execute!
-    result = ffc.jit(ufl_object, parameters=p)
+    try:
+        result = ffc.jit(ufl_object, parameters=p)
+    except Exception as e:
+        tb_text = ''.join(traceback.format_exception(*sys.exc_info()))
+        cpp.dolfin_error("jit.py",
+                         "perform just-in-time compilation of form",
+                         "ffc.jit failed with message:\n%s" % (tb_text,))
+
     if isinstance(ufl_object, ufl.Form):
         compiled_form, module, prefix = result
         compiled_form = cpp.make_ufc_form(compiled_form)
@@ -131,3 +141,7 @@ def jit(ufl_object, form_compiler_parameters=None, mpi_comm=None):
         ufc_element = cpp.make_ufc_finite_element(ufc_element)
         ufc_dofmap = cpp.make_ufc_dofmap(ufc_dofmap)
         return ufc_element, ufc_dofmap
+    elif isinstance(ufl_object, ufl.Mesh):
+        ufc_coordinate_mapping = result
+        ufc_coordinate_mapping = cpp.make_ufc_coordinate_mapping(ufc_coordinate_mapping)
+        return ufc_coordinate_mapping
