@@ -21,17 +21,4 @@
 
 import dolfin.cpp as cpp
 
-#--- Deprecation of down_cast ---
-
-def _down_cast(self):
-    cpp.warning("foo.down_cast() is deprecated, please use as_backend_type(foo).")
-    return cpp.as_backend_type(self)
-
-cpp.Matrix.down_cast = _down_cast
-cpp.Vector.down_cast = _down_cast
-
-def down_cast(foo):
-    cpp.warning("down_cast(foo) is deprecated, please use as_backend_type(foo).")
-    return cpp.as_backend_type(foo)
-
 #--- Deprecation of ... etc. ---
