@@ -26,11 +26,7 @@ into a cpp.Form (dolfin::Form)."""
 # Import SWIG-generated extension module (DOLFIN C++)
 import dolfin.cpp as cpp
 
-# Import ufl and ufc
 import ufl
-import ufc
-
-# Import JIT compiler
 from dolfin.compilemodules.jit import jit
 
 from dolfin.cpp import deprecation
@@ -89,9 +85,13 @@ class Form(cpp.Form):
                              "Expecting to find a Mesh in the form.")
 
         # Jit the module, this will take some time...
-        self._compiled_form, module, prefix \
-            = jit(form, form_compiler_parameters,
-                  mpi_comm=mesh.mpi_comm())
+        jit_result = jit(form, form_compiler_parameters,
+                         mpi_comm=mesh.mpi_comm())
+        if jit_result is None:
+            cpp.dolfin_error("form.py",
+                             "creating dolfin.Form",
+                             "jit failure.")
+        self._compiled_form, module, prefix = jit_result
 
         # Extract function spaces of form arguments
         if function_spaces is None:
@@ -125,21 +125,25 @@ class Form(cpp.Form):
             self.coefficients.append(original_coefficients[j])
 
         # Type checking coefficients
-        if not all(isinstance(c, cpp.GenericFunction)
+        if not all( isinstance(c, (cpp.GenericFunction, cpp.MultiMeshFunction))
                    for c in self.coefficients):
+            # Developer note:
+            # The form accepts a MultiMeshFunction but does not set the
+            # correct coefficients. This is done in assemble_multimesh
+            # at the moment
             coefficient_error = "Error while extracting coefficients. "
             raise TypeError(coefficient_error +
                             "Either provide a dict of cpp.GenericFunctions, " +
                             "or use Function to define your form.")
 
         for i in range(self.num_coefficients()):
-            self.set_coefficient(i, self.coefficients[i])
+            if isinstance(self.coefficients[i], cpp.GenericFunction):
+                self.set_coefficient(i, self.coefficients[i])
 
         # Attach mesh (because function spaces and coefficients may be
         # empty lists)
         if not function_spaces:
             self.set_mesh(mesh)
-
         # Type checking subdomain data
         # Delete None entries
         for k in list(self.subdomains.keys()):
