@@ -21,6 +21,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 
+from six import string_types
 import ufl
 from ufl import inner, grad, div, curl, dx, FiniteElement, VectorElement, Coefficient
 from math import sqrt
@@ -99,7 +100,7 @@ def norm(v, norm_type="L2", mesh=None):
                          "expected a GenericVector or GenericFunction")
 
     # Check arguments
-    if not isinstance(norm_type, str):
+    if not isinstance(norm_type, string_types):
         cpp.dolfin_error("norms.py",
                          "compute norm",
                          "Norm type must be a string, not " +
