@@ -30,6 +30,7 @@ import dolfin.cpp as cpp
 
 # Local imports
 from dolfin.functions.function import *
+from dolfin.functions.multimeshfunction import *
 from dolfin.functions.expression import *
 from dolfin.functions.functionspace import *
 from dolfin.fem.assembling import *
@@ -84,6 +85,7 @@ def project(v, V=None, bcs=None, mesh=None,
         # Create function space based on Expression element if trying
         # to project an Expression
         if isinstance(v, Expression):
+            # FIXME: Add handling of cpp.MultiMesh
             if mesh is not None and isinstance(mesh, cpp.Mesh):
                 V = FunctionSpace(mesh, v.ufl_element())
             else:
@@ -95,12 +97,35 @@ def project(v, V=None, bcs=None, mesh=None,
             V = _extract_function_space(v, mesh)
 
     # Check arguments
-    if not isinstance(V, FunctionSpace):
+    if not isinstance(V, (FunctionSpace, cpp.MultiMeshFunctionSpace)):
         cpp.dolfin_error("projection.py",
                          "compute projection",
-                         "Illegal function space for projection, not a FunctionSpace: " +
+                         "Illegal function space for projection, not a FunctionSpace or MultiMeshFunctionSpace: " +
                          str(v))
 
+    # Projection into a MultiMeshFunctionSpace
+    if isinstance(V, cpp.MultiMeshFunctionSpace):
+
+        # Create the measuresum of uncut and cut-cells
+        dX = ufl.dx() + ufl.dC()
+
+        # Define variational problem for projection
+        w = TestFunction(V)
+        Pv = TrialFunction(V)
+        a = ufl.inner(w, Pv)*dX
+        L = ufl.inner(w, v)*dX
+
+        # Assemble linear system
+        A = assemble_multimesh(a, form_compiler_parameters=form_compiler_parameters)
+        b = assemble_multimesh(L, form_compiler_parameters=form_compiler_parameters)
+
+        # Solve linear system for projection
+        if function is None:
+            function = MultiMeshFunction(V)
+        cpp.la_solve(A, function.vector(), b, solver_type, preconditioner_type)
+
+        return function
+
     # Ensure we have a mesh and attach to measure
     if mesh is None:
         mesh = V.mesh()
