@@ -68,6 +68,7 @@ __all__ = ["Expression"]
 # Python imports
 import types
 from six import add_metaclass # Requires newer six version than some buildbots have
+from six import string_types
 
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
 import ufl
@@ -206,7 +207,7 @@ def create_python_derived_expression_class(class_name, user_bases, user_dict):
     """
 
     # Check args
-    assert isinstance(class_name, str)
+    assert isinstance(class_name, string_types)
     assert isinstance(user_bases, list)
     assert isinstance(user_dict, dict)
 
@@ -314,7 +315,7 @@ class ExpressionMetaClass(type):
     def __new__(mcs, class_name, bases, dict_):
         """Returns a new Expression class"""
 
-        assert isinstance(class_name, str), "Expecting a 'str'"
+        assert isinstance(class_name, string_types), "Expecting a 'str'"
         assert isinstance(bases, tuple), "Expecting a 'tuple'"
         assert isinstance(dict_, dict), "Expecting a 'dict'"
 
@@ -617,9 +618,9 @@ class Expression(object):
         .. code-block:: python
 
             class MyExpression1(Expression):
-                def __init__(self, mesh, domain):
-                    self._mesh = mesh
-                    self._domain = domain
+                def __init__(self, **kwargs):
+                    self._mesh = kwargs["mesh"]
+                    self._domain = kwargs["domain"]
                 def eval(self, values, x):
                     ...
 
@@ -771,7 +772,7 @@ class Expression(object):
             raise TypeError("expected at least 1 argument")
 
         # Test for ufl restriction
-        if len(args) == 1 and isinstance(args[0], str):
+        if len(args) == 1 and isinstance(args[0], string_types):
             if args[0] in ('+', '-'):
                 return ufl.Coefficient.__call__(self, *args)
 
@@ -851,18 +852,15 @@ def _check_cppcode(cppcode):
     "Check that cppcode makes sense"
 
     # Check that we get a string expression or nested expression
-    if not isinstance(cppcode, (str, tuple, list)):
+    if not isinstance(cppcode, string_types + (tuple, list)):
         raise TypeError("Please provide a 'str', 'tuple' or 'list' for "\
                         "the 'cppcode' argument.")
 
 
-def _auto_select_element_from_shape(shape, degree=None, cell=None):
+def _auto_select_element_from_shape(shape, degree, cell=None):
     "Automatically select an appropriate element from cppcode."
-
-    if degree is None:
-        cpp.deprecation("Automatic determination of degree for Expressions",
-                        "2016.1",
-                        "Specify the polynomial degree for interpolation of Expression, or provide a FiniteElement")
+    if not isinstance(degree, int):
+        raise TypeError("Expression needs an integer argument 'degree' if no 'element' is provided.")
 
     # Default element, change to quadrature when working
     if degree == 0:
