@@ -21,18 +21,11 @@ elements.
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-#
-# Modified by Anders Logg 2008, 2015
-# Modified by Martin Sandve Alnæs 2008-2015
-# Modified by Kent-Andre Mardal 2009
-# Modified by Marie E. Rognes 2012
-# Modified by Jan Blechta 2015
 
-__all__ = ["FunctionSpace", "EnrichedFunctionSpace", "MixedFunctionSpace",
-           "VectorFunctionSpace", "TensorFunctionSpace",
-           "FunctionSpaceFromCpp", "FunctionSpaceBase",
+__all__ = ["FunctionSpace", "VectorFunctionSpace", "TensorFunctionSpace",
            "MultiMeshFunctionSpace"]
 
+from six import string_types
 import numpy
 
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
@@ -238,7 +231,7 @@ class FunctionSpace(ufl.FunctionSpace, cpp.FunctionSpace):
     def _init_convenience(self, mesh, family, degree, form_degree=None,
                           constrained_domain=None, restriction=None):
         # Check arguments
-        if not isinstance(family, str):
+        if not isinstance(family, string_types):
             cpp.dolfin_error("functionspace.py",
                              "create function space",
                              "Illegal argument for finite element family, not a string: " + str(family))
@@ -296,24 +289,10 @@ class FunctionSpace(ufl.FunctionSpace, cpp.FunctionSpace):
         "Return the UFL cell."
         return self.ufl_domain().ufl_cell()
 
-    def cell(self):
-        "Return the UFL cell."
-        cpp.deprecation("'FunctionSpace.cell()'", "2016.1",
-                        "Use 'FunctionSpace.ufl_cell()' instead.")
-        return self.ufl_cell()
-
     def dolfin_element(self):
         "Return the DOLFIN element."
         return self.element()
 
-    def __add__(self, other):
-        "Create enriched function space."
-        return EnrichedFunctionSpace((self, other))
-
-    def __mul__(self, other):
-        "Create mixed function space."
-        return MixedFunctionSpace((self, other))
-
     def __str__(self):
         "Pretty-print."
         return "<Function space of dimension %d (%s)>" % \
@@ -405,107 +384,6 @@ class FunctionSpace(ufl.FunctionSpace, cpp.FunctionSpace):
             return V
 
 
-def FunctionSpaceFromCpp(cppV):
-    cpp.deprecation("'FunctionSpaceFromCpp'", "2016.1",
-                    "Use 'FunctionSpace(cppV)' instead.")
-    return FunctionSpace(cppV)
-
-
-def FunctionSpaceBase(mesh, element, constrained_domain=None):
-    cpp.deprecation("'FunctionSpaceBase'", "2016.1",
-                    "Use 'FunctionSpace(...)' instead.")
-    return FunctionSpace(mesh, element, constrained_domain=None)
-
-
-def EnrichedFunctionSpace(spaces):
-    """
-    Create enriched finite element function space.
-
-    *Arguments*
-        spaces
-            a list (or tuple) of :py:class:`FunctionSpaces
-            <dolfin.functions.functionspace.FunctionSpace>`.
-
-    *Usage*
-        The function space may be created by
-
-        .. code-block:: python
-
-            V = EnrichedFunctionSpace(spaces)
-
-    """
-    cpp.deprecation("'EnrichedFunctionSpace'", "2016.1",
-                    "Use 'FunctionSpace(mesh, EnrichedElement(...))'.")
-
-    # Check arguments
-    if not len(spaces) > 0:
-        cpp.dolfin_error("functionspace.py",
-                         "create enriched function space",
-                         "Need at least one subspace")
-    if not all(isinstance(V, FunctionSpace) for V in spaces):
-        cpp.dolfin_error("functionspace.py",
-                         "create enriched function space",
-                         "Invalid subspaces: " + str(spaces))
-
-    # Get common mesh and constrained_domain, must all be the same
-    mesh, constrained_domain = _get_common_mesh_and_constrained_domain(spaces)
-
-    # Create element
-    element = ufl.EnrichedElement(*[V.ufl_element() for V in spaces])
-
-    return FunctionSpace(mesh, element,
-                         constrained_domain=constrained_domain)
-
-
-def MixedFunctionSpace(spaces):
-    """
-    Create mixed finite element function space.
-
-    *Arguments*
-        spaces
-            a list (or tuple) of :py:class:`FunctionSpaces
-            <dolfin.functions.functionspace.FunctionSpace>`.
-
-    *Examples of usage*
-        The function space may be created by
-
-        .. code-block:: python
-
-            V = MixedFunctionSpace(spaces)
-
-        ``spaces`` may consist of multiple occurances of the same space:
-
-        .. code-block:: python
-
-            P1  = FunctionSpace(mesh, "CG", 1)
-            P2v = VectorFunctionSpace(mesh, "Lagrange", 2)
-
-            ME  = MixedFunctionSpace([P2v, P1, P1, P1])
-
-    """
-    cpp.deprecation("'MixedFunctionSpace'", "2016.1",
-                    "Use 'FunctionSpace(mesh, MixedElement(...))'.")
-
-    # Check arguments
-    if not len(spaces) > 0:
-        cpp.dolfin_error("functionspace.py",
-                         "create mixed function space",
-                         "Need at least one subspace")
-    if not all(isinstance(V, FunctionSpace) for V in spaces):
-        cpp.dolfin_error("functionspace.py",
-                         "create mixed function space",
-                         "Invalid subspaces: " + str(spaces))
-
-    # Get common mesh and constrained_domain, must all be the same
-    mesh, constrained_domain = _get_common_mesh_and_constrained_domain(spaces)
-
-    # Create UFL element
-    element = ufl.MixedElement(*[V.ufl_element() for V in spaces])
-
-    return FunctionSpace(mesh, element,
-                         constrained_domain=constrained_domain)
-
-
 def VectorFunctionSpace(mesh, family, degree, dim=None, form_degree=None,
                         constrained_domain=None, restriction=None):
     """Create vector-valued finite element function space.
@@ -616,10 +494,6 @@ def MultiMeshFunctionSpace(multimesh, family, degree=None):
 
             element = FiniteElement("Lagrange", triangle, 1)
             V = MultiMeshFunctionSpace(mesh, element)
-
-    Note: Multimesh function spaces does currently not support
-    function space algebra. For this reason, mixed function spaces
-    and the like must be created via a finite element.
     """
 
     # Check arguments
