@@ -21,7 +21,7 @@
 #
 # Modified by Johan Hake 2008-2009
 # Modified by Garth N. Wells 2010
-# Modified by Martin S. Alnaes 2013-2014
+# Modified by Martin Sandve Alnæs 2013-2014
 
 __all__ = ["MeshCoordinates", "FacetArea", "FacetNormal", "CellSize", "CellVolume",
            'SpatialCoordinate', 'CellNormal', 'Circumradius', 'MinFacetEdgeLength', 'MaxFacetEdgeLength']
@@ -35,21 +35,33 @@ from dolfin.functions.expression import Expression
 
 def _mesh2domain(mesh):
     "Deprecation mechanism for symbolic geometry."
-    if isinstance(mesh, ufl.Cell):
+
+    # Handle MultiMesh
+    if isinstance(mesh, cpp.MultiMesh):
+        mesh = mesh.part(0)
+
+    if isinstance(mesh, ufl.cell.AbstractCell):
         raise TypeError("Cannot construct geometry from a Cell. Pass the mesh instead, for example use FacetNormal(mesh) instead of FacetNormal(triangle) or triangle.n")
-    return ufl.as_domain(mesh)
+    return mesh.ufl_domain()
 
 class MeshCoordinates(Expression, ufl.Coefficient, cpp.MeshCoordinates):
     def __init__(self, mesh):
         "Create function that evaluates to the mesh coordinates at each vertex."
+        # Initialize C++ part
         cpp.MeshCoordinates.__init__(self, mesh)
-        self._ufl_element = ufl.VectorElement("Lagrange", mesh.ufl_domain(), 1)
-        ufl.Coefficient.__init__(self, self._ufl_element, count=self.id())
+
+        # Initialize UFL part
+        ufl_element = mesh.ufl_domain().ufl_coordinate_element()
+        if ufl_element.family() != "Lagrange" or ufl_element.degree() != 1:
+            cpp.dolfin_error("specialfunctions.py",
+                             "initialize MeshCoordinates",
+                             "dolfin::MeshCoordinates only supports affine meshes")
+        ufl_function_space = ufl.FunctionSpace(mesh.ufl_domain(), ufl_element)
+        ufl.Coefficient.__init__(self, ufl_function_space, count=self.id())
 
 MeshCoordinates.__doc__ = cpp.MeshCoordinates.__doc__
 
 class FacetArea(Expression, ufl.Coefficient, cpp.FacetArea):
-
     def __init__(self, mesh):
         """
         Create function that evaluates to the facet area/length on each facet.
@@ -66,9 +78,19 @@ class FacetArea(Expression, ufl.Coefficient, cpp.FacetArea):
                 fa = FacetArea(mesh)
 
         """
+
+        # Handle MultiMesh
+        if isinstance(mesh, cpp.MultiMesh):
+            mesh = mesh.part(0)
+
+        # Initialize C++ part
         cpp.FacetArea.__init__(self, mesh)
-        self._ufl_element = ufl.FiniteElement("Discontinuous Lagrange", mesh.ufl_domain(), 0)
-        ufl.Coefficient.__init__(self, self._ufl_element, count=self.id())
+
+        # Initialize UFL part
+        # NB! This is defined as a piecewise constant function for each cell, not for each facet!
+        ufl_element = ufl.FiniteElement("Discontinuous Lagrange", mesh.ufl_cell(), 0)
+        ufl_function_space = ufl.FunctionSpace(mesh.ufl_domain(), ufl_element)
+        ufl.Coefficient.__init__(self, ufl_function_space, count=self.id())
 
 FacetArea.__doc__ = cpp.FacetArea.__doc__
 
@@ -90,6 +112,7 @@ def FacetNormal(mesh):
             n = FacetNormal(mesh)
 
     """
+
     return ufl.FacetNormal(_mesh2domain(mesh))
 
 # Simple definition of CellSize via UFL
@@ -109,6 +132,7 @@ def CellSize(mesh):
             h = CellSize(mesh)
 
     """
+
     return 2.0*ufl.Circumradius(_mesh2domain(mesh))
 
 # Simple definition of CellVolume via UFL
@@ -128,6 +152,7 @@ def CellVolume(mesh):
             vol = CellVolume(mesh)
 
     """
+
     return ufl.CellVolume(_mesh2domain(mesh))
 
 # Simple definition of SpatialCoordinate via UFL
@@ -147,6 +172,7 @@ def SpatialCoordinate(mesh):
             vol = SpatialCoordinate(mesh)
 
     """
+
     return ufl.SpatialCoordinate(_mesh2domain(mesh))
 
 # Simple definition of CellNormal via UFL
@@ -166,6 +192,7 @@ def CellNormal(mesh):
             vol = CellNormal(mesh)
 
     """
+
     return ufl.CellNormal(_mesh2domain(mesh))
 
 # Simple definition of Circumradius via UFL
@@ -185,6 +212,7 @@ def Circumradius(mesh):
             vol = Circumradius(mesh)
 
     """
+
     return ufl.Circumradius(_mesh2domain(mesh))
 
 # Simple definition of MinFacetEdgeLength via UFL
@@ -204,6 +232,7 @@ def MinFacetEdgeLength(mesh):
             vol = MinFacetEdgeLength(mesh)
 
     """
+
     return ufl.MinFacetEdgeLength(_mesh2domain(mesh))
 
 # Simple definition of MaxFacetEdgeLength via UFL
@@ -223,4 +252,5 @@ def MaxFacetEdgeLength(mesh):
             vol = MaxFacetEdgeLength(mesh)
 
     """
+
     return ufl.MaxFacetEdgeLength(_mesh2domain(mesh))
