@@ -23,9 +23,10 @@ from __future__ import print_function
 import pytest
 from dolfin import *
 import os
-from dolfin_utils.test import skip_in_parallel, fixture, tempdir
+from dolfin_utils.test import skip_in_parallel, skip_if_not_HDF5, fixture, tempdir
 
 @skip_in_parallel
+@skip_if_not_HDF5
 def test_retrieve_all_connectivities(tempdir):
     _test_retrieve(tempdir, "test_retrieve_all_connectivities")
 
@@ -72,6 +73,7 @@ def _test_retrieve(tempdir, basename):
 
 
 @skip_in_parallel
+@skip_if_not_HDF5
 def test_subdirectory(tempdir):
     "Test that retrieve/store works with nonexisting subdirectory"
     filename = os.path.join(tempdir, "test_subdirectory")
