@@ -3,6 +3,7 @@ import ufl
 from dolfin import *
 from dolfin_utils.test import skip_in_parallel
 
+
 @skip_in_parallel
 def test_interior_facet_integral_sides():
     n = 1
@@ -11,14 +12,15 @@ def test_interior_facet_integral_sides():
     subdomain = AutoSubDomain(lambda x, on_boundary: x[0] > x[1]-DOLFIN_EPS)
 
     V = FunctionSpace(mesh, "DG", 0)
-    f = interpolate(Expression("x[0]"), V)
+    f = interpolate(Expression("x[0]", degree=1), V)
 
     # Define forms picking value of f from + and - sides
     scale = 1.0/ufl.FacetArea(mesh)
     Mp = f('+')*scale*dS(domain=mesh)
     Mm = f('-')*scale*dS(domain=mesh)
 
-    # Hack to attach cell markers to dS integral... Need to find a UFL solution to this.
+    # Hack to attach cell markers to dS integral... Need to find a UFL
+    # solution to this.
     Mh = Constant(0)*dx(99, domain=mesh, subdomain_data=markers)
     Mp = Mp + Mh
     Mm = Mm + Mh
