@@ -20,11 +20,14 @@
 import pytest
 import os
 from dolfin import *
-from dolfin_utils.test import skip_if_not_HDF5, skip_in_parallel, fixture, tempdir
+from dolfin_utils.test import skip_in_parallel, fixture, tempdir
 
 # Currently supported XDMF file encoding
 encodings = (XDMFFile.Encoding_HDF5, XDMFFile.Encoding_ASCII)
 
+# Data types supported in templating
+data_types = (('int', int), ('size_t', int), ('double', float), ('bool', bool))
+
 
 def invalid_config(encoding):
     return (not has_hdf5() and encoding == XDMFFile.Encoding_HDF5) \
@@ -44,7 +47,7 @@ def test_save_and_load_1d_mesh(tempdir, encoding):
 
     mesh2 = Mesh()
     file = XDMFFile(mpi_comm_world(), filename)
-    file.read(mesh2, False)
+    file.read(mesh2)
     assert mesh.size_global(0) == mesh2.size_global(0)
     dim = mesh.topology().dim()
     assert mesh.size_global(dim) == mesh2.size_global(dim)
@@ -63,7 +66,7 @@ def test_save_and_load_2d_mesh(tempdir, encoding):
 
     mesh2 = Mesh()
     file = XDMFFile(mpi_comm_world(), filename)
-    file.read(mesh2, False)
+    file.read(mesh2)
     assert mesh.size_global(0) == mesh2.size_global(0)
     dim = mesh.topology().dim()
     assert mesh.size_global(dim) == mesh2.size_global(dim)
@@ -82,7 +85,7 @@ def test_save_and_load_3d_mesh(tempdir, encoding):
 
     mesh2 = Mesh()
     file = XDMFFile(mpi_comm_world(), filename)
-    file.read(mesh2, False)
+    file.read(mesh2)
     assert mesh.size_global(0) == mesh2.size_global(0)
     dim = mesh.topology().dim()
     assert mesh.size_global(dim) == mesh2.size_global(dim)
@@ -231,103 +234,193 @@ def test_save_1d_mesh(tempdir, encoding):
 
 
 @pytest.mark.parametrize("encoding", encodings)
-def test_save_2D_cell_function(tempdir, encoding):
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_2D_cell_function(tempdir, encoding, data_type):
     if invalid_config(encoding):
         pytest.xfail("XDMF unsupported in current configuration")
-    filename = os.path.join(tempdir, "mf_2D.xdmf")
+
+    dtype_str, dtype = data_type
+
+    filename = os.path.join(tempdir, "mf_2D_%s.xdmf" % dtype_str)
     mesh = UnitSquareMesh(32, 32)
-    mf = CellFunction("size_t", mesh)
+    mf = CellFunction(dtype_str, mesh)
+    mf.rename("cells", "cells")
     for cell in cells(mesh):
-        mf[cell] = cell.index()
+        mf[cell] = dtype(cell.index())
 
     file = XDMFFile(mesh.mpi_comm(), filename)
     file.write(mf, encoding)
     del file
 
+    mf_in = CellFunction(dtype_str, mesh)
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+    xdmf.read(mf_in, "cells")
+    del xdmf
+
+    diff = 0
+    for cell in cells(mesh):
+        diff += (mf_in[cell] - mf[cell])
+    assert diff == 0
+
 
 @pytest.mark.parametrize("encoding", encodings)
-def test_save_3D_cell_function(tempdir, encoding):
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_3D_cell_function(tempdir, encoding, data_type):
     if invalid_config(encoding):
         pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
     mesh = UnitCubeMesh(8, 8, 8)
-    mf = CellFunction("size_t", mesh)
+    mf = CellFunction(dtype_str, mesh)
+    mf.rename("cells", "cells")
     for cell in cells(mesh):
-        mf[cell] = cell.index()
-    filename = os.path.join(tempdir, "mf_3D.xdmf")
+        mf[cell] = dtype(cell.index())
+    filename = os.path.join(tempdir, "mf_3D_%s.xdmf" % dtype_str)
 
     file = XDMFFile(mesh.mpi_comm(), filename)
     file.write(mf, encoding)
     del file
 
+    mf_in = CellFunction(dtype_str, mesh)
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+    xdmf.read(mf_in, "cells")
+    del xdmf
+
+    diff = 0
+    for cell in cells(mesh):
+        diff += (mf_in[cell] - mf[cell])
+    assert diff == 0
 
 @pytest.mark.parametrize("encoding", encodings)
-def test_save_2D_facet_function(tempdir, encoding):
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_2D_facet_function(tempdir, encoding, data_type):
     if invalid_config(encoding):
         pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
     mesh = UnitSquareMesh(32, 32)
-    mf = FacetFunction("size_t", mesh)
+    mf = FacetFunction(dtype_str, mesh)
+    mf.rename("facets", "facets")
+
+    if (MPI.size(mesh.mpi_comm()) == 1):
+        for facet in facets(mesh):
+            mf[facet] = dtype(facet.index())
+    else:
+        for facet in facets(mesh):
+            mf[facet] = dtype(facet.global_index())
+    filename = os.path.join(tempdir, "mf_facet_2D_%s.xdmf" % dtype_str)
+
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+    xdmf.write(mf, encoding)
+    del xdmf
+
+    mf_in = FacetFunction(dtype_str, mesh)
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+    xdmf.read(mf_in, "facets")
+    del xdmf
+
+    diff = 0
     for facet in facets(mesh):
-        mf[facet] = facet.index()
-    filename = os.path.join(tempdir, "mf_facet_2D.xdmf")
+        diff += (mf_in[facet] - mf[facet])
+    assert diff == 0
+
+@pytest.mark.parametrize("encoding", encodings)
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_3D_facet_function(tempdir, encoding, data_type):
+    if invalid_config(encoding):
+        pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
+    mesh = UnitCubeMesh(8, 8, 8)
+    mf = FacetFunction(dtype_str, mesh)
+    mf.rename("facets", "facets")
+
+    if (MPI.size(mesh.mpi_comm()) == 1):
+        for facet in facets(mesh):
+            mf[facet] = dtype(facet.index())
+    else:
+        for facet in facets(mesh):
+            mf[facet] = dtype(facet.global_index())
+    filename = os.path.join(tempdir, "mf_facet_3D_%s.xdmf" % dtype_str)
 
     file = XDMFFile(mesh.mpi_comm(), filename)
     file.write(mf, encoding)
     del file
 
+    mf_in = FacetFunction(dtype_str, mesh)
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+    xdmf.read(mf_in, "facets")
+    del xdmf
 
-@pytest.mark.parametrize("encoding", encodings)
-def test_save_3D_facet_function(tempdir, encoding):
-    if invalid_config(encoding):
-        pytest.xfail("XDMF unsupported in current configuration")
-    mesh = UnitCubeMesh(8, 8, 8)
-    mf = FacetFunction("size_t", mesh)
+    diff = 0
     for facet in facets(mesh):
-        mf[facet] = facet.index()
-    filename = os.path.join(tempdir, "mf_facet_3D.xdmf")
-
-    file = XDMFFile(mesh.mpi_comm(), filename)
-    file.write(mf, encoding)
-    del file
-
+        diff += (mf_in[facet] - mf[facet])
+    assert diff == 0
 
 @pytest.mark.parametrize("encoding", encodings)
-def test_save_3D_edge_function(tempdir, encoding):
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_3D_edge_function(tempdir, encoding, data_type):
     if invalid_config(encoding):
         pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
     mesh = UnitCubeMesh(8, 8, 8)
-    mf = EdgeFunction("size_t", mesh)
+    mf = EdgeFunction(dtype_str, mesh)
+    mf.rename("edges", "edges")
     for edge in edges(mesh):
-        mf[edge] = edge.index()
+        mf[edge] = dtype(edge.index())
 
-    file = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "mf_edge_3D.xdmf"))
+    filename = os.path.join(tempdir, "mf_edge_3D_%s.xdmf" % dtype_str)
+    file = XDMFFile(mesh.mpi_comm(), filename)
     file.write(mf, encoding)
     del file
 
-
 @pytest.mark.parametrize("encoding", encodings)
-def test_save_2D_vertex_function(tempdir, encoding):
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_2D_vertex_function(tempdir, encoding, data_type):
     if invalid_config(encoding):
         pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
     mesh = UnitSquareMesh(32, 32)
-    mf = VertexFunction("size_t", mesh)
+    mf = VertexFunction(dtype_str, mesh)
+    mf.rename("vertices", "vertices")
     for vertex in vertices(mesh):
-        mf[vertex] = vertex.index()
-    filename = os.path.join(tempdir, "mf_vertex_2D.xdmf")
+        mf[vertex] = dtype(vertex.global_index())
+    filename = os.path.join(tempdir, "mf_vertex_2D_%s.xdmf" % dtype_str)
 
     file = XDMFFile(mesh.mpi_comm(), filename)
     file.write(mf, encoding)
     del file
 
+    mf_in = VertexFunction(dtype_str, mesh)
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+    xdmf.read(mf_in, "vertices")
+    del xdmf
+
+    diff = 0
+    for v in vertices(mesh):
+        diff += (mf_in[v] - mf[v])
+    assert diff == 0
 
 @pytest.mark.parametrize("encoding", encodings)
-def test_save_3D_vertex_function(tempdir, encoding):
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_3D_vertex_function(tempdir, encoding, data_type):
     if invalid_config(encoding):
         pytest.xfail("XDMF unsupported in current configuration")
-    filename = os.path.join(tempdir, "mf_vertex_3D.xdmf")
+
+    dtype_str, dtype = data_type
+
+    filename = os.path.join(tempdir, "mf_vertex_3D_%s.xdmf" % dtype_str)
     mesh = UnitCubeMesh(8, 8, 8)
-    mf = VertexFunction("size_t", mesh)
+    mf = VertexFunction(dtype_str, mesh)
     for vertex in vertices(mesh):
-        mf[vertex] = vertex.index()
+        mf[vertex] = dtype(vertex.index())
 
     file = XDMFFile(mesh.mpi_comm(), filename)
     file.write(mf, encoding)
@@ -347,16 +440,10 @@ def test_save_points_2D(tempdir, encoding):
     vals = numpy.array(values)
 
     file = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "points_2D.xdmf"))
-    if encoding == XDMFFile.Encoding_ASCII: # Not implemented
-        with pytest.raises(RuntimeError):
-            file.write(points, encoding)
-    else:
-        file.write(points, encoding)
-    del file
+    file.write(points, encoding)
 
     file = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir,
                                                   "points_values_2D.xdmf"))
-
     file.write(points, vals, encoding)
     del file
 
@@ -374,44 +461,197 @@ def test_save_points_3D(tempdir, encoding):
     vals = numpy.array(values)
 
     file = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "points_3D.xdmf"))
-    if encoding == XDMFFile.Encoding_ASCII: # Not implemented
-        with pytest.raises(RuntimeError):
-            file.write(points, encoding)
-    else:
-        file.write(points, encoding)
-    del file
+    file.write(points, encoding)
 
     file = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "points_values_3D.xdmf"))
-
     file.write(points, vals, encoding)
     del file
 
-
 @pytest.mark.parametrize("encoding", encodings)
-def test_save_mesh_value_collection(tempdir, encoding):
+@pytest.mark.parametrize("data_type", data_types)
+def test_save_mesh_value_collection(tempdir, encoding, data_type):
     if invalid_config(encoding):
         pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
     mesh = UnitCubeMesh(4, 4, 4)
     tdim = mesh.topology().dim()
 
-    meshfn = CellFunction("size_t", mesh, 0)
+    meshfn = CellFunction(dtype_str, mesh, 0)
     meshfn.rename("volume_marker", "Volume Markers")
     for c in cells(mesh):
         if c.midpoint().y() > 0.1:
-            meshfn[c] = 1
+            meshfn[c] = dtype(1)
         if c.midpoint().y() > 0.9:
-            meshfn[c] = 2
+            meshfn[c] = dtype(2)
 
     for mvc_dim in range(0, tdim + 1):
-        mvc = MeshValueCollection("size_t", mesh, 2)
-        mvc.rename("dim_%d_marker" % mvc_dim, "BC")
+        mvc = MeshValueCollection(dtype_str, mesh, mvc_dim)
+        tag = "dim_%d_marker" % mvc_dim
+        mvc.rename(tag, "BC")
         mesh.init(mvc_dim, tdim)
         for e in cpp.entities(mesh, mvc_dim):
             if (e.midpoint().x() > 0.5):
-                mvc.set_value(e.index(), 1)
+                mvc.set_value(e.index(), dtype(1))
 
-        xdmf = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "mvc_%d.xdmf"
-                                                      % mvc_dim))
-        xdmf.parameters['time_series'] = False
+        filename = os.path.join(tempdir, "mvc_%d.xdmf" % mvc_dim)
+
+        xdmf = XDMFFile(mesh.mpi_comm(), filename)
         xdmf.write(meshfn, encoding)
         xdmf.write(mvc, encoding)
+
+        del xdmf
+
+        xdmf = XDMFFile(mesh.mpi_comm(), filename)
+        mvc = MeshValueCollection(dtype_str, mesh)
+        xdmf.read(mvc, tag)
+
+@skip_in_parallel
+@pytest.mark.parametrize("encoding", encodings)
+def test_quadratic_mesh(tempdir, encoding):
+    if invalid_config(encoding):
+        pytest.xfail("XDMF unsupported in current configuration")
+    mesh = UnitDiscMesh(mpi_comm_world(), 2, 2, 2)
+    Q = FunctionSpace(mesh, "CG", 1)
+    u = Function(Q)
+    u.interpolate(Constant(1.0))
+    xdmf = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "quadratic1.xdmf"))
+    xdmf.write(u)
+
+    Q = FunctionSpace(mesh, "CG", 2)
+    u = Function(Q)
+    u.interpolate(Constant(1.0))
+    xdmf = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "quadratic2.xdmf"))
+    xdmf.write(u)
+
+    xdmf = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "qmesh.xdmf"))
+    xdmf.write(mesh)
+    c0 = mesh.coordinates()
+
+    mesh = Mesh()
+    xdmf = XDMFFile(mesh.mpi_comm(), os.path.join(tempdir, "qmesh.xdmf"))
+    xdmf.read(mesh)
+    c1 = mesh.coordinates()
+
+    assert (c0 - c1).sum() == 0.0
+
+@pytest.mark.parametrize("encoding", encodings)
+@pytest.mark.parametrize("data_type", data_types)
+def test_append_and_load_mesh_functions(tempdir, encoding, data_type):
+    if invalid_config(encoding):
+        pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
+    meshes = [UnitSquareMesh(32, 32), UnitCubeMesh(8, 8, 8)]
+
+    for mesh in meshes:
+        dim = mesh.topology().dim()
+
+        vf = VertexFunction(dtype_str, mesh)
+        vf.rename("vertices", "vertices")
+        ff = FacetFunction(dtype_str, mesh)
+        ff.rename("facets", "facets")
+        cf = CellFunction(dtype_str, mesh)
+        cf.rename("cells", "cells")
+
+        if (MPI.size(mesh.mpi_comm()) == 1):
+            for vertex in vertices(mesh):
+                vf[vertex] = dtype(vertex.index())
+            for facet in facets(mesh):
+                ff[facet] = dtype(facet.index())
+            for cell in cells(mesh):
+                cf[cell] = dtype(cell.index())
+        else:
+            for vertex in vertices(mesh):
+                vf[vertex] = dtype(vertex.global_index())
+            for facet in facets(mesh):
+                ff[facet] = dtype(facet.global_index())
+            for cell in cells(mesh):
+                cf[cell] = dtype(cell.global_index())
+
+        filename = os.path.join(tempdir, "appended_mf_%dD.xdmf" % dim)
+
+        xdmf = XDMFFile(mesh.mpi_comm(), filename)
+        xdmf.write(mesh)
+        xdmf.write(vf, encoding)
+        xdmf.write(ff, encoding)
+        xdmf.write(cf, encoding)
+        del xdmf
+
+        xdmf = XDMFFile(mesh.mpi_comm(), filename)
+
+        vf_in = VertexFunction(dtype_str, mesh)
+        xdmf.read(vf_in, "vertices")
+        ff_in = FacetFunction(dtype_str, mesh)
+        xdmf.read(ff_in, "facets")
+        cf_in = CellFunction(dtype_str, mesh)
+        xdmf.read(cf_in, "cells")
+        del xdmf
+
+        diff = 0
+        for vertex in vertices(mesh):
+            diff += (vf_in[vertex] - vf[vertex])
+        for facet in facets(mesh):
+            diff += (ff_in[facet] - ff[facet])
+        for cell in cells(mesh):
+            diff += (cf_in[cell] - cf[cell])
+        assert diff == 0
+
+
+@pytest.mark.parametrize("encoding", encodings)
+@pytest.mark.parametrize("data_type", data_types)
+def test_append_and_load_mesh_value_collections(tempdir, encoding, data_type):
+    if invalid_config(encoding):
+        pytest.xfail("XDMF unsupported in current configuration")
+
+    dtype_str, dtype = data_type
+
+    mesh = UnitCubeMesh(8, 8, 8)
+    mesh.init()
+    for d in range(mesh.geometry().dim() + 1):
+        mesh.init_global(d)
+
+    mvc_v = MeshValueCollection(dtype_str, mesh, 0)
+    mvc_v.rename("vertices", "vertices")
+    mvc_e = MeshValueCollection(dtype_str, mesh, 1)
+    mvc_e.rename("edges", "edges")
+    mvc_f = MeshValueCollection(dtype_str, mesh, 2)
+    mvc_f.rename("facets", "facets")
+    mvc_c = MeshValueCollection(dtype_str, mesh, 3)
+    mvc_c.rename("cells", "cells")
+
+    mvcs = [mvc_v, mvc_e, mvc_f, mvc_c]
+
+    filename = os.path.join(tempdir, "appended_mvcs.xdmf")
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+
+    for mvc in mvcs:
+        for ent in entities(mesh, mvc.dim()):
+            assert(mvc.set_value(ent.index(), dtype(ent.global_index())))
+        xdmf.write(mvc)
+
+    del xdmf
+
+    mvc_v_in = MeshValueCollection(dtype_str, mesh, 0)
+    mvc_e_in = MeshValueCollection(dtype_str, mesh, 1)
+    mvc_f_in = MeshValueCollection(dtype_str, mesh, 2)
+    mvc_c_in = MeshValueCollection(dtype_str, mesh, 3)
+
+    xdmf = XDMFFile(mesh.mpi_comm(), filename)
+    xdmf.read(mvc_v_in, "vertices")
+    xdmf.read(mvc_e_in, "edges")
+    xdmf.read(mvc_f_in, "facets")
+    xdmf.read(mvc_c_in, "cells")
+
+    mvcs_in = [mvc_v_in, mvc_e_in, mvc_f_in, mvc_c_in]
+
+    for (mvc, mvc_in) in zip(mvcs, mvcs_in):
+        mf = MeshFunction(dtype_str, mesh, mvc)
+        mf_in = MeshFunction(dtype_str, mesh, mvc_in)
+
+        diff = 0
+        for ent in entities(mesh, mf.dim()):
+            diff += (mf_in[ent] - mf[ent])
+        assert(diff == 0)
